@@ -57,6 +57,7 @@ struct ShaderObject;
 struct ShaderBinaryData;
 struct CreatePipelineLayout;
 struct CreateBuffer;
+struct CmdBindDescriptorBuffers;
 }  // namespace chassis
 
 namespace vvl {
@@ -119,6 +120,8 @@ class Instance : public Logger {
         PreCallRecordCreateDevice(physicalDevice, pCreateInfo, pAllocator, pDevice, record_obj);
     }
     void CopyDispatchState() { instance = dispatch_instance_->instance; }
+    // Because this object was created before dispatch_instance_ can query for supported extensions, we must copy them afterwards here
+    void CopyExtensions() { extensions = dispatch_instance_->extensions; }
     VkInstance VkHandle() const { return instance; }
 
 #if defined(DEBUG_CAPTURE_KEYBOARD)
@@ -439,6 +442,13 @@ class Device : public Logger {
         PreCallRecordCreateBuffer(device, pCreateInfo, pAllocator, pBuffer, record_obj);
     }
 
+    virtual void PreCallRecordCmdBindDescriptorBuffersEXT(VkCommandBuffer commandBuffer, uint32_t bufferCount,
+                                                          const VkDescriptorBufferBindingInfoEXT* pBindingInfos,
+                                                          const RecordObject& record_obj,
+                                                          chassis::CmdBindDescriptorBuffers& chassis_state) {
+        PreCallRecordCmdBindDescriptorBuffersEXT(commandBuffer, bufferCount, pBindingInfos, record_obj);
+    }
+
 #include "generated/validation_object_device_methods.h"
 };
 
