@@ -17,6 +17,7 @@
  * limitations under the License.
  */
 
+#include <sstream>
 #include <string>
 
 #include <vulkan/vk_enum_string_helper.h>
@@ -88,39 +89,39 @@ bool CoreChecks::ValidateCreateBufferDescriptorBuffer(const VkBufferCreateInfo &
     bool skip = false;
 
     if (usage & VK_BUFFER_USAGE_2_SAMPLER_DESCRIPTOR_BUFFER_BIT_EXT) {
-        if (create_info.size + device_state->samplerDescriptorBufferAddressSpaceSize >
+        if (create_info.size + device_state->descriptor_buffer_address_space.sampler >
             phys_dev_ext_props.descriptor_buffer_props.samplerDescriptorBufferAddressSpaceSize) {
             skip |= LogError(
                 "VUID-VkBufferCreateInfo-usage-08097", device, create_info_loc.dot(Field::size),
                 "(%" PRIuLEAST64 ") + current total (%" PRIuLEAST64
                 ") is greater than specified in properties field samplerDescriptorBufferAddressSpaceSize (%" PRIuLEAST64 ").",
-                create_info.size, device_state->samplerDescriptorBufferAddressSpaceSize.load(),
+                create_info.size, device_state->descriptor_buffer_address_space.sampler.load(),
                 phys_dev_ext_props.descriptor_buffer_props.samplerDescriptorBufferAddressSpaceSize);
-        } else if (create_info.size + device_state->descriptorBufferAddressSpaceSize >
+        } else if (create_info.size + device_state->descriptor_buffer_address_space.all >
                    phys_dev_ext_props.descriptor_buffer_props.descriptorBufferAddressSpaceSize) {
             skip |= LogError("VUID-VkBufferCreateInfo-usage-08097", device, create_info_loc.dot(Field::size),
                              "(%" PRIuLEAST64 ") + current total (%" PRIuLEAST64
                              ") is greater than specified in properties field descriptorBufferAddressSpaceSize (%" PRIuLEAST64 ")",
-                             create_info.size, device_state->descriptorBufferAddressSpaceSize.load(),
+                             create_info.size, device_state->descriptor_buffer_address_space.all.load(),
                              phys_dev_ext_props.descriptor_buffer_props.descriptorBufferAddressSpaceSize);
         }
     }
 
     if (usage & VK_BUFFER_USAGE_2_RESOURCE_DESCRIPTOR_BUFFER_BIT_EXT) {
-        if (create_info.size + device_state->resourceDescriptorBufferAddressSpaceSize >
+        if (create_info.size + device_state->descriptor_buffer_address_space.resource >
             phys_dev_ext_props.descriptor_buffer_props.resourceDescriptorBufferAddressSpaceSize) {
             skip |= LogError(
                 "VUID-VkBufferCreateInfo-usage-08098", device, create_info_loc.dot(Field::size),
                 "(%" PRIuLEAST64 ") + current total (%" PRIuLEAST64
                 ") is greater than specified in properties field resourceDescriptorBufferAddressSpaceSize (%" PRIuLEAST64 ").",
-                create_info.size, device_state->resourceDescriptorBufferAddressSpaceSize.load(),
+                create_info.size, device_state->descriptor_buffer_address_space.resource.load(),
                 phys_dev_ext_props.descriptor_buffer_props.resourceDescriptorBufferAddressSpaceSize);
-        } else if (create_info.size + device_state->descriptorBufferAddressSpaceSize >
+        } else if (create_info.size + device_state->descriptor_buffer_address_space.all >
                    phys_dev_ext_props.descriptor_buffer_props.descriptorBufferAddressSpaceSize) {
             skip |= LogError("VUID-VkBufferCreateInfo-usage-08098", device, create_info_loc.dot(Field::size),
                              "(%" PRIuLEAST64 ") + current total (%" PRIuLEAST64
                              ") is greater than specified in properties field descriptorBufferAddressSpaceSize (%" PRIuLEAST64 ").",
-                             create_info.size, device_state->descriptorBufferAddressSpaceSize.load(),
+                             create_info.size, device_state->descriptor_buffer_address_space.all.load(),
                              phys_dev_ext_props.descriptor_buffer_props.descriptorBufferAddressSpaceSize);
         }
     }
@@ -153,7 +154,7 @@ bool CoreChecks::ValidateCreateBufferDescriptorBuffer(const VkBufferCreateInfo &
             skip |=
                 LogError("VUID-VkBufferCreateInfo-usage-08102", device, create_info_loc.dot(Field::usage),
                          "has VK_BUFFER_USAGE_2_PUSH_DESCRIPTORS_DESCRIPTOR_BUFFER_BIT_EXT set but the bufferlessPushDescriptors "
-                         "device feature is enabled.");
+                         "device property is VK_TRUE.");
         }
 
         if (!(usage &
@@ -257,7 +258,9 @@ bool CoreChecks::PreCallValidateCreateBufferView(VkDevice device, const VkBuffer
 
     if (enabled_features.texelBufferAlignment) {
         VkDeviceSize texel_block_size = GetTexelBufferFormatSize(pCreateInfo->format);
+        bool texel_size_of_three = false;
         if ((texel_block_size % 3) == 0) {
+            texel_size_of_three = true;
             texel_block_size /= 3;
         }
         if (buffer_state.usage & VK_BUFFER_USAGE_2_STORAGE_TEXEL_BUFFER_BIT) {
@@ -266,17 +269,26 @@ bool CoreChecks::PreCallValidateCreateBufferView(VkDevice device, const VkBuffer
                 alignment_requirement = std::min(alignment_requirement, texel_block_size);
             }
             if (SafeModulo(pCreateInfo->offset, alignment_requirement) != 0) {
-                skip |= LogError("VUID-VkBufferViewCreateInfo-buffer-02750", objlist, create_info_loc.dot(Field::buffer),
-                                 "was created with VK_BUFFER_USAGE_2_STORAGE_TEXEL_BUFFER_BIT, so the offset (%" PRIuLEAST64
-                                 ") must be a multiple of the lesser of "
-                                 "storageTexelBufferOffsetAlignmentBytes (%" PRIuLEAST64
-                                 ") or, if storageTexelBufferOffsetSingleTexelAlignment "
-                                 "(%" PRId32
-                                 ") is VK_TRUE, the size of a texel of the requested format. "
-                                 "If the size of a texel is a multiple of three bytes, then the size of a "
-                                 "single component of format is used instead",
-                                 pCreateInfo->offset, phys_dev_props_core13.storageTexelBufferOffsetAlignmentBytes,
-                                 phys_dev_props_core13.storageTexelBufferOffsetSingleTexelAlignment);
+                std::stringstream ss;
+                ss << "was created with VK_BUFFER_USAGE_2_STORAGE_TEXEL_BUFFER_BIT, so the offset (" << pCreateInfo->offset
+                   << ") must be a multiple of " << alignment_requirement << "\n";
+                if (phys_dev_props_core13.storageTexelBufferOffsetSingleTexelAlignment) {
+                    ss << "storageTexelBufferOffsetSingleTexelAlignment is VK_TRUE, so we take "
+                          "min(storageTexelBufferOffsetAlignmentBytes, texelBlockSize("
+                       << string_VkFormat(pCreateInfo->format) << ")) which is min("
+                       << phys_dev_props_core13.storageTexelBufferOffsetAlignmentBytes << ", " << texel_block_size << ")";
+                    if (texel_size_of_three) {
+                        ss << "\nThe size of a texel " << (texel_block_size * 3)
+                           << " was a multiple of three bytes, so the size of a single component of "
+                           << string_VkFormat(pCreateInfo->format) << " was used instead";
+                    }
+                } else {
+                    ss << "storageTexelBufferOffsetSingleTexelAlignment is VK_FALSE and storageTexelBufferOffsetAlignmentBytes is "
+                       << phys_dev_props_core13.storageTexelBufferOffsetAlignmentBytes;
+                }
+
+                skip |= LogError("VUID-VkBufferViewCreateInfo-buffer-02750", objlist, create_info_loc.dot(Field::buffer), "%s",
+                                 ss.str().c_str());
             }
         }
         if (buffer_state.usage & VK_BUFFER_USAGE_2_UNIFORM_TEXEL_BUFFER_BIT) {
@@ -285,19 +297,33 @@ bool CoreChecks::PreCallValidateCreateBufferView(VkDevice device, const VkBuffer
                 alignment_requirement = std::min(alignment_requirement, texel_block_size);
             }
             if (SafeModulo(pCreateInfo->offset, alignment_requirement) != 0) {
-                skip |= LogError("VUID-VkBufferViewCreateInfo-buffer-02751", objlist, create_info_loc.dot(Field::buffer),
-                                 "was created with VK_BUFFER_USAGE_2_UNIFORM_TEXEL_BUFFER_BIT, so the offset (%" PRIuLEAST64
-                                 ") must be a multiple of the lesser of "
-                                 "uniformTexelBufferOffsetAlignmentBytes (%" PRIuLEAST64
-                                 ") or, if uniformTexelBufferOffsetSingleTexelAlignment "
-                                 "(%" PRId32
-                                 ") is VK_TRUE, the size of a texel of the requested format. "
-                                 "If the size of a texel is a multiple of three bytes, then the size of a "
-                                 "single component of format is used instead",
-                                 pCreateInfo->offset, phys_dev_props_core13.uniformTexelBufferOffsetAlignmentBytes,
-                                 phys_dev_props_core13.uniformTexelBufferOffsetSingleTexelAlignment);
+                std::stringstream ss;
+                ss << "was created with VK_BUFFER_USAGE_2_UNIFORM_TEXEL_BUFFER_BIT, so the offset (" << pCreateInfo->offset
+                   << ") must be a multiple of " << alignment_requirement << "\n";
+                if (phys_dev_props_core13.uniformTexelBufferOffsetSingleTexelAlignment) {
+                    ss << "uniformTexelBufferOffsetSingleTexelAlignment is VK_TRUE, so we take "
+                          "min(uniformTexelBufferOffsetAlignmentBytes, texelBlockSize("
+                       << string_VkFormat(pCreateInfo->format) << ")) which is min("
+                       << phys_dev_props_core13.uniformTexelBufferOffsetAlignmentBytes << ", " << texel_block_size << ")";
+                    if (texel_size_of_three) {
+                        ss << "\nThe size of a texel " << (texel_block_size * 3)
+                           << " was a multiple of three bytes, so the size of a single component of "
+                           << string_VkFormat(pCreateInfo->format) << " was used instead";
+                    }
+                } else {
+                    ss << "uniformTexelBufferOffsetSingleTexelAlignment is VK_FALSE and uniformTexelBufferOffsetAlignmentBytes is "
+                       << phys_dev_props_core13.uniformTexelBufferOffsetAlignmentBytes;
+                }
+
+                skip |= LogError("VUID-VkBufferViewCreateInfo-buffer-02751", objlist, create_info_loc.dot(Field::buffer), "%s",
+                                 ss.str().c_str());
             }
         }
+    } else if ((pCreateInfo->offset % phys_dev_props.limits.minTexelBufferOffsetAlignment) != 0) {
+        skip |= LogError("VUID-VkBufferViewCreateInfo-offset-02749", pCreateInfo->buffer, create_info_loc.dot(Field::offset),
+                         "(%" PRIuLEAST64
+                         ") must be a multiple of VkPhysicalDeviceLimits::minTexelBufferOffsetAlignment (%" PRIuLEAST64 ").",
+                         pCreateInfo->offset, phys_dev_props.limits.minTexelBufferOffsetAlignment);
     }
 
     // vkspec.html#resources-buffer-views-usage
