@@ -1,6 +1,7 @@
 /* Copyright (c) 2023-2025 The Khronos Group Inc.
  * Copyright (c) 2023-2025 Valve Corporation
  * Copyright (c) 2023-2025 LunarG, Inc.
+ * Copyright (c) 2025 Arm Limited.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -36,11 +37,12 @@ class SamplerDescriptor;
 class CommandBuffer;
 class Sampler;
 class DescriptorSet;
+class TensorDescriptor;
 
 class DescriptorValidator : public Logger {
   public:
     DescriptorValidator(DeviceProxy& dev, vvl::CommandBuffer& cb_state, vvl::DescriptorSet& descriptor_set, uint32_t set_index,
-                        VkFramebuffer framebuffer, const VulkanTypedHandle* shader_handle, const Location& loc);
+                        VkFramebuffer framebuffer, const LogObjectList* objlist, const Location& loc);
 
     // Used with normal validation where we know which descriptors are accessed.
     bool ValidateBindingStatic(const spirv::ResourceInterfaceVariable& binding_info, const vvl::DescriptorBinding& binding) const;
@@ -49,8 +51,10 @@ class DescriptorValidator : public Logger {
     bool ValidateBindingDynamic(const spirv::ResourceInterfaceVariable& binding_info, DescriptorBinding& binding,
                                 const uint32_t index);
     void SetSetIndexForGpuAv(uint32_t set_index) { this->set_index = set_index; }
-    void SetShaderHandleForGpuAv(const VulkanTypedHandle* shader_handle) { this->shader_handle = shader_handle; }
-    void SetLocationForGpuAv(const Location& loc) { this->loc = LocationCapture(loc); }
+    void SetObjlistForGpuAv(const LogObjectList* objlist) { this->objlist = objlist; }
+    void SetLocationForGpuAv(const Location& gpuav_loc);
+    void SetOriginalSpirv(const std::vector<uint32_t>* spirv) { this->original_spirv = spirv; };
+    void SetInstructionPositionOffset(uint32_t position_offset) { this->instruction_position_offset = position_offset; };
 
   private:
     template <typename T>
@@ -71,6 +75,8 @@ class DescriptorValidator : public Logger {
                             VkDescriptorType descriptor_type, const vvl::AccelerationStructureDescriptor& descriptor) const;
     bool ValidateDescriptor(const spirv::ResourceInterfaceVariable& binding_info, const uint32_t index,
                             VkDescriptorType descriptor_type, const vvl::SamplerDescriptor& descriptor) const;
+    bool ValidateDescriptor(const spirv::ResourceInterfaceVariable& binding_info, uint32_t index, VkDescriptorType descriptor_type,
+                            const vvl::TensorDescriptor& descriptor) const;
 
     // helper for the common parts of ImageSamplerDescriptor and SamplerDescriptor validation
     bool ValidateSamplerDescriptor(const spirv::ResourceInterfaceVariable& binding_info, uint32_t index, VkSampler sampler,
@@ -79,16 +85,23 @@ class DescriptorValidator : public Logger {
     std::string DescribeDescriptor(const spirv::ResourceInterfaceVariable& binding_info, uint32_t index,
                                    VkDescriptorType type) const;
 
+    std::string DescribeInstruction() const;
+
     vvl::DeviceProxy& dev_proxy;
+    const bool is_gpu_av;
     vvl::CommandBuffer& cb_state;
     vvl::DescriptorSet& descriptor_set;
     const VkFramebuffer framebuffer;
     LocationCapture loc;
-    const DrawDispatchVuid& vuids;
+    const DrawDispatchVuid* vuids;
+
+    const std::vector<uint32_t>* original_spirv;
+    uint32_t instruction_position_offset;
 
     // For GPU-AV, these can become aliased and need to be mutable between descriptor accesses
-    uint32_t set_index;
     // A descriptor set might be used between multiple shaders and need to adjust which one was found
-    const VulkanTypedHandle* shader_handle;  // VkPipeline or VkShaderObject
+    uint32_t set_index;
+
+    const LogObjectList* objlist;  // VkPipeline or VkShaderObject
 };
 }  // namespace vvl
