@@ -37,6 +37,7 @@
 #include "state_tracker/sampler_state.h"
 #include "state_tracker/shader_object_state.h"
 #include "state_tracker/tensor_state.h"
+#include "state_tracker/descriptor_mode.h"
 
 namespace gpuav {
 
@@ -49,10 +50,19 @@ class CommandBufferSubState : public vvl::CommandBufferSubState {
         const std::vector<std::string> &initial_label_stack;
     };
 
-    using OnInstrumentionDescSetUpdate =
-        stdext::inplace_function<void(CommandBufferSubState &cb, VkPipelineBindPoint bind_point,
+    using InstrumentationErrorLogger =
+        stdext::inplace_function<bool(Validator &gpuav, const Location &loc, const uint32_t *error_record,
+                                      std::string &out_error_msg, std::string &out_vuid_msg)>;
+    using OnInstrumentationErrorLoggerRegister = stdext::inplace_function<InstrumentationErrorLogger(
+        Validator &gpuav, CommandBufferSubState &cb, const LastBound &last_bound)>;
+    using OnInstrumentationDescSetUpdate =
+        stdext::inplace_function<void(CommandBufferSubState &cb, VkPipelineBindPoint bind_point, const Location &loc,
                                       VkDescriptorBufferInfo &out_buffer_info, uint32_t &out_dst_binding),
                                  48>;
+    using OnInstrumentationDescBufferUpdate =
+        stdext::inplace_function<void(CommandBufferSubState &cb, VkPipelineBindPoint bind_point,
+                                      VkDescriptorAddressInfoEXT &out_address_info, uint32_t &out_dst_binding),
+                                 48>;
     using OnCommandBufferSubmission =
         stdext::inplace_function<void(Validator &gpuav, CommandBufferSubState &cb, VkCommandBuffer per_submission_cb)>;
     using OnCommandBufferCompletion =
@@ -60,22 +70,24 @@ class CommandBufferSubState : public vvl::CommandBufferSubState {
                                       const CommandBufferSubState::LabelLogging &label_logging, const Location &submission_loc),
                                  64>;
     using OnPreCommandBufferSubmission =
-        stdext::inplace_function<void(Validator &gpuav, CommandBufferSubState &cb, VkCommandBuffer per_pre_submission_cb)>;
+        stdext::inplace_function<void(Validator &gpuav, CommandBufferSubState &cb, VkCommandBuffer per_pre_submission_cb), 48>;
     using OnPostCommandBufferSubmission =
         stdext::inplace_function<void(Validator &gpuav, CommandBufferSubState &cb, VkCommandBuffer per_post_submission_cb)>;
-    std::vector<OnInstrumentionDescSetUpdate> on_instrumentation_desc_set_update_functions;
+    std::vector<OnInstrumentationErrorLoggerRegister> on_instrumentation_error_logger_register_functions;
+    std::vector<OnInstrumentationDescSetUpdate> on_instrumentation_desc_set_update_functions;
+    std::vector<OnInstrumentationDescBufferUpdate> on_instrumentation_desc_buffer_update_functions;
     std::vector<OnPreCommandBufferSubmission> on_pre_cb_submission_functions;
     std::vector<OnPostCommandBufferSubmission> on_post_cb_submission_functions;
     std::vector<OnCommandBufferCompletion> on_cb_completion_functions;
 
-    vko::SharedResourcesCache shared_resources_cache;
+    vko::SharedResourcesCache<false> shared_resources_cache;
 
     // Used to track which spot in the command buffer the error came from
-    bool max_actions_cmd_validation_reached_ = false;
     uint32_t draw_index = 0;
     uint32_t compute_index = 0;
     uint32_t trace_rays_index = 0;
-    uint32_t action_command_count = 0;
+    uint32_t GetActionCommandIndex(VkPipelineBindPoint bind_point) const;
+    void IncrementActionCommandCount(VkPipelineBindPoint bind_point);
 
     std::vector<PushConstantData> push_constant_data_chunks;
     std::array<VkPipelineLayout, vvl::BindPointCount> push_constant_latest_used_layout{};
@@ -104,8 +116,6 @@ class CommandBufferSubState : public vvl::CommandBufferSubState {
         return cmd_errors_counts_buffer_.VkHandle();
     }
 
-    void IncrementCommandCount(VkPipelineBindPoint bind_point, const Location &loc);
-
     std::string GetDebugLabelRegion(uint32_t label_command_i, const std::vector<std::string> &initial_label_stack) const;
 
     void Destroy() final;
@@ -125,16 +135,18 @@ class CommandBufferSubState : public vvl::CommandBufferSubState {
 
     // Using stdext::inplace_function over std::function to allocate memory in place
     using ErrorLoggerFunc =
-        stdext::inplace_function<bool(const uint32_t *error_record, const Location &loc, const LogObjectList &objlist,
-                                      const std::vector<std::string> &initial_label_stack),
-                                 288 /*lambda storage size (bytes), large enough to store biggest error lambda*/>;
+        stdext::inplace_function<bool(const uint32_t *error_record, const Location &loc_with_debug_region,
+                                      const LogObjectList &objlist),
+                                 80 /*lambda storage size (bytes), large enough to store biggest error lambda*/>;
     struct CommandErrorLogger {
         vvl::LocationCapture loc;
         LogObjectList objlist;
         ErrorLoggerFunc error_logger_func;
-        int32_t label_cmd_i = -1;
+        uint32_t label_cmd_i;
     };
-    std::vector<CommandErrorLogger> command_error_loggers;
+    void AddCommandErrorLogger(const Location &loc, const LastBound *last_bound, ErrorLoggerFunc error_logger_func);
+    uint32_t GetErrorLoggerIndex() { return (uint32_t)command_error_loggers_.size(); }
+    const CommandErrorLogger &GetErrorLogger(uint32_t i) { return command_error_loggers_[i]; }
 
     // Buffer storing GPU-AV errors
     vko::BufferRange error_output_buffer_range_;
@@ -142,6 +154,9 @@ class CommandBufferSubState : public vvl::CommandBufferSubState {
     // Used to limit the number of errors a single command can emit.
     vko::Buffer cmd_errors_counts_buffer_;
 
+    // Track which index we have bound our Descriptor Buffer in CmdBindDescriptorBuffersEXT
+    uint32_t resource_descriptor_buffer_index_;
+
   private:
     void AllocateResources(const Location &loc);
     void ResetCBState(bool should_destroy);
@@ -149,6 +164,7 @@ class CommandBufferSubState : public vvl::CommandBufferSubState {
 
     Validator &gpuav_;
     VkDescriptorSetLayout instrumentation_desc_set_layout_ = VK_NULL_HANDLE;
+    std::vector<CommandErrorLogger> command_error_loggers_;
 };
 
 static inline CommandBufferSubState &SubState(vvl::CommandBuffer &cb) {
@@ -185,7 +201,7 @@ class QueueSubState : public vvl::QueueSubState {
     void PostSubmit(std::deque<vvl::QueueSubmission> &submissions) override;
     void Retire(vvl::QueueSubmission &) override;
 
-    vko::SharedResourcesCache shared_resources_cache;
+    vko::SharedResourcesCache<false> shared_resources_cache;
 
   protected:
     void SubmitBarrier(const Location &loc, uint64_t seq);
@@ -365,9 +381,12 @@ class PipelineSubState : public vvl::PipelineSubState {
 
     void Destroy() override;
 
-    VkPipelineLayout GetPipelineLayoutUnion(const Location &loc) const;
+    VkPipelineLayout GetPipelineLayoutUnion(const Location &loc, vvl::DescriptorMode mode) const;
 
   private:
+    // Multiple threads can record multiple commands using the same pipeline,
+    // so pipeline layout recreation has to be thread safe
+    mutable std::mutex recreated_layout_mutex{};
     mutable VkPipelineLayout recreated_layout = VK_NULL_HANDLE;
     Validator &gpuav_;
 };
