@@ -43,7 +43,7 @@ bool MaxCmdErrorsCountReached() {
     return cmd_errors_count >= kMaxErrorsPerCmd;
 }
 
-void GpuavLogError4(uint error_group, uint error_sub_code, uint param_0, uint param_1, uint param_2, uint param_3) {
+void GpuavLogError4(uint error_group, uint error_sub_code, uint dword_0, uint dword_1, uint dword_2, uint dword_3) {
     if (MaxCmdErrorsCountReached()) return;
 
     uint vo_idx = atomicAdd(errors_count, kErrorRecordSize);
@@ -54,12 +54,12 @@ void GpuavLogError4(uint error_group, uint error_sub_code, uint param_0, uint pa
     errors_buffer[vo_idx + kHeaderErrorRecordSizeOffset] = kErrorRecordSize;
     errors_buffer[vo_idx + kHeaderActionIdErrorLoggerIdOffset] = (action_index[0] << kActionIdShift) | resource_index[0];
 
-    errors_buffer[vo_idx + kPreActionParamOffset_0] = param_0;
-    errors_buffer[vo_idx + kPreActionParamOffset_1] = param_1;
-    errors_buffer[vo_idx + kPreActionParamOffset_2] = param_2;
-    errors_buffer[vo_idx + kPreActionParamOffset_3] = param_3;
+    errors_buffer[vo_idx + kValCmdErrorPayloadDword_0] = dword_0;
+    errors_buffer[vo_idx + kValCmdErrorPayloadDword_1] = dword_1;
+    errors_buffer[vo_idx + kValCmdErrorPayloadDword_2] = dword_2;
+    errors_buffer[vo_idx + kValCmdErrorPayloadDword_3] = dword_3;
 }
 
-void GpuavLogError2(uint error_group, uint error_sub_code, uint param_0, uint param_1) {
-    GpuavLogError4(error_group, error_sub_code, param_0, param_1, 0, 0);
+void GpuavLogError2(uint error_group, uint error_sub_code, uint dword_0, uint dword_1) {
+    GpuavLogError4(error_group, error_sub_code, dword_0, dword_1, 0, 0);
 }
