@@ -65,9 +65,9 @@ void vvl::DescriptorPool::Allocate(const VkDescriptorSetAllocateInfo *alloc_info
         available_counts_[it->first] -= ds_data.required_descriptors_by_type.at(it->first);
     }
 
-    const auto *variable_count_info = vku::FindStructInPNextChain<VkDescriptorSetVariableDescriptorCountAllocateInfo>(alloc_info->pNext);
-    const bool variable_count_valid =
-        variable_count_info && variable_count_info->descriptorSetCount == alloc_count;
+    const auto *variable_count_info =
+        vku::FindStructInPNextChain<VkDescriptorSetVariableDescriptorCountAllocateInfo>(alloc_info->pNext);
+    const bool variable_count_valid = variable_count_info && variable_count_info->descriptorSetCount == alloc_count;
 
     // Create tracking object for each descriptor set; insert into global map and the pool's set.
     for (uint32_t i = 0; i < alloc_count; i++) {
@@ -180,12 +180,12 @@ vvl::DescriptorClass vvl::DescriptorTypeToClass(VkDescriptorType type) {
             return DescriptorClass::InlineUniform;
         case VK_DESCRIPTOR_TYPE_ACCELERATION_STRUCTURE_KHR:
         case VK_DESCRIPTOR_TYPE_ACCELERATION_STRUCTURE_NV:
+        case VK_DESCRIPTOR_TYPE_PARTITIONED_ACCELERATION_STRUCTURE_NV:
             return DescriptorClass::AccelerationStructure;
         case VK_DESCRIPTOR_TYPE_MUTABLE_EXT:
             return DescriptorClass::Mutable;
         case VK_DESCRIPTOR_TYPE_TENSOR_ARM:
             return DescriptorClass::Tensor;
-        case VK_DESCRIPTOR_TYPE_PARTITIONED_ACCELERATION_STRUCTURE_NV:
         case VK_DESCRIPTOR_TYPE_MAX_ENUM:
             break;
     }
@@ -197,15 +197,6 @@ using DescriptorSetLayout = vvl::DescriptorSetLayout;
 using DescriptorSetLayoutDef = vvl::DescriptorSetLayoutDef;
 using DescriptorSetLayoutId = vvl::DescriptorSetLayoutId;
 
-// Canonical dictionary of DescriptorSetLayoutDef (without any handle/device specific information)
-vvl::DescriptorSetLayoutDict descriptor_set_layout_dict;
-
-static DescriptorSetLayoutId GetCanonicalId(const VkDescriptorSetLayoutCreateInfo *p_create_info, vvl::DeviceState &device_state) {
-    return descriptor_set_layout_dict.LookUp(DescriptorSetLayoutDef(device_state, p_create_info));
-}
-
-void ClearDescriptorSetLayoutCanonicalIdDict() { descriptor_set_layout_dict.Clear(); }
-
 std::string DescriptorSetLayoutDef::DescribeDifference(uint32_t index, const DescriptorSetLayoutDef &other) const {
     std::ostringstream ss;
     ss << "Set " << index << " ";
@@ -233,40 +224,48 @@ std::string DescriptorSetLayoutDef::DescribeDifference(uint32_t index, const Des
     } else if (lhs_bindings.size() != rhs_bindings.size()) {
         ss << "binding count " << lhs_bindings.size() << " doesn't match " << rhs_bindings.size();
     } else {
+        bool found = false;
         for (uint32_t i = 0; i < lhs_bindings.size(); i++) {
+            if (found) {
+                break;
+            }
             const auto &l = lhs_bindings[i];
             const auto &r = rhs_bindings[i];
             if (l.binding != r.binding) {
                 ss << "VkDescriptorSetLayoutBinding::binding " << l.binding << " doesn't match " << r.binding;
-                break;
+                found = true;
             } else if (l.descriptorType != r.descriptorType) {
                 ss << "binding " << i << " descriptorType " << string_VkDescriptorType(l.descriptorType) << " doesn't match "
                    << string_VkDescriptorType(r.descriptorType);
-                break;
+                found = true;
             } else if (l.descriptorCount != r.descriptorCount) {
                 ss << "binding " << i << " descriptorCount " << l.descriptorCount << " doesn't match " << r.descriptorCount;
-                break;
+                found = true;
             } else if (l.stageFlags != r.stageFlags) {
                 ss << "binding " << i << " stageFlags " << string_VkShaderStageFlags(l.stageFlags) << " doesn't match "
                    << string_VkShaderStageFlags(r.stageFlags);
-                break;
+                found = true;
             } else if ((l.pImmutableSamplers && !r.pImmutableSamplers) || (!l.pImmutableSamplers && r.pImmutableSamplers)) {
                 ss << "binding " << i << " pImmutableSamplers doesn't match as one is null and one in non-null";
-                break;
+                found = true;
             } else if (l.pImmutableSamplers) {
                 for (uint32_t s = 0; s < l.descriptorCount; s++) {
                     if (l.pImmutableSamplers[s] != r.pImmutableSamplers[s]) {
                         ss << "binding " << i << " pImmutableSamplers[" << s << "] " << l.pImmutableSamplers[s] << " doesn't match "
                            << r.pImmutableSamplers[s];
+                        found = true;
                         break;
                     }
                 }
             } else if (GetMutableTypes(i) != other.GetMutableTypes(i)) {
                 // These have been sorted already so can direct compare
-                ss << "Mutable types doesn't match at binding " << i << "\n[" << PrintMutableTypes(i) << "]\ndoesn't match"
-                   << "\n[" << other.PrintMutableTypes(i) << "]";
+                ss << "Mutable types doesn't match at binding " << i << "\n[" << PrintMutableTypes(i) << "]\ndoesn't match" << "\n["
+                   << other.PrintMutableTypes(i) << "]";
+                found = true;
             }
         }
+        // If we got here, we failed IsBoundSetCompatible() but didn't find what was different, likely missing a case
+        assert(found);
     }
     ss << '\n';
     return ss.str();
@@ -277,6 +276,7 @@ std::string DescriptorSetLayoutDef::DescribeDifference(uint32_t index, const Des
 vvl::DescriptorSetLayoutDef::DescriptorSetLayoutDef(vvl::DeviceState &device_state,
                                                     const VkDescriptorSetLayoutCreateInfo *p_create_info)
     : flags_(p_create_info->flags),
+      has_ycbcr_samplers_(false),
       binding_count_(0),
       descriptor_count_(0),
       non_inline_descriptor_count_(0),
@@ -332,14 +332,20 @@ vvl::DescriptorSetLayoutDef::DescriptorSetLayoutDef(vvl::DeviceState &device_sta
         }
         // Get immutable samplers info
         if (binding_info.pImmutableSamplers != nullptr) {
+            // Lazy allocation to avoid allocating array for layouts that don't use immutable samplers.
+            // Elements that correspond to bindings that do not use immutable samplers will be empty.
             if (immutable_sampler_create_infos_.empty()) {
                 immutable_sampler_create_infos_.resize(binding_count_);
                 immutable_sampler_combined_hashes_.resize(binding_count_, 0);
             }
+
             immutable_sampler_create_infos_[binding_index].resize(binding_info.descriptorCount, {});
             hash_util::HashCombiner samplers_hc;
             for (uint32_t array_index = 0; array_index < binding_info.descriptorCount; array_index++) {
                 if (auto sampler = device_state.Get<vvl::Sampler>(binding_info.pImmutableSamplers[array_index])) {
+                    if (sampler->sampler_conversion != VK_NULL_HANDLE) {
+                        has_ycbcr_samplers_ = true;
+                    }
                     immutable_sampler_create_infos_[binding_index][array_index] = sampler->safe_create_info;
                     const size_t sampler_hash = HashSamplerCreateInfo(*sampler->safe_create_info.ptr());
                     samplers_hc << sampler_hash;
@@ -430,8 +436,7 @@ const vvl::IndexRange &vvl::DescriptorSetLayoutDef::GetGlobalIndexRangeFromIndex
 
 // For the given binding, return the global index range (half open)
 // As start and end are often needed in pairs, get both with a single lookup.
-const vvl::IndexRange &vvl::DescriptorSetLayoutDef::GetGlobalIndexRangeFromBinding(
-    const uint32_t binding) const {
+const vvl::IndexRange &vvl::DescriptorSetLayoutDef::GetGlobalIndexRangeFromBinding(const uint32_t binding) const {
     uint32_t index = GetIndexFromBinding(binding);
     return GetGlobalIndexRangeFromIndex(index);
 }
@@ -534,19 +539,17 @@ bool vvl::ImmutableSamplersAreEqual(const DescriptorSetLayoutDef &dsl_def1, cons
     if (hash1 != hash2) {
         return false;
     }
-    for (uint32_t i = 0; i < binding_index; i++) {
-        const std::vector<vku::safe_VkSamplerCreateInfo> &create_infos1 =
-            dsl_def1.GetImmutableSamplerCreateInfosFromIndex(binding_index);
-        const std::vector<vku::safe_VkSamplerCreateInfo> &create_infos2 =
-            dsl_def1.GetImmutableSamplerCreateInfosFromIndex(binding_index);
-        if (create_infos1.size() != create_infos2.size()) {
+    const std::vector<vku::safe_VkSamplerCreateInfo> &create_infos1 =
+        dsl_def1.GetImmutableSamplerCreateInfosFromIndex(binding_index);
+    const std::vector<vku::safe_VkSamplerCreateInfo> &create_infos2 =
+        dsl_def1.GetImmutableSamplerCreateInfosFromIndex(binding_index);
+    if (create_infos1.size() != create_infos2.size()) {
+        return false;
+    }
+    for (size_t s = 0; s < create_infos1.size(); s++) {
+        if (!CompareSamplerCreateInfo(*create_infos1[s].ptr(), *create_infos2[s].ptr())) {
             return false;
         }
-        for (size_t s = 0; s < create_infos1.size(); s++) {
-            if (!CompareSamplerCreateInfo(*create_infos1[s].ptr(), *create_infos2[s].ptr())) {
-                return false;
-            }
-        }
     }
     return true;
 }
@@ -594,14 +597,17 @@ bool vvl::DescriptorSetLayout::IsCompatible(DescriptorSetLayout const *rh_ds_lay
 vvl::DescriptorSetLayout::DescriptorSetLayout(vvl::DeviceState &device_state, const VkDescriptorSetLayoutCreateInfo *pCreateInfo,
                                               const VkDescriptorSetLayout handle)
     : StateObject(handle, kVulkanObjectTypeDescriptorSetLayout),
-      layout_id_(GetCanonicalId(pCreateInfo, device_state)),
+      layout_id_(device_state.GetCanonicalId(pCreateInfo)),
       desc_set_layout_ci(pCreateInfo) {
-    if (pCreateInfo->flags & VK_DESCRIPTOR_SET_LAYOUT_CREATE_DESCRIPTOR_BUFFER_BIT_EXT) {
+    const bool is_descriptor_buffer = (pCreateInfo->flags & VK_DESCRIPTOR_SET_LAYOUT_CREATE_DESCRIPTOR_BUFFER_BIT_EXT) != 0;
+    if (is_descriptor_buffer) {
         DispatchGetDescriptorSetLayoutSizeEXT(device_state.VkHandle(), handle, &layout_size_in_bytes_);
     }
+
     if (pCreateInfo->pBindings) {
         for (uint32_t i = 0; i < pCreateInfo->bindingCount; i++) {
-            binding_to_original_index_map_[pCreateInfo->pBindings[i].binding] = i;
+            const uint32_t binding = pCreateInfo->pBindings[i].binding;
+            binding_to_original_index_map_[binding] = i;
         }
     }
 }
@@ -723,7 +729,7 @@ void vvl::DescriptorSet::NotifyInvalidate(const NodeList &invalid_nodes, bool un
 uint32_t vvl::DescriptorSet::GetDynamicOffsetIndexFromBinding(uint32_t dynamic_binding) const {
     const uint32_t index = layout_->GetIndexFromBinding(dynamic_binding);
     if (index == bindings_.size()) {  // binding not found
-        return vvl::kU32Max;
+        return vvl::kNoIndex32;
     }
     assert(IsDynamicDescriptor(bindings_[index]->type));
     uint32_t dynamic_offset_index = 0;
@@ -1063,7 +1069,7 @@ void vvl::ImageDescriptor::CopyUpdate(DescriptorSet &set_state, const vvl::Devic
 void vvl::ImageDescriptor::UpdateImageLayoutDrawState(vvl::CommandBuffer &cb_state) {
     // Add binding for image
     if (auto iv_state = GetImageViewState()) {
-        cb_state.TrackImageViewFirstLayout(*iv_state, image_layout_);
+        cb_state.TrackImageViewFirstLayout(*iv_state, image_layout_, nullptr);
     }
 }
 
@@ -1389,7 +1395,7 @@ void vvl::MutableDescriptor::CopyUpdate(DescriptorSet &set_state, const vvl::Dev
         case DescriptorClass::TexelBuffer: {
             ReplaceStatePtr(set_state, buffer_view_state_, static_cast<const TexelDescriptor &>(src).GetSharedBufferViewState(),
                             is_bindless);
-            buffer_size = buffer_view_state_ ? buffer_view_state_->Size() : vvl::kU32Max;
+            buffer_size = buffer_view_state_ ? buffer_view_state_->Size() : vvl::kNoIndex32;
             break;
         }
         case DescriptorClass::GeneralBuffer: {
@@ -1498,7 +1504,7 @@ void vvl::MutableDescriptor::UpdateImageLayoutDrawState(vvl::CommandBuffer &cb_s
     const vvl::DescriptorClass active_class = ActiveClass();
     if (active_class == DescriptorClass::Image || active_class == DescriptorClass::ImageSampler) {
         if (image_view_state_) {
-            cb_state.TrackImageViewFirstLayout(*image_view_state_, image_layout_);
+            cb_state.TrackImageViewFirstLayout(*image_view_state_, image_layout_, nullptr);
         }
     }
 }
