@@ -48,7 +48,7 @@ class Instruction {
     uint32_t Word(uint32_t index) const { return words_[index]; }
     // Skips pass any optional Result or Result Type word
     uint32_t Operand(uint32_t index) const { return words_[operand_index_ + index]; }
-
+    // Length of words_ array, saved in 2 higher bytes of the first element
     uint32_t Length() const { return words_[0] >> 16; }
 
     uint32_t Opcode() const { return words_[0] & 0x0ffffu; }
@@ -81,6 +81,9 @@ class Instruction {
     spv::Dim FindImageDim() const;
     bool IsImageArray() const;
     bool IsImageMultisampled() const;
+    bool IsTensor() const;
+    bool IsConstant() const;
+    bool IsSpecConstant() const;
 
     // Auto-generated helper functions
     spv::StorageClass StorageClass() const;
@@ -114,7 +117,7 @@ class Instruction {
     //   414 of 423 had 6 or less operands
     //   361 of 423 had 5 or less operands
     //   287 of 423 had 4 or less operands
-    // An extra word is still needed because each insturction has one word prior to the operands
+    // An extra word is still needed because each instruction has one word prior to the operands
     static constexpr uint32_t word_vector_length = 7;
 
     // Max capacity needs to be uint32_t because an instruction can have a string operand that is (2^16)-1 bytes long
