@@ -1,4 +1,5 @@
 /* Copyright (c) 2021-2025 The Khronos Group Inc.
+ * Copyright (c) 2025 Arm Limited.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -23,7 +24,6 @@
 #include <cstring>
 #include <string>
 #include <vector>
-#include <optional>
 
 #include "containers/custom_containers.h"
 #include "state_tracker/shader_instruction.h"
@@ -39,11 +39,14 @@ class Pipeline;
 namespace spirv {
 struct EntryPoint;
 struct Module;
+struct ParsedInfo;
 
-static constexpr uint32_t kInvalidValue = std::numeric_limits<uint32_t>::max();
+// We can assume the upper 3 uint max values are not going to be used for anything meaningful in SPIR-V
+static constexpr uint32_t kInvalidValue = vvl::kNoIndex32;
 
 // Need to find a way to know if actually array length of zero, or a runtime array.
-static constexpr uint32_t kRuntimeArray = std::numeric_limits<uint32_t>::max();
+static constexpr uint32_t kRuntimeArray = vvl::kNoIndex32 - 1;
+static constexpr uint32_t kSpecConstant = vvl::kNoIndex32 - 2;
 
 struct LocalSize {
     uint32_t x = 0;
@@ -70,6 +73,7 @@ struct DecorationBase {
         input_attachment_bit = 1 << 8,
         per_task_nv = 1 << 9,
         per_primitive_ext = 1 << 10,
+        sample_bit = 1 << 11,
     };
 
     // bits to know if things have been set or not by a Decoration
@@ -94,8 +98,8 @@ struct DecorationBase {
 // Can't have nested structs with OpMemberDecorate, this class prevents accidently creating a 2nd level of member decorations,
 struct DecorationSet : public DecorationBase {
     // For descriptors
-    uint32_t set = 0;
-    uint32_t binding = 0;
+    uint32_t set = kInvalidValue;
+    uint32_t binding = kInvalidValue;
 
     // Value of InputAttachmentIndex the variable starts
     uint32_t input_attachment_index_start = kInvalidValue;
@@ -111,67 +115,89 @@ struct DecorationSet : public DecorationBase {
 
 // Tracking of OpExecutionMode / OpExecutionModeId values
 struct ExecutionModeSet {
-    enum FlagBit {
-        output_points_bit = 1 << 0,
-        point_mode_bit = 1 << 1,
-        post_depth_coverage_bit = 1 << 2,
-        local_size_bit = 1 << 3,
-        local_size_id_bit = 1 << 4,
-        iso_lines_bit = 1 << 5,
-        xfb_bit = 1 << 6,
-        early_fragment_test_bit = 1 << 7,
-        subgroup_uniform_control_flow_bit = 1 << 8,
+    enum FlagBit : uint64_t {
+        output_points_bit = 1ull << 0,
+        output_lines_bit = 1ull << 1,
+        output_triangle_bit = 1ull << 2,
 
-        signed_zero_inf_nan_preserve_width_16 = 1 << 9,
-        signed_zero_inf_nan_preserve_width_32 = 1 << 10,
-        signed_zero_inf_nan_preserve_width_64 = 1 << 11,
-        denorm_preserve_width_16 = 1 << 12,
-        denorm_preserve_width_32 = 1 << 13,
-        denorm_preserve_width_64 = 1 << 14,
-        denorm_flush_to_zero_width_16 = 1 << 15,
-        denorm_flush_to_zero_width_32 = 1 << 16,
-        denorm_flush_to_zero_width_64 = 1 << 17,
-        rounding_mode_rte_width_16 = 1 << 18,
-        rounding_mode_rte_width_32 = 1 << 19,
-        rounding_mode_rte_width_64 = 1 << 20,
-        rounding_mode_rtz_width_16 = 1 << 21,
-        rounding_mode_rtz_width_32 = 1 << 22,
-        rounding_mode_rtz_width_64 = 1 << 23,
+        subdivision_iso_lines_bit = 1ull << 3,
+        subdivision_triangle_bit = 1ull << 4,
+        subdivision_quad_bit = 1ull << 5,
 
-        depth_replacing_bit = 1 << 24,
-        stencil_ref_replacing_bit = 1 << 25,
+        vertex_order_cw_bit = 1ull << 6,
+        vertex_order_ccw_bit = 1ull << 7,
 
-        fp_fast_math_default = 1 << 26,
+        spacing_equal_bit = 1ull << 8,
+        spacing_fractional_even_bit = 1ull << 9,
+        spacing_fractional_odd_bit = 1ull << 10,
 
-        derivative_group_linear = 1 << 27,
-        derivative_group_quads = 1 << 28,
+        point_mode_bit = 1ull << 11,
+        post_depth_coverage_bit = 1ull << 12,
+        local_size_bit = 1ull << 13,
+        local_size_id_bit = 1ull << 14,
+        xfb_bit = 1ull << 15,
+        early_fragment_test_bit = 1ull << 16,
+        subgroup_uniform_control_flow_bit = 1ull << 17,
+
+        signed_zero_inf_nan_preserve_width_16 = 1ull << 18,
+        signed_zero_inf_nan_preserve_width_32 = 1ull << 19,
+        signed_zero_inf_nan_preserve_width_64 = 1ull << 20,
+        denorm_preserve_width_16 = 1ull << 21,
+        denorm_preserve_width_32 = 1ull << 22,
+        denorm_preserve_width_64 = 1ull << 23,
+        denorm_flush_to_zero_width_16 = 1ull << 24,
+        denorm_flush_to_zero_width_32 = 1ull << 25,
+        denorm_flush_to_zero_width_64 = 1ull << 26,
+        rounding_mode_rte_width_16 = 1ull << 27,
+        rounding_mode_rte_width_32 = 1ull << 28,
+        rounding_mode_rte_width_64 = 1ull << 29,
+        rounding_mode_rtz_width_16 = 1ull << 30,
+        rounding_mode_rtz_width_32 = 1ull << 31,
+        rounding_mode_rtz_width_64 = 1ull << 32,
+
+        depth_replacing_bit = 1ull << 33,
+        stencil_ref_replacing_bit = 1ull << 34,
+
+        fp_fast_math_default = 1ull << 35,
+
+        derivative_group_linear = 1ull << 36,
+        derivative_group_quads = 1ull << 37,
+
+        geometry_input_points_bit = 1ull << 38,
+        geometry_input_line_bit = 1ull << 39,
+        geometry_input_line_adjacency_bit = 1ull << 40,
+        geometry_input_triangle_bit = 1ull << 41,
+        geometry_input_triangle_adjacency_bit = 1ull << 42,
+
+        shader_64bit_indexing = 1ull << 43,
     };
 
     // bits to know if things have been set or not by a Decoration
-    uint32_t flags = 0;
-
-    VkPrimitiveTopology input_primitive_topology = VK_PRIMITIVE_TOPOLOGY_MAX_ENUM;
-    VkPrimitiveTopology primitive_topology = VK_PRIMITIVE_TOPOLOGY_MAX_ENUM;
+    uint64_t flags = 0;
 
     // SPIR-V spec says only LocalSize or LocalSizeId can be used, so can share
     LocalSize local_size = {kInvalidValue, kInvalidValue, kInvalidValue};
 
-    uint32_t output_vertices = vvl::kU32Max;
+    uint32_t output_vertices = kInvalidValue;
     uint32_t output_primitives = 0;
     uint32_t invocations = 0;
 
-    uint32_t tessellation_subdivision = 0;
-    uint32_t tessellation_orientation = 0;
-    uint32_t tessellation_spacing = 0;
-
     void Add(const Instruction &insn);
     bool Has(FlagBit flag_bit) const { return (flags & flag_bit) != 0; }
+
+    // Helpers for the various input/output stuff Geom/Tess/Mesh has
+    uint32_t GetTessellationSubdivision() const;
+    uint32_t GetTessellationOrientation() const;
+    uint32_t GetTessellationSpacing() const;
+    VkPrimitiveTopology GetTessellationEvalOutputTopology() const;
+    VkPrimitiveTopology GetGeometryInputTopology() const;
+    VkPrimitiveTopology GetGeometryMeshOutputTopology() const;
 };
 
 struct AtomicInstructionInfo {
     uint32_t storage_class;
     uint32_t bit_width;
-    uint32_t type;  // ex. OpTypeInt
+    uint32_t type;             // ex. OpTypeInt
     uint32_t vector_size = 0;  // 0 for scalar, otherwise number of components
 };
 
@@ -190,6 +216,7 @@ struct TypeStructInfo {
     const uint32_t id;
     const uint32_t length;  // number of elements
     const DecorationSet &decorations;
+    bool has_runtime_array;
 
     // data about each member in struct
     struct Member {
@@ -220,9 +247,6 @@ constexpr uint32_t read_mask = read | atomic_read | image_read;
 constexpr uint32_t write_mask = write | atomic_write | image_write;
 }  // namespace AccessBit
 
-// Mapping of < variable ID, AccessBit >
-using VariableAccessMap = vvl::unordered_map<uint32_t, uint32_t>;
-
 // Track all paths from %param to %arg so can walk back functions
 //
 // %arg   = OpVariable
@@ -234,39 +258,28 @@ using FuncParameterMap = vvl::unordered_map<uint32_t, std::vector<uint32_t>>;
 
 // Represents the OpImage* instructions and how it maps to the variable
 // This is created in the Module but then used with VariableBase objects
-struct ImageAccess {
-    const Instruction &image_insn;  // OpImage*
+//
+// This is "static" because it is useless for descriptor indexing.
+// If 2 different OpImage* access the same variable (that is an array of descriptors), both must be valid, but for descriptor
+// indexing, these attributes aren't tied to the whole variable, just to the index accessed
+struct StaticImageAccess {
     std::vector<const Instruction *> variable_image_insn;
     // If there is a OpSampledImage there will also be a sampler variable
     std::vector<const Instruction *> variable_sampler_insn;
     // incase uncaught set of SPIR-V instruction is found, skips validating instead of crashing
     bool valid_access = true;
 
-    bool is_dref = false;
-    bool is_sampler_implicitLod_dref_proj = false;
-    bool is_sampler_sampled = false;  // OpImageSample* or OpImageSparseSample*
-    bool is_sampler_bias_offset = false;
-    bool is_sampler_offset = false;  // ConstOffset or Offset (not ConstOffsets)
-    bool is_sign_extended = false;
-    bool is_zero_extended = false;
+    const ImageInstruction image_insn;
+
     uint32_t access_mask = AccessBit::empty;
 
     uint32_t image_access_chain_index = kInvalidValue;    // OpAccessChain's Index 0
     uint32_t sampler_access_chain_index = kInvalidValue;  // OpAccessChain's Index 0
     uint32_t texel_component_count = kInvalidValue;
 
-    ImageAccess(const Module &module_state, const Instruction &image_insn, const FuncParameterMap &func_parameter_map);
+    StaticImageAccess(const Module &module_state, const Instruction &insn, const FuncParameterMap &func_parameter_map);
 };
 
-// <Image OpVariable Result ID, [ImageAccess, ImageAccess, etc] > - used for faster lookup
-// Many ImageAccess can point to a single Image Variable
-using ImageAccessMap = vvl::unordered_map<uint32_t, std::vector<std::shared_ptr<const ImageAccess>>>;
-// < Variable ID, [ OpAccessChain ] >
-// Allows for grouping the access chains by which variables they are actually accessing
-using AccessChainVariableMap = vvl::unordered_map<uint32_t, std::vector<const Instruction *>>;
-// Mapping of OpName instructions
-using DebugNameMap = vvl::unordered_map<uint32_t, const Instruction *>;
-
 // A slot is a <Location, Component> mapping
 struct InterfaceSlot {
     // A Location is made up of 4 Components
@@ -306,22 +319,24 @@ enum NumericType {
     NumericTypeFloat = 1,    // UNORM, SNORM, FLOAT, USCALED, SSCALED, SRGB -- anything we consider float in the shader
     NumericTypeSint = 2,
     NumericTypeUint = 4,
+    NumericTypeBool = 5,
 };
 uint32_t GetFormatType(VkFormat format);
-char const *string_NumericType(uint32_t type);
+const char *string_NumericType(uint32_t type);
+VkFormat GetTensorFormat(NumericType numeric_type, uint32_t bit_width);
 
 // Common info needed for all OpVariable
 struct VariableBase {
     const uint32_t id;
     const uint32_t type_id;
+    const uint32_t data_type_id;
     const spv::StorageClass storage_class;
     const DecorationSet &decorations;
     std::shared_ptr<const TypeStructInfo> type_struct_info;  // null if no struct type
     // The variable may have different access for a given entrypoint
     uint32_t access_mask;  // AccessBit
     const VkShaderStageFlagBits stage;
-    VariableBase(const Module &module_state, const Instruction &insn, VkShaderStageFlagBits stage,
-                 const VariableAccessMap &variable_access_map, const DebugNameMap &debug_name_map);
+    VariableBase(const Module &module_state, const Instruction &insn, VkShaderStageFlagBits stage, const ParsedInfo &parsed);
 
     // When no SPIR-V debug info is used, this will be empty strings
     // We need to store a std::string since the original SPIR-V string will be gone when we need to print this in an error message
@@ -351,8 +366,10 @@ struct VariableBase {
     // Something like textureSize() will access the OpVariable, but not the image itself
     bool IsImageAccessed() const { return access_mask & AccessBit::image_mask; }
 
+    bool IsUntyped() const { return data_type_id != 0; }
+
   private:
-    static const char *FindDebugName(const VariableBase &variable, const DebugNameMap &debug_name_map);
+    static const char *FindDebugName(const VariableBase &variable, const ParsedInfo &parsed);
 };
 
 // These are Input/Output OpVariable that go in-between stages
@@ -379,7 +396,7 @@ struct StageInterfaceVariable : public VariableBase {
     uint32_t total_builtin_components = 0;
 
     StageInterfaceVariable(const Module &module_state, const Instruction &insn, VkShaderStageFlagBits stage,
-                           const VariableAccessMap &variable_access_map, const DebugNameMap &debug_name_map);
+                           const ParsedInfo &parsed);
 
   protected:
     static bool IsPerTaskNV(const StageInterfaceVariable &variable);
@@ -400,65 +417,72 @@ struct StageInterfaceVariable : public VariableBase {
 struct ResourceInterfaceVariable : public VariableBase {
     // If the type is a OpTypeArray save the length
     // Will be kRuntimeArray (non-zero) for runtime arrays
-    uint32_t array_length;
+    // Will be kSpecConstant (non-zero) if needs to be resolved later
+    uint32_t array_length{0};
+    bool IsRuntimeArray() const { return array_length == kRuntimeArray; }
+    bool IsArray() const { return array_length != 0; }
 
     // OpTypeSampledImage (used for combined image samplers)
     bool is_type_sampled_image;
 
     // The index of vector is index of image. (TODO - this doesn't work for GPU-AV)
     std::vector<vvl::unordered_set<SamplerUsedByImage>> samplers_used_by_image;
-    // workaround for YCbCr to track sampler variables until |samplers_used_by_image| is fixed
-    vvl::unordered_set<uint32_t> sampled_image_sampler_variable_ids;
 
     // For storage images - list of Texel component length the OpImageWrite
     std::vector<uint32_t> write_without_formats_component_count_list;
 
-    // A variable can have an array of indexes, need to track which are written to
-    // can't use bitset because number of indexes isn't known until runtime
-    // This array will match the OpTypeArray and not consider the InputAttachmentIndex
-    std::vector<bool> input_attachment_index_read;
+    // Index into input attachment array that have been read
+    // If there is no array, index 0 is marked if the single image is read
+    vvl::unordered_set<uint32_t> input_attachment_index_read;
 
     // Type once array/pointer are stripped
-    // most likly will be OpTypeImage, OpTypeStruct, OpTypeSampler, or OpTypeAccelerationStructureKHR
+    // most likely will be OpTypeImage, OpTypeStruct, OpTypeSampler, or OpTypeAccelerationStructureKHR
     const Instruction &base_type;
 
     // True if the Resource variable itself is runtime descriptor array
     // Online example to showcase various arrays we do/don't care about here https://godbolt.org/z/h9jhsKaPn
     bool is_runtime_descriptor_array;
 
+    // "constant integral expressions" is fancy spec language to mean "you are not doing dynamic descriptor indexing into an array"
+    // NOTE - This just checks if there is ANY non-costant access
+    bool all_constant_integral_expressions{true};
+
     // All info regarding what will be validated from requirements imposed by the pipeline on a descriptor. These
-    // can't be checked at pipeline creation time as they depend on the Image or ImageView bound.
+    // can't be checked at pipeline creation time as they depend on the object bound (Image/Tensor) or its view.
     // That is perf-critical code and hashing if 2 variables have same info provides a 20% perf bonus
+    //
+    // This info can be broken down into two further parts:
+    //  1. Things that are tied to the OpVariable (ex, Image Dim can be found from the OpTypeImage)
+    //  2. Things that depend on the image access operation (ex, Dref is not known looking at a OpVariable)
+    // For Descriptor Indexing, we have to be careful about the 2nd case as it is possible to an OpVariable to contain descriptors
+    // in the array that match correctly depending on the index used. (For normal 1.0 descriptor, all access to the array must be
+    // valid for all indexes in the array)
     struct Info {
-        // the 'format' operand of OpTypeImage as the corresponding Vulkan Format
-        VkFormat image_format{VK_FORMAT_UNDEFINED};
-        // the 'Sampled Type' operand of OpTypeImage,as a numeric type (float, uint, int)
-        NumericType image_sampled_type_numeric{NumericTypeUnknown};
-        // the 'Sampled Type' operand of OpTypeImage as the bit width (64 is the largest bit width in SPIR-V)
-        uint8_t image_sampled_type_width{0};
+        // the 'format' operand of OpTypeImage as the corresponding Vulkan Format. Computed for OpTypeTensorARM
+        VkFormat vk_format{VK_FORMAT_UNDEFINED};
+        // the 'Type' operand of OpTypeImage/OpTypeTensorARM, as a numeric type (float, uint, int, bool)
+        NumericType numeric_type{NumericTypeUnknown};
+        // the width in bits of the 'Type' operand of OpTypeImage/OpTypeTensorARM (64 is the largest bit width in SPIR-V)
+        uint8_t bit_width{0};
 
         spv::Dim image_dim;
-        bool is_image_array;
-        bool is_multisampled;
+        bool is_image_array{false};
+        bool is_multisampled{false};
+
+        ImageInstruction image_insn;
+
+        uint32_t tensor_rank{0};
 
-        bool is_sampler_sampled{false};  // OpImageSample* or OpImageSparseSample*
-        bool is_sampler_implicitLod_dref_proj{false};
-        bool is_sampler_bias_offset{false};
-        bool is_sampler_offset{false};        // ConstOffset or Offset (not ConstOffsets)
         bool is_read_without_format{false};   // For storage images
         bool is_write_without_format{false};  // For storage images
-        bool is_dref{false};
 
-        // vkspec.html#spirvenv-image-signedness describes how SignExtend/ZeroExtend can be used per-access to adjust the Signedness
-        // Only need to check if one access has explicit signedness, mixing should be caught in spirv-val
-        bool is_sign_extended{false};  // if at least one access has SignExtended
-        bool is_zero_extended{false};  // if at least one access has ZeroExtended
-
-        // If a variable is used as a function arguement, but never actually used, it will be found in EntryPoint::accessible_ids so
+        // If a variable is used as a function argument, but never actually used, it will be found in EntryPoint::accessible_ids so
         // we need to have a dedicated mark if it was accessed.
         // We use this for variable hashing, but the VariableBase has the helper functions to read this value.
         uint32_t access_mask{AccessBit::empty};
     } info;
+    // For non descriptor indexing usages, this hash allows use to skip re-validating because a different VkImageView bound will
+    // result in the same outcome
     uint64_t descriptor_hash = 0;
     bool IsImage() const { return base_type.Opcode() == spv::OpTypeImage; }
 
@@ -466,15 +490,17 @@ struct ResourceInterfaceVariable : public VariableBase {
     bool is_storage_image{false};
     bool is_storage_texel_buffer{false};
     const bool is_storage_buffer;
+    const bool is_uniform_buffer;
     bool is_input_attachment{false};
+    bool is_storage_tensor{false};
 
     ResourceInterfaceVariable(const Module &module_state, const EntryPoint &entrypoint, const Instruction &insn,
-                              const ImageAccessMap &image_access_map, const AccessChainVariableMap &access_chain_map,
-                              const VariableAccessMap &variable_access_map, const DebugNameMap &debug_name_map);
+                              const ParsedInfo &parsed);
 
   protected:
     static const Instruction &FindBaseType(ResourceInterfaceVariable &variable, const Module &module_state);
     static bool IsStorageBuffer(const ResourceInterfaceVariable &variable);
+    static bool IsUniformBuffer(const ResourceInterfaceVariable &variable);
 };
 
 // Used to help detect if different variable is being used
@@ -491,7 +517,13 @@ struct PushConstantVariable : public VariableBase {
     uint32_t size;    // total size of block
 
     PushConstantVariable(const Module &module_state, const Instruction &insn, VkShaderStageFlagBits stage,
-                         const VariableAccessMap &variable_access_map, const DebugNameMap &debug_name_map);
+                         const ParsedInfo &parsed);
+};
+
+struct TaskPayloadVariable : public VariableBase {
+    uint32_t size;
+
+    TaskPayloadVariable(const Module &module_state, const Instruction &insn, VkShaderStageFlagBits stage, const ParsedInfo &parsed);
 };
 
 // Represents a single Entrypoint into a Shader Module
@@ -499,6 +531,7 @@ struct EntryPoint {
     // "A module must not have two OpEntryPoint instructions with the same Execution Model and the same Name string."
     // There is no single unique item for a single entry point
     const Instruction &entrypoint_insn;  // OpEntryPoint instruction
+    const bool is_data_graph;
     // For things like MeshNV vs MeshEXT, we need the execution_model
     const spv::ExecutionModel execution_model;
     const VkShaderStageFlagBits stage;
@@ -515,6 +548,8 @@ struct EntryPoint {
 
     // only one Push Constant block is allowed per entry point
     std::shared_ptr<const PushConstantVariable> push_constant_variable;
+    // For both Task and Mesh entry point, there can be one TaskPayloadWorkgroupEXT variable
+    std::shared_ptr<const TaskPayloadVariable> task_payload_variable;
     const std::vector<ResourceInterfaceVariable> resource_interface_variables;
     const std::vector<StageInterfaceVariable> stage_interface_variables;
     // Easier to lookup without having to check for the is_builtin bool
@@ -549,24 +584,18 @@ struct EntryPoint {
 
     bool has_physical_storage_buffer_interface{false};
 
-    EntryPoint(const Module &module_state, const Instruction &entrypoint_insn, const ImageAccessMap &image_access_map,
-               const AccessChainVariableMap &access_chain_map, const VariableAccessMap &variable_access_map,
-               const DebugNameMap &debug_name_map);
+    EntryPoint(const Module &module_state, const Instruction &entrypoint_insn, const ParsedInfo &parsed);
 
     bool HasBuiltIn(spv::BuiltIn built_in) const;
 
   protected:
     static vvl::unordered_set<uint32_t> GetAccessibleIds(const Module &module_state, EntryPoint &entrypoint);
-    static std::vector<StageInterfaceVariable> GetStageInterfaceVariables(const Module &module_state, const EntryPoint &entrypoint,
-                                                                          const VariableAccessMap &variable_access_map,
-                                                                          const DebugNameMap &debug_name_map);
+    static std::vector<StageInterfaceVariable> GetStageInterfaceVariables(const Module &module_state, EntryPoint &entrypoint,
+                                                                          const ParsedInfo &parsed);
     static std::vector<ResourceInterfaceVariable> GetResourceInterfaceVariables(const Module &module_state, EntryPoint &entrypoint,
-                                                                                const ImageAccessMap &image_access_map,
-                                                                                const AccessChainVariableMap &access_chain_map,
-                                                                                const VariableAccessMap &variable_access_map,
-                                                                                const DebugNameMap &debug_name_map);
+                                                                                const ParsedInfo &parsed);
     static bool IsBuiltInWritten(spv::BuiltIn built_in, const Module &module_state, const StageInterfaceVariable &variable,
-                                 const AccessChainVariableMap &access_chain_map);
+                                 const ParsedInfo &parsed);
 };
 
 // Info to capture while parsing the SPIR-V, but will only be used by SpirvValidator::Validate and don't need to save after
@@ -582,6 +611,8 @@ struct StatelessData {
     std::vector<const Instruction *> group_inst;
     // OpEmitStreamVertex/OpEndStreamPrimitive - only allowed in Geometry shader
     std::vector<const Instruction *> transform_feedback_stream_inst;
+    std::vector<const Instruction *> fma_inst;
+    std::vector<const Instruction *> tensor_inst;
 
     // simpler to just track all OpExecutionModeId and parse things needed later
     std::vector<const Instruction *> execution_mode_id_inst;
@@ -599,7 +630,7 @@ struct Module {
     // The goal of this struct is to move everything that is ready only into here
     struct StaticData {
         StaticData() = default;
-        StaticData(const Module &module_state, StatelessData *stateless_data = nullptr);
+        StaticData(const Module &module_state, bool parse, StatelessData *stateless_data);
         StaticData &operator=(StaticData &&) = default;
         StaticData(StaticData &&) = default;
 
@@ -623,8 +654,10 @@ struct Module {
         // Find all decoration instructions to prevent relooping module later - many checks need this info
         std::vector<const Instruction *> decoration_inst;
         std::vector<const Instruction *> member_decoration_inst;
-        // Find all variable instructions to prevent relookping module later
+        // Find all variable instructions to build faster LUT
         std::vector<const Instruction *> variable_inst;
+        // Both variables and instruction explicitly accessing untyped variables
+        std::vector<const Instruction *> explicit_memory_inst;
         // For shader tile image - OpDepthAttachmentReadEXT/OpStencilAttachmentReadEXT/OpColorAttachmentReadEXT
         bool has_shader_tile_image_depth_read{false};
         bool has_shader_tile_image_stencil_read{false};
@@ -637,8 +670,16 @@ struct Module {
         uint32_t builtin_workgroup_size_id = 0;
 
         std::vector<const Instruction *> cooperative_matrix_inst;
-
         std::vector<const Instruction *> cooperative_vector_inst;
+        std::vector<const Instruction *> emit_mesh_tasks_inst;
+        std::vector<const Instruction *> array_length_inst;
+
+        // For descriptor indexing there are 3 situations
+        //  1. OpConstant is used, we will find later
+        //  2. The value is dynamic, caught in GPU-AV
+        //  3. OpSpecConstant, which we can detect, but not until we have the values
+        // We only need to check index 1, since you can't have double array of descriptors
+        std::vector<const Instruction*> descriptor_indexing_spec_const_ac_inst;
 
         std::vector<spv::Capability> capability_list;
         // Code on the hot path can cache capabilities for fast access.
@@ -678,14 +719,20 @@ struct Module {
     VulkanTypedHandle handle_;                            // Will be updated once its known its valid SPIR-V
     VulkanTypedHandle handle() const { return handle_; }  // matches normal convention to get handle
 
-    // Used for when modifying the SPIR-V (spirv-opt, GPU-AV instrumentation, etc) and need reparse it for VVL validation
-    Module(vvl::span<const uint32_t> code) : valid_spirv(true), words_(code.begin(), code.end()), static_data_(*this) {}
+    // Only currently used for when modifying the SPIR-V after spirv-opt and we need reparse it for VVL validation
+    explicit Module(vvl::span<const uint32_t> code)
+        : valid_spirv(true), words_(code.begin(), code.end()), static_data_(*this, true, nullptr) {}
 
+    // Used when we want to create a spirv::Module object (to make it easier to have a handle everywhere) but don't actually want to
+    // store/parse the SPIR-V itself (because it is turned off via settings)
+    explicit Module(bool is_valid_spirv) : valid_spirv(is_valid_spirv) {}
+
+    // "Normal" case
     // StatelessData is a pointer as we have cases were we don't need it and simpler to just null check the few cases that use it
-    Module(size_t codeSize, const uint32_t *pCode, StatelessData *stateless_data = nullptr)
-        : valid_spirv(pCode && pCode[0] == spv::MagicNumber && ((codeSize % 4) == 0)),
+    Module(size_t codeSize, const uint32_t *pCode, bool is_valid_spirv, bool parse, StatelessData *stateless_data)
+        : valid_spirv(is_valid_spirv),
           words_(pCode, pCode + codeSize / sizeof(uint32_t)),
-          static_data_(*this, stateless_data) {}
+          static_data_(*this, parse, stateless_data) {}
 
     const Instruction *FindDef(uint32_t id) const {
         auto it = static_data_.definitions.find(id);
@@ -718,17 +765,16 @@ struct Module {
     std::string DescribeVariable(uint32_t id) const;
     std::string DescribeInstruction(const Instruction &error_insn) const;
 
-    // Note that some shaders can have an input and output topology
-    std::optional<VkPrimitiveTopology> GetTopology(const EntryPoint &entrypoint) const;
-
-    std::shared_ptr<const EntryPoint> FindEntrypoint(char const *name, VkShaderStageFlagBits stageBits) const;
+    std::shared_ptr<const EntryPoint> FindEntrypoint(const char *name, VkShaderStageFlagBits stageBits) const;
     LocalSize FindLocalSize(const EntryPoint &entrypoint) const;
 
     uint32_t CalculateWorkgroupSharedMemory() const;
-    uint32_t CalculateTaskPayloadMemory() const;
 
-    const Instruction *GetConstantDef(uint32_t id) const;
+    const Instruction *GetAnyConstantDef(uint32_t id) const;
     uint32_t GetConstantValueById(uint32_t id) const;
+    bool GetBoolIfConstant(const spirv::Instruction &insn, bool *value) const;
+    bool GetInt32IfConstant(const spirv::Instruction &insn, uint32_t *value) const;
+
     uint32_t GetLocationsConsumedByType(uint32_t type) const;
     uint32_t GetComponentsConsumedByType(uint32_t type) const;
     NumericType GetNumericType(uint32_t type) const;
@@ -742,10 +788,13 @@ struct Module {
     uint32_t GetBaseType(const Instruction *insn) const;
     const Instruction *GetBaseTypeInstruction(uint32_t type) const;
     const Instruction *GetVariablePointerType(const spirv::Instruction &var_insn) const;
+    const Instruction *GetVariableDataType(const spirv::Instruction &var_insn) const;
     uint32_t GetTypeId(uint32_t id) const;
     uint32_t GetTexelComponentCount(const Instruction &insn) const;
     uint32_t GetFlattenArraySize(const Instruction &insn) const;
     AtomicInstructionInfo GetAtomicInfo(const Instruction &insn) const;
+    spv::StorageClass StorageClass(const Instruction &insn) const;
+    bool UsesStorageCapabilityStorageClass(const Instruction &insn) const;
 
     bool HasCapability(spv::Capability find_capability) const {
         return std::any_of(static_data_.capability_list.begin(), static_data_.capability_list.end(),
@@ -755,8 +804,15 @@ struct Module {
 
 }  // namespace spirv
 
+struct GlobalSettings;
+
 // Represents a VkShaderModule handle
 namespace vvl {
+
+// Need to allow a way to not waste time copying over to spirv::Module::words_ when we don't want to store the SPIR-V
+std::shared_ptr<spirv::Module> CreateSpirvModuleState(size_t codeSize, const uint32_t *pCode, const GlobalSettings &global_settings,
+                                                      spirv::StatelessData *stateless_data = nullptr);
+
 struct ShaderModule : public StateObject {
     ShaderModule(VkShaderModule handle, std::shared_ptr<spirv::Module> &spirv_module)
         : StateObject(handle, kVulkanObjectTypeShaderModule), spirv(spirv_module) {
@@ -772,5 +828,9 @@ struct ShaderModule : public StateObject {
     // TODO - This (and vvl::ShaderObject) could be unique, but need handle multiple ValidationObjects
     // https://github.com/KhronosGroup/Vulkan-ValidationLayers/pull/6265/files
     std::shared_ptr<spirv::Module> spirv;
+
+    // Used by GPU-AV to make sure instrumentation only occurs in a single thread at a time
+    // (Currently we don't need seem to need this for ShaderObjects)
+    std::mutex module_mutex_;
 };
 }  // namespace vvl
