@@ -68,11 +68,31 @@ static vku::safe_VkImageCreateInfo GetImageCreateInfo(const VkSwapchainCreateInf
 
 namespace vvl {
 
+void SwapchainImage::ResetAcquireState() {
+    acquired = false;
+    acquire_semaphore.reset();
+    acquire_fence.reset();
+    acquire_semaphore_status = AcquireSyncStatus::NotSpecified;
+    acquire_fence_status = AcquireSyncStatus::NotSpecified;
+}
+
 void SwapchainImage::ResetPresentWaitSemaphores() {
+    const bool swapchain_has_completed_presentation = !present_wait_semaphores.empty();
     for (auto &semaphore : present_wait_semaphores) {
         semaphore->ClearSwapchainWaitInfo();
     }
     present_wait_semaphores.clear();
+
+    // If the current swapchain has completed at least one presentation then the previous
+    // presentations from the old swapchain are also finished. Mark present wait semaphores
+    // from the old swapchain as not in-use.
+    // NOTE: that's the algorithm we use to track old semaphores without swapchain maintenance1 extension.
+    if (swapchain_has_completed_presentation && image_state->bind_swapchain) {
+        for (auto &old_present_wait_semaphore : image_state->bind_swapchain->old_swapchain_present_wait_semaphores) {
+            old_present_wait_semaphore->ClearSwapchainWaitInfo();
+        }
+        image_state->bind_swapchain->old_swapchain_present_wait_semaphores.clear();
+    }
 }
 
 Swapchain::Swapchain(vvl::DeviceState &dev_data_, const VkSwapchainCreateInfoKHR *pCreateInfo, VkSwapchainKHR handle)
@@ -87,23 +107,22 @@ Swapchain::Swapchain(vvl::DeviceState &dev_data_, const VkSwapchainCreateInfoKHR
       dev_data(dev_data_) {
     // Initialize with visible values for debugging purposes.
     // This helps to show used slots during the first few frames.
-    acquire_history.fill(vvl::kU32Max);
+    acquire_history.fill(vvl::kNoIndex32);
 }
 
 void Swapchain::PresentImage(uint32_t image_index, uint64_t present_id, const SubmissionReference &present_submission_ref,
                              vvl::span<std::shared_ptr<vvl::Semaphore>> present_wait_semaphores) {
-    if (image_index >= images.size()) return;
-    assert(acquired_images > 0);
-    if (!shared_presentable) {
+    if (image_index >= images.size()) {
+        return;
+    }
+    if (shared_presentable) {
+        images[image_index].image_state->layout_locked = true;
+    } else if (acquired_images > 0) {
         acquired_images--;
-        images[image_index].acquired = false;
-        images[image_index].acquire_semaphore.reset();
-        images[image_index].acquire_fence.reset();
-    } else {
-        images[image_index].image_state->layout_locked = true;
+        images[image_index].ResetAcquireState();
     }
-    images[image_index].present_submission_ref = present_submission_ref;
 
+    images[image_index].present_submission_ref = present_submission_ref;
     images[image_index].present_wait_semaphores.clear();
     for (const auto &semaphore : present_wait_semaphores) {
         images[image_index].present_wait_semaphores.emplace_back(semaphore);
@@ -112,16 +131,26 @@ void Swapchain::PresentImage(uint32_t image_index, uint64_t present_id, const Su
     if (present_id > max_present_id) {
         max_present_id = present_id;
     }
+
+    // If this swapchain is retired (became oldSwapchain) then register the swapchain present
+    // wait semaphores in the *new* swapchain. It will be responsible for tracking semaphores in-use status.
+    // Old swapchain can't track semaphore in-use status anymore. That functionality is part of acquire logic
+    // and old swapchain can't acquire new images.
+    if (new_swapchain) {
+        auto &old_wait_semaphores = new_swapchain->old_swapchain_present_wait_semaphores;
+        old_wait_semaphores.insert(old_wait_semaphores.end(), present_wait_semaphores.begin(), present_wait_semaphores.end());
+    }
 }
 
 void Swapchain::ReleaseImage(uint32_t image_index) {
-    if (image_index >= images.size()) return;
-    assert(acquired_images > 0);
-    acquired_images--;
-    images[image_index].acquired = false;
-    images[image_index].acquire_semaphore.reset();
-    images[image_index].acquire_fence.reset();
-    images[image_index].ResetPresentWaitSemaphores();
+    if (image_index >= images.size()) {
+        return;
+    }
+    if (acquired_images > 0) {
+        acquired_images--;
+        images[image_index].ResetAcquireState();
+        images[image_index].ResetPresentWaitSemaphores();
+    }
 }
 
 void Swapchain::AcquireImage(uint32_t image_index, const std::shared_ptr<vvl::Semaphore> &semaphore_state,
@@ -130,9 +159,30 @@ void Swapchain::AcquireImage(uint32_t image_index, const std::shared_ptr<vvl::Se
     images[image_index].acquired = true;
     images[image_index].acquire_semaphore = semaphore_state;
     images[image_index].acquire_fence = fence_state;
-    if (fence_state && images[image_index].present_submission_ref.has_value()) {
-        fence_state->SetPresentSubmissionRef(*images[image_index].present_submission_ref);
-        images[image_index].present_submission_ref.reset();
+
+    if (semaphore_state) {
+        if (semaphore_state->Scope() == vvl::Semaphore::kInternal) {
+            images[image_index].acquire_semaphore_status = AcquireSyncStatus::Signaled;
+        } else {
+            // For external semaphores (where waits can't be tracked), assume an optimistic scenario
+            // in which the semaphore has been waited on.
+            images[image_index].acquire_semaphore_status = AcquireSyncStatus::WasWaitedOn;
+        }
+        semaphore_state->SetAcquiredImage(shared_from_this(), image_index);
+    }
+    if (fence_state) {
+        if (fence_state->Scope() == vvl::Fence::kInternal) {
+            images[image_index].acquire_fence_status = AcquireSyncStatus::Signaled;
+        } else {
+            // For external fences (where waits can't be tracked), assume an optimistic scenario
+            // in which the fence has been waited on.
+            images[image_index].acquire_fence_status = AcquireSyncStatus::WasWaitedOn;
+        }
+        fence_state->SetAcquiredImage(shared_from_this(), image_index);
+        if (images[image_index].present_submission_ref.has_value()) {
+            fence_state->SetPresentSubmissionRef(*images[image_index].present_submission_ref);
+            images[image_index].present_submission_ref.reset();
+        }
     }
     if (shared_presentable) {
         images[image_index].image_state->shared_presentable = shared_presentable;
@@ -208,7 +258,7 @@ uint32_t Swapchain::GetAcquiredImageIndexFromHistory(uint32_t acquire_history_in
     const uint32_t ring_buffer_index = global_index % acquire_history_max_length;
 
     const uint32_t acquire_image_index = acquire_history[ring_buffer_index];
-    assert(acquire_image_index != vvl::kU32Max);
+    assert(acquire_image_index != vvl::kNoIndex32);
     return acquire_image_index;
 }
 
@@ -302,7 +352,7 @@ vvl::span<const vku::safe_VkSurfaceFormat2KHR> Surface::GetFormats(bool get_surf
         } else {
             result.resize(count);
             for (uint32_t surface_format_index = 0; surface_format_index < count; ++surface_format_index) {
-                result.emplace_back(&formats2[surface_format_index]);
+                result[surface_format_index] = &formats2[surface_format_index];
             }
         }
     } else {
