@@ -1,6 +1,6 @@
-/* Copyright (c) 2015-2024 The Khronos Group Inc.
- * Copyright (c) 2015-2024 Valve Corporation
- * Copyright (c) 2015-2024 LunarG, Inc.
+/* Copyright (c) 2015-2025 The Khronos Group Inc.
+ * Copyright (c) 2015-2025 Valve Corporation
+ * Copyright (c) 2015-2025 LunarG, Inc.
  * Copyright (C) 2015-2024 Google Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
@@ -19,10 +19,12 @@
 #include "stateless/stateless_validation.h"
 #include "generated/dispatch_functions.h"
 
-bool StatelessValidation::manual_PreCallValidateCmdSetViewportWithCount(VkCommandBuffer commandBuffer, uint32_t viewportCount,
-                                                                        const VkViewport *pViewports,
-                                                                        const ErrorObject &error_obj) const {
+namespace stateless {
+
+bool Device::manual_PreCallValidateCmdSetViewportWithCount(VkCommandBuffer commandBuffer, uint32_t viewportCount,
+                                                           const VkViewport *pViewports, const Context &context) const {
     bool skip = false;
+    const auto &error_obj = context.error_obj;
 
     if (!enabled_features.multiViewport) {
         if (viewportCount != 1) {
@@ -51,34 +53,24 @@ bool StatelessValidation::manual_PreCallValidateCmdSetViewportWithCount(VkComman
     return skip;
 }
 
-bool StatelessValidation::manual_PreCallValidateCmdSetScissorWithCount(VkCommandBuffer commandBuffer, uint32_t scissorCount,
-                                                                       const VkRect2D *pScissors,
-                                                                       const ErrorObject &error_obj) const {
+bool Device::manual_PreCallValidateCmdSetScissorWithCount(VkCommandBuffer commandBuffer, uint32_t scissorCount,
+                                                          const VkRect2D *pScissors, const Context &context) const {
     bool skip = false;
+    const auto &error_obj = context.error_obj;
     if (!enabled_features.multiViewport) {
         if (scissorCount != 1) {
             skip |= LogError("VUID-vkCmdSetScissorWithCount-scissorCount-03398", commandBuffer,
                              error_obj.location.dot(Field::scissorCount),
-                             "(%" PRIu32
-                             ") must "
-                             "be 1 when the multiViewport feature is disabled.",
-                             scissorCount);
+                             "(%" PRIu32 ") must be 1 when the multiViewport feature is disabled.", scissorCount);
         }
     } else {  // multiViewport enabled
         if (scissorCount == 0) {
             skip |= LogError("VUID-vkCmdSetScissorWithCount-scissorCount-03397", commandBuffer,
-                             error_obj.location.dot(Field::scissorCount),
-                             "(%" PRIu32
-                             ") must "
-                             "be great than zero.",
-                             scissorCount);
+                             error_obj.location.dot(Field::scissorCount), "(%" PRIu32 ") must be greater than zero.", scissorCount);
         } else if (scissorCount > device_limits.maxViewports) {
-            skip |= LogError("VUID-vkCmdSetScissorWithCount-scissorCount-03397", commandBuffer,
-                             error_obj.location.dot(Field::scissorCount),
-                             "(%" PRIu32
-                             ") must "
-                             "not be greater than maxViewports (%" PRIu32 ").",
-                             scissorCount, device_limits.maxViewports);
+            skip |= LogError(
+                "VUID-vkCmdSetScissorWithCount-scissorCount-03397", commandBuffer, error_obj.location.dot(Field::scissorCount),
+                "(%" PRIu32 ") must not be greater than maxViewports (%" PRIu32 ").", scissorCount, device_limits.maxViewports);
         }
     }
 
@@ -116,11 +108,13 @@ bool StatelessValidation::manual_PreCallValidateCmdSetScissorWithCount(VkCommand
     return skip;
 }
 
-bool StatelessValidation::manual_PreCallValidateCmdSetVertexInputEXT(
-    VkCommandBuffer commandBuffer, uint32_t vertexBindingDescriptionCount,
-    const VkVertexInputBindingDescription2EXT *pVertexBindingDescriptions, uint32_t vertexAttributeDescriptionCount,
-    const VkVertexInputAttributeDescription2EXT *pVertexAttributeDescriptions, const ErrorObject &error_obj) const {
+bool Device::manual_PreCallValidateCmdSetVertexInputEXT(VkCommandBuffer commandBuffer, uint32_t vertexBindingDescriptionCount,
+                                                        const VkVertexInputBindingDescription2EXT *pVertexBindingDescriptions,
+                                                        uint32_t vertexAttributeDescriptionCount,
+                                                        const VkVertexInputAttributeDescription2EXT *pVertexAttributeDescriptions,
+                                                        const Context &context) const {
     bool skip = false;
+    const auto &error_obj = context.error_obj;
 
     if (vertexBindingDescriptionCount > device_limits.maxVertexInputBindings) {
         skip |= LogError("VUID-vkCmdSetVertexInputEXT-vertexBindingDescriptionCount-04791", commandBuffer,
@@ -159,8 +153,8 @@ bool StatelessValidation::manual_PreCallValidateCmdSetVertexInputEXT(
             auto const &binding_it = vertex_bindings.find(binding);
             if (binding_it != vertex_bindings.cend()) {
                 skip |= LogError("VUID-vkCmdSetVertexInputEXT-pVertexBindingDescriptions-04794", commandBuffer,
-                                 error_obj.location.dot(Field::pVertexBindingDescriptions, binding),
-                                 "is already in pVertexBindingDescriptions[%" PRIu32 "].", *binding_it);
+                                 error_obj.location.dot(Field::pVertexBindingDescriptions, i),
+                                 "and pVertexBindingDescriptions[%" PRIu32 "] are both %" PRIu32 ".", *binding_it, binding);
                 break;
             }
             vertex_bindings.insert(binding);
@@ -172,8 +166,8 @@ bool StatelessValidation::manual_PreCallValidateCmdSetVertexInputEXT(
             auto const &location_it = vertex_locations.find(location);
             if (location_it != vertex_locations.cend()) {
                 skip |= LogError("VUID-vkCmdSetVertexInputEXT-pVertexAttributeDescriptions-04795", commandBuffer,
-                                 error_obj.location.dot(Field::pVertexAttributeDescriptions, location),
-                                 "is already in pVertexAttributeDescriptions[%" PRIu32 "].", *location_it);
+                                 error_obj.location.dot(Field::pVertexAttributeDescriptions, i),
+                                 "and pVertexAttributeDescriptions[%" PRIu32 "] are both %" PRIu32 ".", *location_it, location);
                 break;
             }
             vertex_locations.insert(location);
@@ -210,13 +204,11 @@ bool StatelessValidation::manual_PreCallValidateCmdSetVertexInputEXT(
                                  "vertexAttributeInstanceRateDivisor feature was not enabled",
                                  pVertexBindingDescriptions[binding].divisor);
             } else {
-                if (pVertexBindingDescriptions[binding].divisor >
-                    phys_dev_ext_props.vertex_attribute_divisor_props.maxVertexAttribDivisor) {
+                if (pVertexBindingDescriptions[binding].divisor > phys_dev_props_core14.maxVertexAttribDivisor) {
                     skip |= LogError("VUID-VkVertexInputBindingDescription2EXT-divisor-06226", commandBuffer,
                                      binding_loc.dot(Field::divisor),
                                      "(%" PRIu32 ") is greater than maxVertexAttribDivisor (%" PRIu32 ")",
-                                     pVertexBindingDescriptions[binding].divisor,
-                                     phys_dev_ext_props.vertex_attribute_divisor_props.maxVertexAttribDivisor);
+                                     pVertexBindingDescriptions[binding].divisor, phys_dev_props_core14.maxVertexAttribDivisor);
                 }
 
                 if (pVertexBindingDescriptions[binding].inputRate != VK_VERTEX_INPUT_RATE_INSTANCE) {
@@ -267,12 +259,11 @@ bool StatelessValidation::manual_PreCallValidateCmdSetVertexInputEXT(
     return skip;
 }
 
-bool StatelessValidation::manual_PreCallValidateCmdSetDiscardRectangleEXT(VkCommandBuffer commandBuffer,
-                                                                          uint32_t firstDiscardRectangle,
-                                                                          uint32_t discardRectangleCount,
-                                                                          const VkRect2D *pDiscardRectangles,
-                                                                          const ErrorObject &error_obj) const {
+bool Device::manual_PreCallValidateCmdSetDiscardRectangleEXT(VkCommandBuffer commandBuffer, uint32_t firstDiscardRectangle,
+                                                             uint32_t discardRectangleCount, const VkRect2D *pDiscardRectangles,
+                                                             const Context &context) const {
     bool skip = false;
+    const auto &error_obj = context.error_obj;
 
     if (!pDiscardRectangles) {
         return skip;
@@ -281,7 +272,7 @@ bool StatelessValidation::manual_PreCallValidateCmdSetDiscardRectangleEXT(VkComm
         const Location loc = error_obj.location.dot(Field::pDiscardRectangles, i);
         const int64_t x_sum =
             static_cast<int64_t>(pDiscardRectangles[i].offset.x) + static_cast<int64_t>(pDiscardRectangles[i].extent.width);
-        if (x_sum > std::numeric_limits<int32_t>::max()) {
+        if (x_sum > vvl::kI32Max) {
             skip |= LogError("VUID-vkCmdSetDiscardRectangleEXT-offset-00588", commandBuffer, loc,
                              "offset.x (%" PRId32 ") + extent.width (%" PRIu32 ") is %" PRIi64 " which will overflow int32_t.",
                              pDiscardRectangles[i].offset.x, pDiscardRectangles[i].extent.width, x_sum);
@@ -289,7 +280,7 @@ bool StatelessValidation::manual_PreCallValidateCmdSetDiscardRectangleEXT(VkComm
 
         const int64_t y_sum =
             static_cast<int64_t>(pDiscardRectangles[i].offset.y) + static_cast<int64_t>(pDiscardRectangles[i].extent.height);
-        if (y_sum > std::numeric_limits<int32_t>::max()) {
+        if (y_sum > vvl::kI32Max) {
             skip |= LogError("VUID-vkCmdSetDiscardRectangleEXT-offset-00589", commandBuffer, loc,
                              "offset.y (%" PRId32 ") + extent.height (%" PRIu32 ") is %" PRIi64 " which will overflow int32_t.",
                              pDiscardRectangles[i].offset.y, pDiscardRectangles[i].extent.height, y_sum);
@@ -299,10 +290,10 @@ bool StatelessValidation::manual_PreCallValidateCmdSetDiscardRectangleEXT(VkComm
     return skip;
 }
 
-bool StatelessValidation::manual_PreCallValidateCmdSetDiscardRectangleEnableEXT(VkCommandBuffer commandBuffer,
-                                                                                VkBool32 discardRectangleEnable,
-                                                                                const ErrorObject &error_obj) const {
+bool Device::manual_PreCallValidateCmdSetDiscardRectangleEnableEXT(VkCommandBuffer commandBuffer, VkBool32 discardRectangleEnable,
+                                                                   const Context &context) const {
     bool skip = false;
+    const auto &error_obj = context.error_obj;
     if (discard_rectangles_extension_version < 2) {
         skip |= LogError("VUID-vkCmdSetDiscardRectangleEnableEXT-specVersion-07851", commandBuffer, error_obj.location,
                          "Requires support for version 2 of VK_EXT_discard_rectangles.");
@@ -310,10 +301,11 @@ bool StatelessValidation::manual_PreCallValidateCmdSetDiscardRectangleEnableEXT(
     return skip;
 }
 
-bool StatelessValidation::manual_PreCallValidateCmdSetDiscardRectangleModeEXT(VkCommandBuffer commandBuffer,
-                                                                              VkDiscardRectangleModeEXT discardRectangleMode,
-                                                                              const ErrorObject &error_obj) const {
+bool Device::manual_PreCallValidateCmdSetDiscardRectangleModeEXT(VkCommandBuffer commandBuffer,
+                                                                 VkDiscardRectangleModeEXT discardRectangleMode,
+                                                                 const Context &context) const {
     bool skip = false;
+    const auto &error_obj = context.error_obj;
     if (discard_rectangles_extension_version < 2) {
         skip |= LogError("VUID-vkCmdSetDiscardRectangleModeEXT-specVersion-07852", commandBuffer, error_obj.location,
                          "Requires support for version 2 of VK_EXT_discard_rectangles.");
@@ -321,12 +313,12 @@ bool StatelessValidation::manual_PreCallValidateCmdSetDiscardRectangleModeEXT(Vk
     return skip;
 }
 
-bool StatelessValidation::manual_PreCallValidateCmdSetExclusiveScissorEnableNV(VkCommandBuffer commandBuffer,
-                                                                               uint32_t firstExclusiveScissor,
-                                                                               uint32_t exclusiveScissorCount,
-                                                                               const VkBool32 *pExclusiveScissorEnables,
-                                                                               const ErrorObject &error_obj) const {
+bool Device::manual_PreCallValidateCmdSetExclusiveScissorEnableNV(VkCommandBuffer commandBuffer, uint32_t firstExclusiveScissor,
+                                                                  uint32_t exclusiveScissorCount,
+                                                                  const VkBool32 *pExclusiveScissorEnables,
+                                                                  const Context &context) const {
     bool skip = false;
+    const auto &error_obj = context.error_obj;
     if (scissor_exclusive_extension_version < 2) {
         skip |= LogError("VUID-vkCmdSetExclusiveScissorEnableNV-exclusiveScissor-07853", commandBuffer, error_obj.location,
                          "Requires support for version 2 of VK_NV_scissor_exclusive.");
@@ -334,12 +326,11 @@ bool StatelessValidation::manual_PreCallValidateCmdSetExclusiveScissorEnableNV(V
     return skip;
 }
 
-bool StatelessValidation::manual_PreCallValidateCmdSetExclusiveScissorNV(VkCommandBuffer commandBuffer,
-                                                                         uint32_t firstExclusiveScissor,
-                                                                         uint32_t exclusiveScissorCount,
-                                                                         const VkRect2D *pExclusiveScissors,
-                                                                         const ErrorObject &error_obj) const {
+bool Device::manual_PreCallValidateCmdSetExclusiveScissorNV(VkCommandBuffer commandBuffer, uint32_t firstExclusiveScissor,
+                                                            uint32_t exclusiveScissorCount, const VkRect2D *pExclusiveScissors,
+                                                            const Context &context) const {
     bool skip = false;
+    const auto &error_obj = context.error_obj;
 
     if (!enabled_features.multiViewport) {
         if (firstExclusiveScissor != 0) {
@@ -396,26 +387,28 @@ bool StatelessValidation::manual_PreCallValidateCmdSetExclusiveScissorNV(VkComma
     return skip;
 }
 
-bool StatelessValidation::manual_PreCallValidateCmdSetViewportWScalingNV(VkCommandBuffer commandBuffer, uint32_t firstViewport,
-                                                                         uint32_t viewportCount,
-                                                                         const VkViewportWScalingNV *pViewportWScalings,
-                                                                         const ErrorObject &error_obj) const {
+bool Device::manual_PreCallValidateCmdSetViewportWScalingNV(VkCommandBuffer commandBuffer, uint32_t firstViewport,
+                                                            uint32_t viewportCount, const VkViewportWScalingNV *pViewportWScalings,
+                                                            const Context &context) const {
     bool skip = false;
+    const auto &error_obj = context.error_obj;
     const uint64_t sum = static_cast<uint64_t>(firstViewport) + static_cast<uint64_t>(viewportCount);
     if ((sum < 1) || (sum > device_limits.maxViewports)) {
         skip |= LogError("VUID-vkCmdSetViewportWScalingNV-firstViewport-01324", commandBuffer, error_obj.location,
                          "firstViewport (%" PRIu32 ") + viewportCount (%" PRIu32 ") is %" PRIu64
-                         ", which must be between 1 and VkPhysicalDeviceLimits::maxViewports (%" PRIu32 "), inculsive.",
+                         ", which must be between 1 and VkPhysicalDeviceLimits::maxViewports (%" PRIu32 "), inclusive.",
                          firstViewport, viewportCount, sum, device_limits.maxViewports);
     }
 
     return skip;
 }
 
-bool StatelessValidation::manual_PreCallValidateCmdSetViewportShadingRatePaletteNV(
-    VkCommandBuffer commandBuffer, uint32_t firstViewport, uint32_t viewportCount,
-    const VkShadingRatePaletteNV *pShadingRatePalettes, const ErrorObject &error_obj) const {
+bool Device::manual_PreCallValidateCmdSetViewportShadingRatePaletteNV(VkCommandBuffer commandBuffer, uint32_t firstViewport,
+                                                                      uint32_t viewportCount,
+                                                                      const VkShadingRatePaletteNV *pShadingRatePalettes,
+                                                                      const Context &context) const {
     bool skip = false;
+    const auto &error_obj = context.error_obj;
 
     if (!enabled_features.multiViewport) {
         if (firstViewport != 0) {
@@ -441,12 +434,13 @@ bool StatelessValidation::manual_PreCallValidateCmdSetViewportShadingRatePalette
     return skip;
 }
 
-bool StatelessValidation::manual_PreCallValidateCmdSetCoarseSampleOrderNV(VkCommandBuffer commandBuffer,
-                                                                          VkCoarseSampleOrderTypeNV sampleOrderType,
-                                                                          uint32_t customSampleOrderCount,
-                                                                          const VkCoarseSampleOrderCustomNV *pCustomSampleOrders,
-                                                                          const ErrorObject &error_obj) const {
+bool Device::manual_PreCallValidateCmdSetCoarseSampleOrderNV(VkCommandBuffer commandBuffer,
+                                                             VkCoarseSampleOrderTypeNV sampleOrderType,
+                                                             uint32_t customSampleOrderCount,
+                                                             const VkCoarseSampleOrderCustomNV *pCustomSampleOrders,
+                                                             const Context &context) const {
     bool skip = false;
+    const auto &error_obj = context.error_obj;
 
     if (sampleOrderType != VK_COARSE_SAMPLE_ORDER_TYPE_CUSTOM_NV && customSampleOrderCount != 0) {
         skip |= LogError("VUID-vkCmdSetCoarseSampleOrderNV-sampleOrderType-02081", commandBuffer, error_obj.location,
@@ -462,10 +456,10 @@ bool StatelessValidation::manual_PreCallValidateCmdSetCoarseSampleOrderNV(VkComm
     return skip;
 }
 
-bool StatelessValidation::manual_PreCallValidateCmdSetViewport(VkCommandBuffer commandBuffer, uint32_t firstViewport,
-                                                               uint32_t viewportCount, const VkViewport *pViewports,
-                                                               const ErrorObject &error_obj) const {
+bool Device::manual_PreCallValidateCmdSetViewport(VkCommandBuffer commandBuffer, uint32_t firstViewport, uint32_t viewportCount,
+                                                  const VkViewport *pViewports, const Context &context) const {
     bool skip = false;
+    const auto &error_obj = context.error_obj;
 
     if (!enabled_features.multiViewport) {
         if (firstViewport != 0) {
@@ -498,11 +492,11 @@ bool StatelessValidation::manual_PreCallValidateCmdSetViewport(VkCommandBuffer c
     return skip;
 }
 
-bool StatelessValidation::manual_PreCallValidateCmdSetDepthClampRangeEXT(VkCommandBuffer commandBuffer,
-                                                                         VkDepthClampModeEXT depthClampMode,
-                                                                         const VkDepthClampRangeEXT *pDepthClampRange,
-                                                                         const ErrorObject &error_obj) const {
+bool Device::manual_PreCallValidateCmdSetDepthClampRangeEXT(VkCommandBuffer commandBuffer, VkDepthClampModeEXT depthClampMode,
+                                                            const VkDepthClampRangeEXT *pDepthClampRange,
+                                                            const Context &context) const {
     bool skip = false;
+    const auto &error_obj = context.error_obj;
     if (depthClampMode == VK_DEPTH_CLAMP_MODE_USER_DEFINED_RANGE_EXT) {
         if (!pDepthClampRange) {
             skip |= LogError("VUID-vkCmdSetDepthClampRangeEXT-pDepthClampRange-09647", device,
@@ -514,10 +508,10 @@ bool StatelessValidation::manual_PreCallValidateCmdSetDepthClampRangeEXT(VkComma
     return skip;
 }
 
-bool StatelessValidation::manual_PreCallValidateCmdSetScissor(VkCommandBuffer commandBuffer, uint32_t firstScissor,
-                                                              uint32_t scissorCount, const VkRect2D *pScissors,
-                                                              const ErrorObject &error_obj) const {
+bool Device::manual_PreCallValidateCmdSetScissor(VkCommandBuffer commandBuffer, uint32_t firstScissor, uint32_t scissorCount,
+                                                 const VkRect2D *pScissors, const Context &context) const {
     bool skip = false;
+    const auto &error_obj = context.error_obj;
 
     if (!enabled_features.multiViewport) {
         if (firstScissor != 0) {
@@ -572,9 +566,9 @@ bool StatelessValidation::manual_PreCallValidateCmdSetScissor(VkCommandBuffer co
     return skip;
 }
 
-bool StatelessValidation::manual_PreCallValidateCmdSetLineWidth(VkCommandBuffer commandBuffer, float lineWidth,
-                                                                const ErrorObject &error_obj) const {
+bool Device::manual_PreCallValidateCmdSetLineWidth(VkCommandBuffer commandBuffer, float lineWidth, const Context &context) const {
     bool skip = false;
+    const auto &error_obj = context.error_obj;
 
     if (!enabled_features.wideLines && (lineWidth != 1.0f)) {
         skip |= LogError("VUID-vkCmdSetLineWidth-lineWidth-00788", commandBuffer, error_obj.location.dot(Field::lineWidth),
@@ -584,9 +578,10 @@ bool StatelessValidation::manual_PreCallValidateCmdSetLineWidth(VkCommandBuffer
     return skip;
 }
 
-bool StatelessValidation::manual_PreCallValidateCmdSetLineStipple(VkCommandBuffer commandBuffer, uint32_t lineStippleFactor,
-                                                                  uint16_t lineStipplePattern, const ErrorObject &error_obj) const {
+bool Device::manual_PreCallValidateCmdSetLineStipple(VkCommandBuffer commandBuffer, uint32_t lineStippleFactor,
+                                                     uint16_t lineStipplePattern, const Context &context) const {
     bool skip = false;
+    const auto &error_obj = context.error_obj;
 
     if (lineStippleFactor < 1 || lineStippleFactor > 256) {
         skip |= LogError("VUID-vkCmdSetLineStipple-lineStippleFactor-02776", commandBuffer,
@@ -595,3 +590,4 @@ bool StatelessValidation::manual_PreCallValidateCmdSetLineStipple(VkCommandBuffe
 
     return skip;
 }
+}  // namespace stateless
