@@ -16,9 +16,15 @@
  * limitations under the License.
  */
 
+#include <vulkan/utility/vk_format_utils.h>
+#include "error_message/error_location.h"
 #include "stateless/stateless_validation.h"
 #include "generated/enum_flag_bits.h"
 #include "error_message/error_strings.h"
+#include "containers/span.h"
+#include "containers/container_utils.h"
+#include "utils/image_utils.h"
+#include "utils/math_utils.h"
 
 namespace stateless {
 bool Device::ValidateCoarseSampleOrderCustomNV(const VkCoarseSampleOrderCustomNV &order, const Location &order_loc) const {
@@ -86,7 +92,7 @@ bool Device::ValidateCoarseSampleOrderCustomNV(const VkCoarseSampleOrderCustomNV
     // guarantee that 64 bits is enough, but practically it's unlikely for an
     // implementation to support more than 32 bits for samplemask.
     assert(phys_dev_ext_props.shading_rate_image_props.shadingRateMaxCoarseSamples <= 64);
-    uint64_t sample_locations_mask = 0;
+    std::bitset<64> sample_locations_mask = 0;
     for (uint32_t i = 0; i < order.sampleLocationCount; ++i) {
         const VkCoarseSampleLocationNV *sample_loc = &order.pSampleLocations[i];
         if (sample_loc->pixelX >= sample_order_info->width) {
@@ -108,10 +114,13 @@ bool Device::ValidateCoarseSampleOrderCustomNV(const VkCoarseSampleOrderCustomNV
         }
         uint32_t idx =
             sample_loc->sample + order.sampleCount * (sample_loc->pixelX + sample_order_info->width * sample_loc->pixelY);
-        sample_locations_mask |= 1ULL << idx;
+        // Account for idx being greater than or equal to 64 to prevent undefined behavior
+        if (idx < sample_locations_mask.size()) {
+            sample_locations_mask[idx] = 1;
+        }
     }
 
-    uint64_t expected_mask = (order.sampleLocationCount == 64) ? ~0ULL : ((1ULL << order.sampleLocationCount) - 1);
+    std::bitset<64> expected_mask = (order.sampleLocationCount == 64) ? ~0ULL : ((1ULL << order.sampleLocationCount) - 1);
     if (sample_locations_mask != expected_mask) {
         skip |= LogError(
             "VUID-VkCoarseSampleOrderCustomNV-pSampleLocations-02077", device, order_loc,
@@ -177,7 +186,7 @@ bool Device::ValidateSamplerFilterMinMax(const VkSamplerCreateInfo &create_info,
 }
 
 // VK_EXT_custom_border_color
-bool Device::ValidateSamplerCustomBoarderColor(const VkSamplerCreateInfo &create_info, const Location &create_info_loc) const {
+bool Device::ValidateSamplerCustomBorderColor(const VkSamplerCreateInfo &create_info, const Location &create_info_loc) const {
     bool skip = false;
 
     if (create_info.borderColor == VK_BORDER_COLOR_INT_CUSTOM_EXT || create_info.borderColor == VK_BORDER_COLOR_FLOAT_CUSTOM_EXT) {
@@ -191,8 +200,9 @@ bool Device::ValidateSamplerCustomBoarderColor(const VkSamplerCreateInfo &create
         if (!custom_create_info) {
             skip |= LogError("VUID-VkSamplerCreateInfo-borderColor-04011", device, create_info_loc.dot(Field::borderColor),
                              "is %s but there is no VkSamplerCustomBorderColorCreateInfoEXT "
-                             "struct in pNext chain.",
-                             string_VkBorderColor(create_info.borderColor));
+                             "struct in pNext chain.\n%s",
+                             string_VkBorderColor(create_info.borderColor),
+                             PrintPNextChain(Struct::VkSamplerCreateInfo, create_info.pNext).c_str());
         } else {
             if ((custom_create_info->format != VK_FORMAT_UNDEFINED) && !vkuFormatIsDepthAndStencil(custom_create_info->format) &&
                 ((create_info.borderColor == VK_BORDER_COLOR_INT_CUSTOM_EXT &&
@@ -330,20 +340,15 @@ bool Device::ValidateSamplerImageProcessingQCOM(const VkSamplerCreateInfo &creat
     return skip;
 }
 
-bool Device::manual_PreCallValidateCreateSampler(VkDevice device, const VkSamplerCreateInfo *pCreateInfo,
-                                                 const VkAllocationCallbacks *pAllocator, VkSampler *pSampler,
-                                                 const Context &context) const {
+bool Device::ValidateSamplerCreateInfo(const VkSamplerCreateInfo &create_info, const Location &create_info_loc,
+                                       const Context &context) const {
     bool skip = false;
-    const auto &error_obj = context.error_obj;
 
-    const Location create_info_loc = error_obj.location.dot(Field::pCreateInfo);
-    const auto &limits = device_limits;
-
-    if (pCreateInfo->anisotropyEnable == VK_TRUE) {
-        if (!IsBetweenInclusive(pCreateInfo->maxAnisotropy, 1.0F, limits.maxSamplerAnisotropy)) {
+    if (create_info.anisotropyEnable == VK_TRUE) {
+        if (!IsBetweenInclusive(create_info.maxAnisotropy, 1.0F, device_limits.maxSamplerAnisotropy)) {
             skip |= LogError("VUID-VkSamplerCreateInfo-anisotropyEnable-01071", device, create_info_loc.dot(Field::maxAnisotropy),
-                             "is %f but must be in the range of [1.0, %f] (maxSamplerAnistropy).", pCreateInfo->maxAnisotropy,
-                             limits.maxSamplerAnisotropy);
+                             "is %f but must be in the range of [1.0, %f] (maxSamplerAnistropy).", create_info.maxAnisotropy,
+                             device_limits.maxSamplerAnisotropy);
         }
 
         // Anistropy cannot be enabled in sampler unless enabled as a feature
@@ -354,69 +359,69 @@ bool Device::manual_PreCallValidateCreateSampler(VkDevice device, const VkSample
         }
     }
 
-    if (pCreateInfo->unnormalizedCoordinates == VK_TRUE) {
-        if (pCreateInfo->minFilter != pCreateInfo->magFilter) {
+    if (create_info.unnormalizedCoordinates == VK_TRUE) {
+        if (create_info.minFilter != create_info.magFilter) {
             skip |= LogError("VUID-VkSamplerCreateInfo-unnormalizedCoordinates-01072", device,
                              create_info_loc.dot(Field::unnormalizedCoordinates),
                              "is VK_TRUE, but minFilter (%s) is different then magFilter (%s).",
-                             string_VkFilter(pCreateInfo->minFilter), string_VkFilter(pCreateInfo->magFilter));
+                             string_VkFilter(create_info.minFilter), string_VkFilter(create_info.magFilter));
         }
-        if (pCreateInfo->mipmapMode != VK_SAMPLER_MIPMAP_MODE_NEAREST) {
+        if (create_info.mipmapMode != VK_SAMPLER_MIPMAP_MODE_NEAREST) {
             skip |= LogError("VUID-VkSamplerCreateInfo-unnormalizedCoordinates-01073", device,
                              create_info_loc.dot(Field::unnormalizedCoordinates),
                              "is VK_TRUE, but mipmapMode (%s) must be VK_SAMPLER_MIPMAP_MODE_NEAREST.",
-                             string_VkSamplerMipmapMode(pCreateInfo->mipmapMode));
+                             string_VkSamplerMipmapMode(create_info.mipmapMode));
         }
-        if (pCreateInfo->minLod != 0.0f || pCreateInfo->maxLod != 0.0f) {
-            skip |= LogError("VUID-VkSamplerCreateInfo-unnormalizedCoordinates-01074", device,
-                             create_info_loc.dot(Field::unnormalizedCoordinates),
-                             "is VK_TRUE, but minLod (%f) and maxLod (%f) must both be zero.", pCreateInfo->minLod,
-                             pCreateInfo->maxLod);
+        if (create_info.minLod != 0.0f || create_info.maxLod != 0.0f) {
+            skip |=
+                LogError("VUID-VkSamplerCreateInfo-unnormalizedCoordinates-01074", device,
+                         create_info_loc.dot(Field::unnormalizedCoordinates),
+                         "is VK_TRUE, but minLod (%f) and maxLod (%f) must both be zero.", create_info.minLod, create_info.maxLod);
         }
-        if ((pCreateInfo->addressModeU != VK_SAMPLER_ADDRESS_MODE_CLAMP_TO_EDGE &&
-             pCreateInfo->addressModeU != VK_SAMPLER_ADDRESS_MODE_CLAMP_TO_BORDER) ||
-            (pCreateInfo->addressModeV != VK_SAMPLER_ADDRESS_MODE_CLAMP_TO_EDGE &&
-             pCreateInfo->addressModeV != VK_SAMPLER_ADDRESS_MODE_CLAMP_TO_BORDER)) {
+        if ((create_info.addressModeU != VK_SAMPLER_ADDRESS_MODE_CLAMP_TO_EDGE &&
+             create_info.addressModeU != VK_SAMPLER_ADDRESS_MODE_CLAMP_TO_BORDER) ||
+            (create_info.addressModeV != VK_SAMPLER_ADDRESS_MODE_CLAMP_TO_EDGE &&
+             create_info.addressModeV != VK_SAMPLER_ADDRESS_MODE_CLAMP_TO_BORDER)) {
             skip |= LogError("VUID-VkSamplerCreateInfo-unnormalizedCoordinates-01075", device,
                              create_info_loc.dot(Field::unnormalizedCoordinates),
                              "is VK_TRUE, but addressModeU (%s) and addressModeV (%s) must both be "
                              "CLAMP_TO_EDGE or CLAMP_TO_BORDER.",
-                             string_VkSamplerAddressMode(pCreateInfo->addressModeU),
-                             string_VkSamplerAddressMode(pCreateInfo->addressModeV));
+                             string_VkSamplerAddressMode(create_info.addressModeU),
+                             string_VkSamplerAddressMode(create_info.addressModeV));
         }
-        if (pCreateInfo->anisotropyEnable == VK_TRUE) {
+        if (create_info.anisotropyEnable == VK_TRUE) {
             skip |= LogError("VUID-VkSamplerCreateInfo-unnormalizedCoordinates-01076", device, create_info_loc,
                              "anisotropyEnable and unnormalizedCoordinates are both VK_TRUE.");
         }
-        if (pCreateInfo->compareEnable == VK_TRUE) {
+        if (create_info.compareEnable == VK_TRUE) {
             skip |= LogError("VUID-VkSamplerCreateInfo-unnormalizedCoordinates-01077", device, create_info_loc,
                              "compareEnable and unnormalizedCoordinates are both VK_TRUE.");
         }
     }
 
-    if (pCreateInfo->compareEnable == VK_TRUE) {
-        skip |= context.ValidateRangedEnum(create_info_loc.dot(Field::compareOp), vvl::Enum::VkCompareOp, pCreateInfo->compareOp,
+    if (create_info.compareEnable == VK_TRUE) {
+        skip |= context.ValidateRangedEnum(create_info_loc.dot(Field::compareOp), vvl::Enum::VkCompareOp, create_info.compareOp,
                                            "VUID-VkSamplerCreateInfo-compareEnable-01080");
     }
 
-    if ((pCreateInfo->addressModeU == VK_SAMPLER_ADDRESS_MODE_CLAMP_TO_BORDER) ||
-        (pCreateInfo->addressModeV == VK_SAMPLER_ADDRESS_MODE_CLAMP_TO_BORDER) ||
-        (pCreateInfo->addressModeW == VK_SAMPLER_ADDRESS_MODE_CLAMP_TO_BORDER)) {
+    if ((create_info.addressModeU == VK_SAMPLER_ADDRESS_MODE_CLAMP_TO_BORDER) ||
+        (create_info.addressModeV == VK_SAMPLER_ADDRESS_MODE_CLAMP_TO_BORDER) ||
+        (create_info.addressModeW == VK_SAMPLER_ADDRESS_MODE_CLAMP_TO_BORDER)) {
         skip |= context.ValidateRangedEnum(create_info_loc.dot(Field::borderColor), vvl::Enum::VkBorderColor,
-                                           pCreateInfo->borderColor, "VUID-VkSamplerCreateInfo-addressModeU-01078");
+                                           create_info.borderColor, "VUID-VkSamplerCreateInfo-addressModeU-01078");
     }
 
     if (enabled_features.samplerMirrorClampToEdge == VK_FALSE) {
         // Only display at most one of address mode error otherwise it turns into spam
-        if (pCreateInfo->addressModeU == VK_SAMPLER_ADDRESS_MODE_MIRROR_CLAMP_TO_EDGE) {
+        if (create_info.addressModeU == VK_SAMPLER_ADDRESS_MODE_MIRROR_CLAMP_TO_EDGE) {
             skip |= LogError("VUID-VkSamplerCreateInfo-addressModeU-01079", device, create_info_loc.dot(Field::addressModeU),
                              "is VK_SAMPLER_ADDRESS_MODE_MIRROR_CLAMP_TO_EDGE but the "
                              "VK_KHR_sampler_mirror_clamp_to_edge extension or samplerMirrorClampToEdge feature was not enabled.");
-        } else if (pCreateInfo->addressModeV == VK_SAMPLER_ADDRESS_MODE_MIRROR_CLAMP_TO_EDGE) {
+        } else if (create_info.addressModeV == VK_SAMPLER_ADDRESS_MODE_MIRROR_CLAMP_TO_EDGE) {
             skip |= LogError("VUID-VkSamplerCreateInfo-addressModeU-01079", device, create_info_loc.dot(Field::addressModeV),
                              "is VK_SAMPLER_ADDRESS_MODE_MIRROR_CLAMP_TO_EDGE but the "
                              "VK_KHR_sampler_mirror_clamp_to_edge extension or samplerMirrorClampToEdge feature was not enabled.");
-        } else if (pCreateInfo->addressModeW == VK_SAMPLER_ADDRESS_MODE_MIRROR_CLAMP_TO_EDGE) {
+        } else if (create_info.addressModeW == VK_SAMPLER_ADDRESS_MODE_MIRROR_CLAMP_TO_EDGE) {
             skip |= LogError("VUID-VkSamplerCreateInfo-addressModeU-01079", device, create_info_loc.dot(Field::addressModeW),
                              "is VK_SAMPLER_ADDRESS_MODE_MIRROR_CLAMP_TO_EDGE but the "
                              "VK_KHR_sampler_mirror_clamp_to_edge extension or samplerMirrorClampToEdge feature was not enabled.");
@@ -425,47 +430,47 @@ bool Device::manual_PreCallValidateCreateSampler(VkDevice device, const VkSample
 
     // Checks for the IMG cubic filtering extension
     if (IsExtEnabled(extensions.vk_img_filter_cubic)) {
-        if ((pCreateInfo->anisotropyEnable == VK_TRUE) &&
-            ((pCreateInfo->minFilter == VK_FILTER_CUBIC_IMG) || (pCreateInfo->magFilter == VK_FILTER_CUBIC_IMG))) {
+        if ((create_info.anisotropyEnable == VK_TRUE) &&
+            ((create_info.minFilter == VK_FILTER_CUBIC_IMG) || (create_info.magFilter == VK_FILTER_CUBIC_IMG))) {
             skip |= LogError("VUID-VkSamplerCreateInfo-magFilter-01081", device, create_info_loc.dot(Field::anisotropyEnable),
                              "is VK_TRUE, but minFilter (%s) and magFilter (%s) can't be cubic.",
-                             string_VkFilter(pCreateInfo->minFilter), string_VkFilter(pCreateInfo->magFilter));
+                             string_VkFilter(create_info.minFilter), string_VkFilter(create_info.magFilter));
         }
     }
 
     // Check for valid Lod range
-    if (pCreateInfo->minLod > pCreateInfo->maxLod) {
+    if (create_info.minLod > create_info.maxLod) {
         skip |= LogError("VUID-VkSamplerCreateInfo-maxLod-01973", device, create_info_loc.dot(Field::minLod),
-                         "(%f) is greater than maxLod (%f)", pCreateInfo->minLod, pCreateInfo->maxLod);
+                         "(%f) is greater than maxLod (%f)", create_info.minLod, create_info.maxLod);
     }
 
     // Check mipLodBias to device limit
-    if (pCreateInfo->mipLodBias > limits.maxSamplerLodBias) {
+    if (create_info.mipLodBias > device_limits.maxSamplerLodBias) {
         skip |= LogError("VUID-VkSamplerCreateInfo-mipLodBias-01069", device, create_info_loc.dot(Field::mipLodBias),
-                         "(%f) is greater than maxSamplerLodBias (%f)", pCreateInfo->mipLodBias, limits.maxSamplerLodBias);
+                         "(%f) is greater than maxSamplerLodBias (%f)", create_info.mipLodBias, device_limits.maxSamplerLodBias);
     }
 
-    if (vku::FindStructInPNextChain<VkSamplerYcbcrConversionInfo>(pCreateInfo->pNext)) {
-        if ((pCreateInfo->addressModeU != VK_SAMPLER_ADDRESS_MODE_CLAMP_TO_EDGE) ||
-            (pCreateInfo->addressModeV != VK_SAMPLER_ADDRESS_MODE_CLAMP_TO_EDGE) ||
-            (pCreateInfo->addressModeW != VK_SAMPLER_ADDRESS_MODE_CLAMP_TO_EDGE)) {
+    if (vku::FindStructInPNextChain<VkSamplerYcbcrConversionInfo>(create_info.pNext)) {
+        if ((create_info.addressModeU != VK_SAMPLER_ADDRESS_MODE_CLAMP_TO_EDGE) ||
+            (create_info.addressModeV != VK_SAMPLER_ADDRESS_MODE_CLAMP_TO_EDGE) ||
+            (create_info.addressModeW != VK_SAMPLER_ADDRESS_MODE_CLAMP_TO_EDGE)) {
             skip |= LogError("VUID-VkSamplerCreateInfo-addressModeU-01646", device, create_info_loc.dot(Field::pNext),
                              "contains a VkSamplerYcbcrConversionInfo struct, but all address modes must be "
                              "VK_SAMPLER_ADDRESS_MODE_CLAMP_TO_EDGE:\n"
                              "addressModeU (%s)\naddressModeV (%s)\naddressModeW (%s)\n",
-                             string_VkSamplerAddressMode(pCreateInfo->addressModeU),
-                             string_VkSamplerAddressMode(pCreateInfo->addressModeV),
-                             string_VkSamplerAddressMode(pCreateInfo->addressModeW));
-        } else if (pCreateInfo->anisotropyEnable) {
+                             string_VkSamplerAddressMode(create_info.addressModeU),
+                             string_VkSamplerAddressMode(create_info.addressModeV),
+                             string_VkSamplerAddressMode(create_info.addressModeW));
+        } else if (create_info.anisotropyEnable) {
             skip |= LogError("VUID-VkSamplerCreateInfo-addressModeU-01646", device, create_info_loc.dot(Field::pNext),
                              "contains a VkSamplerYcbcrConversionInfo struct, but anisotropyEnable must be VK_FALSE.");
-        } else if (pCreateInfo->unnormalizedCoordinates) {
+        } else if (create_info.unnormalizedCoordinates) {
             skip |= LogError("VUID-VkSamplerCreateInfo-addressModeU-01646", device, create_info_loc.dot(Field::pNext),
                              "contains a VkSamplerYcbcrConversionInfo struct, but unnormalizedCoordinates must be VK_FALSE.");
         }
     }
 
-    if (vku::FindStructInPNextChain<VkSamplerBorderColorComponentMappingCreateInfoEXT>(pCreateInfo->pNext)) {
+    if (vku::FindStructInPNextChain<VkSamplerBorderColorComponentMappingCreateInfoEXT>(create_info.pNext)) {
         if (!enabled_features.borderColorSwizzle) {
             skip |=
                 LogError("VUID-VkSamplerBorderColorComponentMappingCreateInfoEXT-borderColorSwizzle-06437", device, create_info_loc,
@@ -474,35 +479,46 @@ bool Device::manual_PreCallValidateCreateSampler(VkDevice device, const VkSample
         }
     }
 
-    if ((pCreateInfo->flags & VK_SAMPLER_CREATE_NON_SEAMLESS_CUBE_MAP_BIT_EXT) && (!enabled_features.nonSeamlessCubeMap)) {
+    if ((create_info.flags & VK_SAMPLER_CREATE_NON_SEAMLESS_CUBE_MAP_BIT_EXT) && (!enabled_features.nonSeamlessCubeMap)) {
         skip |= LogError("VUID-VkSamplerCreateInfo-nonSeamlessCubeMap-06788", device, create_info_loc.dot(Field::flags),
                          "includes VK_SAMPLER_CREATE_NON_SEAMLESS_CUBE_MAP_BIT_EXT but the "
                          "nonSeamlessCubeMap feature was not enabled.");
     }
 
-    if ((pCreateInfo->flags & VK_SAMPLER_CREATE_DESCRIPTOR_BUFFER_CAPTURE_REPLAY_BIT_EXT) &&
+    if ((create_info.flags & VK_SAMPLER_CREATE_DESCRIPTOR_BUFFER_CAPTURE_REPLAY_BIT_EXT) &&
         !enabled_features.descriptorBufferCaptureReplay) {
         skip |= LogError("VUID-VkSamplerCreateInfo-flags-08110", device, create_info_loc.dot(Field::flags),
                          "includes VK_SAMPLER_CREATE_DESCRIPTOR_BUFFER_CAPTURE_REPLAY_BIT_EXT but descriptorBufferCaptureReplay "
                          "feature was not enabled.");
     }
 
-    if (vku::FindStructInPNextChain<VkOpaqueCaptureDescriptorDataCreateInfoEXT>(pCreateInfo->pNext)) {
-        if (!(pCreateInfo->flags & VK_SAMPLER_CREATE_DESCRIPTOR_BUFFER_CAPTURE_REPLAY_BIT_EXT)) {
+    if (vku::FindStructInPNextChain<VkOpaqueCaptureDescriptorDataCreateInfoEXT>(create_info.pNext)) {
+        if (!(create_info.flags & VK_SAMPLER_CREATE_DESCRIPTOR_BUFFER_CAPTURE_REPLAY_BIT_EXT)) {
             skip |= LogError("VUID-VkSamplerCreateInfo-pNext-08111", device, create_info_loc.dot(Field::flags),
-                             "is %s but VkOpaqueCaptureDescriptorDataCreateInfoEXT is in pNext chain.",
-                             string_VkSamplerCreateFlags(pCreateInfo->flags).c_str());
+                             "is %s but VkOpaqueCaptureDescriptorDataCreateInfoEXT is in pNext chain.\n%s",
+                             string_VkSamplerCreateFlags(create_info.flags).c_str(),
+                             PrintPNextChain(Struct::VkSamplerCreateInfo, create_info.pNext).c_str());
         }
     }
 
-    skip |= ValidateSamplerFilterMinMax(*pCreateInfo, create_info_loc);
-    skip |= ValidateSamplerCustomBoarderColor(*pCreateInfo, create_info_loc);
-    skip |= ValidateSamplerSubsampled(*pCreateInfo, create_info_loc);
-    skip |= ValidateSamplerImageProcessingQCOM(*pCreateInfo, create_info_loc);
+    skip |= ValidateSamplerFilterMinMax(create_info, create_info_loc);
+    skip |= ValidateSamplerCustomBorderColor(create_info, create_info_loc);
+    skip |= ValidateSamplerSubsampled(create_info, create_info_loc);
+    skip |= ValidateSamplerImageProcessingQCOM(create_info, create_info_loc);
 
     return skip;
 }
 
+bool Device::manual_PreCallValidateCreateSampler(VkDevice device, const VkSamplerCreateInfo *pCreateInfo,
+                                                 const VkAllocationCallbacks *pAllocator, VkSampler *pSampler,
+                                                 const Context &context) const {
+    bool skip = false;
+    const auto &error_obj = context.error_obj;
+    const Location create_info_loc = error_obj.location.dot(Field::pCreateInfo);
+    skip |= ValidateSamplerCreateInfo(*pCreateInfo, create_info_loc, context);
+    return skip;
+}
+
 bool Device::ValidateMutableDescriptorTypeCreateInfo(const VkDescriptorSetLayoutCreateInfo &create_info,
                                                      const VkMutableDescriptorTypeCreateInfoEXT &mutable_create_info,
                                                      const Location &create_info_loc) const {
@@ -555,6 +571,10 @@ bool Device::ValidateMutableDescriptorTypeCreateInfo(const VkDescriptorSetLayout
                     skip |= LogError("VUID-VkMutableDescriptorTypeListEXT-pDescriptorTypes-04603", device, type_loc,
                                      "is VK_DESCRIPTOR_TYPE_INLINE_UNIFORM_BLOCK.");
                     break;
+                case VK_DESCRIPTOR_TYPE_TENSOR_ARM:
+                    skip |= LogError("VUID-VkMutableDescriptorTypeListEXT-pDescriptorTypes-09696", device, type_loc,
+                                     "is VK_DESCRIPTOR_TYPE_TENSOR_ARM.");
+                    break;
                 default:
                     break;
             }
@@ -611,14 +631,16 @@ bool Device::ValidateDescriptorSetLayoutCreateInfo(const VkDescriptorSetLayoutCr
                         skip |= LogError(
                             "VUID-VkDescriptorSetLayoutCreateInfo-pBindings-07303", device,
                             binding_loc.pNext(Struct::VkMutableDescriptorTypeCreateInfoEXT, Field::mutableDescriptorTypeListCount),
-                            "(%" PRIu32 ") is less than %" PRIu32 " but descriptorType is VK_DESCRIPTOR_TYPE_MUTABLE_EXT", i,
-                            mutable_descriptor_type->mutableDescriptorTypeListCount);
+                            "(%" PRIu32 ") is less than or equal to %" PRIu32 ", but pBindings[%" PRIu32
+                            " ].descriptorType is VK_DESCRIPTOR_TYPE_MUTABLE_EXT",
+                            mutable_descriptor_type->mutableDescriptorTypeListCount, i, i);
                     }
                 } else {
                     skip |= LogError("VUID-VkDescriptorSetLayoutCreateInfo-pBindings-07303", device,
                                      binding_loc.dot(Field::descriptorType),
                                      "is VK_DESCRIPTOR_TYPE_MUTABLE_EXT but VkMutableDescriptorTypeCreateInfoEXT is not "
-                                     "included in the pNext chain.");
+                                     "included in the pNext chain.\n%s",
+                                     PrintPNextChain(Struct::VkDescriptorSetLayoutCreateInfo, create_info.pNext).c_str());
                 }
                 if (binding.pImmutableSamplers) {
                     skip |= LogError("VUID-VkDescriptorSetLayoutCreateInfo-descriptorType-04594", device,
@@ -1031,17 +1053,22 @@ bool Device::manual_PreCallValidateCreateSamplerYcbcrConversion(VkDevice device,
     }
 
     if (pCreateInfo->ycbcrModel != VK_SAMPLER_YCBCR_MODEL_CONVERSION_RGB_IDENTITY) {
-        // Checks same VU multiple ways in order to give a more useful error message
+        // This VU covers a lot, could have been multiple VUs, so provide a good error message for all cases.
         const char *vuid = "VUID-VkSamplerYcbcrConversionCreateInfo-ycbcrModel-01655";
-        if ((components.r == VK_COMPONENT_SWIZZLE_ONE) || (components.r == VK_COMPONENT_SWIZZLE_ZERO) ||
-            (components.g == VK_COMPONENT_SWIZZLE_ONE) || (components.g == VK_COMPONENT_SWIZZLE_ZERO) ||
-            (components.b == VK_COMPONENT_SWIZZLE_ONE) || (components.b == VK_COMPONENT_SWIZZLE_ZERO)) {
-            skip |=
-                LogError(vuid, device, create_info_loc,
-                         "The ycbcrModel (%s) is not VK_SAMPLER_YCBCR_MODEL_CONVERSION_RGB_IDENTITY so components.r (%s), "
-                         "components.g (%s), nor components.b (%s) can't be VK_COMPONENT_SWIZZLE_ZERO or VK_COMPONENT_SWIZZLE_ONE.",
-                         string_VkSamplerYcbcrModelConversion(pCreateInfo->ycbcrModel), string_VkComponentSwizzle(components.r),
-                         string_VkComponentSwizzle(components.g), string_VkComponentSwizzle(components.b));
+        if (components.r == VK_COMPONENT_SWIZZLE_ZERO || components.g == VK_COMPONENT_SWIZZLE_ZERO ||
+            components.b == VK_COMPONENT_SWIZZLE_ZERO) {
+            skip |= LogError(vuid, device, create_info_loc,
+                             "The ycbcrModel (%s) is not VK_SAMPLER_YCBCR_MODEL_CONVERSION_RGB_IDENTITY so the r, g, and b in "
+                             "pCreateInfo->components can't all be VK_COMPONENT_SWIZZLE_ZERO\n%s",
+                             string_VkSamplerYcbcrModelConversion(pCreateInfo->ycbcrModel),
+                             string_VkComponentMapping(components).c_str());
+        } else if (components.r == VK_COMPONENT_SWIZZLE_ONE || components.g == VK_COMPONENT_SWIZZLE_ONE ||
+                   components.b == VK_COMPONENT_SWIZZLE_ONE) {
+            skip |= LogError(vuid, device, create_info_loc,
+                             "The ycbcrModel (%s) is not VK_SAMPLER_YCBCR_MODEL_CONVERSION_RGB_IDENTITY so the r, g, and b in "
+                             "pCreateInfo->components can't all be VK_COMPONENT_SWIZZLE_ONE\n%s",
+                             string_VkSamplerYcbcrModelConversion(pCreateInfo->ycbcrModel),
+                             string_VkComponentMapping(components).c_str());
         }
 
         // "must not correspond to a component which contains zero or one as a consequence of conversion to RGBA"
@@ -1052,31 +1079,50 @@ bool Device::manual_PreCallValidateCreateSamplerYcbcrConversion(VkDevice device,
         // depth/stencil = no [g,b,a] (shouldn't ever occur, but no VU preventing it)
         const uint32_t component_count = (vkuFormatIsDepthOrStencil(format) == true) ? 1 : vkuFormatComponentCount(format);
 
-        if ((component_count < 4) && ((components.r == VK_COMPONENT_SWIZZLE_A) || (components.g == VK_COMPONENT_SWIZZLE_A) ||
-                                      (components.b == VK_COMPONENT_SWIZZLE_A))) {
+        if (format == VK_FORMAT_UNDEFINED) {
+            // If using external format they should have been caught in GetExternalFormat above. This means the user forgot to pass
+            // in the VkExternalFormatANDROID and will get an error below which will not be obvious what is going on.
             skip |= LogError(vuid, device, create_info_loc,
-                             "The ycbcrModel (%s) is not VK_SAMPLER_YCBCR_MODEL_CONVERSION_RGB_IDENTITY so components.r (%s), "
-                             "components.g (%s), or components.b (%s) can't be VK_COMPONENT_SWIZZLE_A.",
-                             string_VkSamplerYcbcrModelConversion(pCreateInfo->ycbcrModel), string_VkComponentSwizzle(components.r),
-                             string_VkComponentSwizzle(components.g), string_VkComponentSwizzle(components.b));
+                             "The ycbcrModel (%s) is not VK_SAMPLER_YCBCR_MODEL_CONVERSION_RGB_IDENTITY and pCreateInfo->format is "
+                             "VK_FORMAT_UNDEFINED. Either you forgot to set the format here or if you are trying to use an "
+                             "external format, and forgot to pass in VkExternalFormatANDROID (or equivalent) into the "
+                             "pCreateInfo->pNext chain as this check doesn't apply for if externalFormat is non-zero.",
+                             string_VkSamplerYcbcrModelConversion(pCreateInfo->ycbcrModel));
+
+        } else if ((component_count < 4) && ((components.r == VK_COMPONENT_SWIZZLE_A) || (components.g == VK_COMPONENT_SWIZZLE_A) ||
+                                             (components.b == VK_COMPONENT_SWIZZLE_A))) {
+            skip |= LogError(vuid, device, create_info_loc,
+                             "The ycbcrModel (%s) is not VK_SAMPLER_YCBCR_MODEL_CONVERSION_RGB_IDENTITY so the r, g, and b in "
+                             "pCreateInfo->components can't all be VK_COMPONENT_SWIZZLE_A because %s only has %" PRIu32
+                             " components (not 4) so the 'a' component is invalid\n%s",
+                             string_VkSamplerYcbcrModelConversion(pCreateInfo->ycbcrModel), string_VkFormat(format),
+                             component_count, string_VkComponentMapping(components).c_str());
         } else if ((component_count < 3) &&
                    ((components.r == VK_COMPONENT_SWIZZLE_B) || (components.g == VK_COMPONENT_SWIZZLE_B) ||
                     (components.b == VK_COMPONENT_SWIZZLE_B) || (components.b == VK_COMPONENT_SWIZZLE_IDENTITY))) {
-            skip |= LogError(vuid, device, create_info_loc,
-                             "The ycbcrModel (%s) is not VK_SAMPLER_YCBCR_MODEL_CONVERSION_RGB_IDENTITY so components.r (%s), "
-                             "components.g (%s), or components.b (%s) can't be VK_COMPONENT_SWIZZLE_B "
-                             "(components.b also can't be VK_COMPONENT_SWIZZLE_IDENTITY).",
-                             string_VkSamplerYcbcrModelConversion(pCreateInfo->ycbcrModel), string_VkComponentSwizzle(components.r),
-                             string_VkComponentSwizzle(components.g), string_VkComponentSwizzle(components.b));
+            skip |= LogError(
+                vuid, device, create_info_loc,
+                "The ycbcrModel (%s) is not VK_SAMPLER_YCBCR_MODEL_CONVERSION_RGB_IDENTITY so the r, g, and b in "
+                "pCreateInfo->components can't all be VK_COMPONENT_SWIZZLE_B because %s only has %" PRIu32
+                " components (not 3 or 4) so the 'b' component is invalid\n%s%s",
+                string_VkSamplerYcbcrModelConversion(pCreateInfo->ycbcrModel), string_VkFormat(format), component_count,
+                string_VkComponentMapping(components).c_str(),
+                (components.b == VK_COMPONENT_SWIZZLE_IDENTITY)
+                    ? "\n(components.b also can't be VK_COMPONENT_SWIZZLE_IDENTITY as the is equivalent to VK_COMPONENT_SWIZZLE_B)"
+                    : "");
         } else if ((component_count < 2) &&
                    ((components.r == VK_COMPONENT_SWIZZLE_G) || (components.g == VK_COMPONENT_SWIZZLE_G) ||
                     (components.g == VK_COMPONENT_SWIZZLE_IDENTITY) || (components.b == VK_COMPONENT_SWIZZLE_G))) {
-            skip |= LogError(vuid, device, create_info_loc,
-                             "The ycbcrModel (%s) is not VK_SAMPLER_YCBCR_MODEL_CONVERSION_RGB_IDENTITY so components.r (%s), "
-                             "components.g (%s), or components.b (%s) can't be VK_COMPONENT_SWIZZLE_G "
-                             "(components.g also can't be VK_COMPONENT_SWIZZLE_IDENTITY).",
-                             string_VkSamplerYcbcrModelConversion(pCreateInfo->ycbcrModel), string_VkComponentSwizzle(components.r),
-                             string_VkComponentSwizzle(components.g), string_VkComponentSwizzle(components.b));
+            skip |= LogError(
+                vuid, device, create_info_loc,
+                "The ycbcrModel (%s) is not VK_SAMPLER_YCBCR_MODEL_CONVERSION_RGB_IDENTITY so the r, g, and b in "
+                "pCreateInfo->components can't all be VK_COMPONENT_SWIZZLE_G because %s only has %" PRIu32
+                " components (not 2, 3 or 4) so the 'g' component is invalid\n%s%s",
+                string_VkSamplerYcbcrModelConversion(pCreateInfo->ycbcrModel), string_VkFormat(format), component_count,
+                string_VkComponentMapping(components).c_str(),
+                (components.g == VK_COMPONENT_SWIZZLE_IDENTITY)
+                    ? "\n(components.g also can't be VK_COMPONENT_SWIZZLE_IDENTITY as the is equivalent to VK_COMPONENT_SWIZZLE_G)"
+                    : "");
         }
     }
 
@@ -1103,6 +1149,12 @@ bool Device::manual_PreCallValidateGetDescriptorEXT(VkDevice device, const VkDes
             skip |= LogError("VUID-VkDescriptorGetInfoEXT-type-08018", device, descriptor_info_loc.dot(Field::type), "is %s.",
                              string_VkDescriptorType(pDescriptorInfo->type));
             break;
+        case VK_DESCRIPTOR_TYPE_SAMPLER:
+            if (!pDescriptorInfo->data.pSampler) {
+                skip |= LogError("VUID-VkDescriptorGetInfoEXT-pSampler-parameter", device, descriptor_info_loc.dot(Field::type),
+                                 "is VK_DESCRIPTOR_TYPE_SAMPLER, but pSampler is null.");
+            }
+            break;
         case VK_DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER:
             if (!pDescriptorInfo->data.pCombinedImageSampler) {
                 skip |= LogError("VUID-VkDescriptorGetInfoEXT-pCombinedImageSampler-parameter", device,
@@ -1158,14 +1210,74 @@ bool Device::manual_PreCallValidateGetDescriptorEXT(VkDevice device, const VkDes
             }
         }
     }
+
+    const auto *tensor_struct = vku::FindStructInPNextChain<VkDescriptorGetTensorInfoARM>(pDescriptorInfo->pNext);
+    if (tensor_struct) {
+        if (!enabled_features.nullDescriptor && tensor_struct->tensorView == VK_NULL_HANDLE) {
+            skip |= LogError("VUID-VkDescriptorGetTensorInfoARM-nullDescriptor-09899", device,
+                             error_obj.location.dot(Field::pNext).dot(Field::tensorView),
+                             "is VK_NULL_HANDLE and the nullDescriptor feature is not enabled.");
+        }
+    }
     return skip;
 }
 
+bool Device::ValidateCmdSetDescriptorBufferOffsets(VkCommandBuffer commandBuffer, VkPipelineLayout layout, uint32_t setCount,
+                                                   const uint32_t *pBufferIndices, const VkDeviceSize *pOffsets,
+                                                   const Location &loc) const {
+    bool skip = false;
+    const bool is_2 = loc.function != Func::vkCmdSetDescriptorBufferOffsetsEXT;
+
+    if (!enabled_features.descriptorBuffer) {
+        const char *vuid = is_2 ? "VUID-vkCmdSetDescriptorBufferOffsets2EXT-descriptorBuffer-09470"
+                                : "VUID-vkCmdSetDescriptorBufferOffsetsEXT-None-08060";
+        skip |= LogError(vuid, commandBuffer, loc, "descriptorBuffer feature was not enabled.");
+    }
+
+    for (uint32_t i = 0; i < setCount; i++) {
+        const uint32_t buffer_index = pBufferIndices[i];
+        if (buffer_index >= phys_dev_ext_props.descriptor_buffer_props.maxDescriptorBufferBindings) {
+            const char *vuid = is_2 ? "VUID-VkSetDescriptorBufferOffsetsInfoEXT-pBufferIndices-08064"
+                                    : "VUID-vkCmdSetDescriptorBufferOffsetsEXT-pBufferIndices-08064";
+            const LogObjectList objlist(commandBuffer, layout);
+            skip |= LogError(vuid, objlist, loc.dot(Field::pBufferIndices, i),
+                             "(%" PRIu32 ") is greater than maxDescriptorBufferBindings (%" PRIu32 ") ", buffer_index,
+                             phys_dev_ext_props.descriptor_buffer_props.maxDescriptorBufferBindings);
+        }
+
+        const VkDeviceAddress offset = pOffsets[i];
+        if (SafeModulo(offset, phys_dev_ext_props.descriptor_buffer_props.descriptorBufferOffsetAlignment) != 0) {
+            const char *vuid = is_2 ? "VUID-VkSetDescriptorBufferOffsetsInfoEXT-pOffsets-08061"
+                                    : "VUID-vkCmdSetDescriptorBufferOffsetsEXT-pOffsets-08061";
+            const LogObjectList objlist(commandBuffer, layout);
+            skip |= LogError(vuid, objlist, loc.dot(Field::pOffsets, i),
+                             "(%" PRIuLEAST64
+                             ") is not aligned to descriptorBufferOffsetAlignment"
+                             " (%" PRIuLEAST64 ")",
+                             offset, phys_dev_ext_props.descriptor_buffer_props.descriptorBufferOffsetAlignment);
+        }
+    }
+
+    return skip;
+}
+
+bool Device::manual_PreCallValidateCmdSetDescriptorBufferOffsetsEXT(VkCommandBuffer commandBuffer,
+                                                                    VkPipelineBindPoint pipelineBindPoint, VkPipelineLayout layout,
+                                                                    uint32_t firstSet, uint32_t setCount,
+                                                                    const uint32_t *pBufferIndices, const VkDeviceSize *pOffsets,
+                                                                    const Context &context) const {
+    return ValidateCmdSetDescriptorBufferOffsets(commandBuffer, layout, setCount, pBufferIndices, pOffsets,
+                                                 context.error_obj.location);
+}
+
 bool Device::manual_PreCallValidateCmdSetDescriptorBufferOffsets2EXT(
     VkCommandBuffer commandBuffer, const VkSetDescriptorBufferOffsetsInfoEXT *pSetDescriptorBufferOffsetsInfo,
     const Context &context) const {
     bool skip = false;
     const auto &error_obj = context.error_obj;
+    skip |= ValidateCmdSetDescriptorBufferOffsets(
+        commandBuffer, pSetDescriptorBufferOffsetsInfo->layout, pSetDescriptorBufferOffsetsInfo->setCount,
+        pSetDescriptorBufferOffsetsInfo->pBufferIndices, pSetDescriptorBufferOffsetsInfo->pOffsets, error_obj.location);
 
     if (pSetDescriptorBufferOffsetsInfo->layout == VK_NULL_HANDLE) {
         if (!enabled_features.dynamicPipelineLayout) {
