@@ -42,10 +42,13 @@ namespace stateless {
 
 class SpirvValidator : public Logger {
   public:
-    SpirvValidator(DebugReport* debug_report, const vvl::StatelessDeviceData& stateless_device_data);
+    SpirvValidator(DebugReport* debug_report, const vvl::StatelessDeviceData& stateless_device_data, bool disabled);
 
     bool Validate(const spirv::Module& module_state, const spirv::StatelessData& stateless_data, const Location& loc) const;
 
+    // When we create the SpirvValidator, we will see if the runtime setting to disable it was set or not
+    const bool disabled;
+
     const APIVersion& api_version;
     const DeviceExtensions& extensions;
     const VkPhysicalDeviceProperties& phys_dev_props;
@@ -62,6 +65,7 @@ class SpirvValidator : public Logger {
                              const Location& loc) const;
     bool ValidateAtomicsTypes(const spirv::Module& module_state, const spirv::StatelessData& stateless_data,
                               const Location& loc) const;
+    bool ValidateFma(const spirv::Module& module_state, const spirv::StatelessData& stateless_data, const Location& loc) const;
     bool ValidateVariables(const spirv::Module& module_state, const Location& loc) const;
     bool Validate8And16BitStorage(const spirv::Module& module_state, const spirv::Instruction& var_refsn,
                                   const Location& loc) const;
@@ -86,6 +90,7 @@ class SpirvValidator : public Logger {
                                 const spirv::StatelessData& stateless_data, const Location& loc) const;
     bool ValidateConservativeRasterization(const spirv::Module& module_state, const spirv::EntryPoint& entrypoint,
                                            const spirv::StatelessData& stateless_data, const Location& loc) const;
+    bool ValidateShaderTensor(const spirv::Module &module_state, const spirv::EntryPoint &entrypoint, const spirv::StatelessData &stateless_data, const Location& loc) const;
 
     // Auto-generated helper functions
     bool ValidateShaderCapabilitiesAndExtensions(const spirv::Module& module_state, const spirv::Instruction& insn,
