@@ -17,15 +17,19 @@
 
 #pragma once
 #include "sync/sync_common.h"
+#include "sync/sync_barrier.h"
+#include "containers/span.h"
+#include <cstring>  // memcpy
+#include <optional>
 
-class ResourceAccessState;
+namespace syncval {
+
+class AccessState;
 struct WriteState;
 struct ReadState;
-struct ResourceFirstAccess;
-
-namespace syncval_stats {
+struct FirstAccess;
+struct SemaphoreScope;
 struct AccessContextStats;
-}  // namespace syncval_stats
 
 // NOTE: the attachement read flag is put *only* in the access scope and not in the exect scope, since the ordering
 //       rules apply only to this specific access for this stage, and not the stage as a whole. The ordering detection
@@ -96,23 +100,23 @@ SyncHazardInfo GetSyncHazardInfo(SyncHazard hazard);
 class HazardResult {
   public:
     struct HazardState {
-        std::unique_ptr<const ResourceAccessState> access_state;
-        std::unique_ptr<const ResourceFirstAccess> recorded_access;
+        std::unique_ptr<const AccessState> access_state;
+        std::unique_ptr<const FirstAccess> recorded_access;
         SyncAccessIndex access_index = std::numeric_limits<SyncAccessIndex>::max();
         SyncAccessIndex prior_access_index;
         ResourceUsageTag tag = ResourceUsageTag();
         uint32_t handle_index = vvl::kNoIndex32;
         SyncHazard hazard = NONE;
-        HazardState(const ResourceAccessState *access_state, const SyncAccessInfo &usage_info, SyncHazard hazard,
+        HazardState(const AccessState *access_state, const SyncAccessInfo &usage_info, SyncHazard hazard,
                     SyncAccessIndex prior_access_index, ResourceUsageTagEx tag_ex);
     };
 
-    static HazardResult HazardVsPriorWrite(const ResourceAccessState *access_state, const SyncAccessInfo &usage_info,
-                                           SyncHazard hazard, const WriteState &prior_write);
-    static HazardResult HazardVsPriorRead(const ResourceAccessState *access_state, const SyncAccessInfo &usage_info,
-                                          SyncHazard hazard, const ReadState &prior_read);
+    static HazardResult HazardVsPriorWrite(const AccessState *access_state, const SyncAccessInfo &usage_info, SyncHazard hazard,
+                                           const WriteState &prior_write);
+    static HazardResult HazardVsPriorRead(const AccessState *access_state, const SyncAccessInfo &usage_info, SyncHazard hazard,
+                                          const ReadState &prior_read);
 
-    void AddRecordedAccess(const ResourceFirstAccess &first_access);
+    void AddRecordedAccess(const FirstAccess &first_access);
 
     bool IsHazard() const { return state_.has_value() && NONE != state_->hazard; }
     bool IsWAWHazard() const;
@@ -128,7 +132,7 @@ class HazardResult {
         assert(state_);
         return state_->hazard;
     }
-    const std::unique_ptr<const ResourceFirstAccess> &RecordedAccess() const {
+    const std::unique_ptr<const FirstAccess> &RecordedAccess() const {
         assert(state_);
         return state_->recorded_access;
     }
@@ -141,89 +145,29 @@ class HazardResult {
     std::optional<HazardState> state_;
 };
 
-struct SyncExecScope {
-    VkPipelineStageFlags2 mask_param = 0;  // the xxxStageMask parameter passed by the caller
-    VkPipelineStageFlags2 exec_scope = 0;  // all earlier or later stages that would be affected by a barrier using this scope.
-    SyncAccessFlags valid_accesses;        // all valid accesses that can be used with this scope.
-
-    static SyncExecScope MakeSrc(VkQueueFlags queue_flags, VkPipelineStageFlags2 src_stage_mask,
-                                 const VkPipelineStageFlags2 disabled_feature_mask = 0);
-    static SyncExecScope MakeDst(VkQueueFlags queue_flags, VkPipelineStageFlags2 src_stage_mask);
-};
-
-struct SemaphoreScope : SyncExecScope {
-    SemaphoreScope(QueueId qid, const SyncExecScope &exec_scope) : SyncExecScope(exec_scope), queue(qid) {}
-    SemaphoreScope() = default;
-    QueueId queue;
-};
-
-struct SyncBarrier {
-    struct AllAccess {};
-    SyncExecScope src_exec_scope;
-    SyncAccessFlags src_access_scope;
-    SyncExecScope dst_exec_scope;
-    SyncAccessFlags dst_access_scope;
-
-    SyncBarrier() = default;
-    SyncBarrier(const SyncExecScope &src_exec, const SyncExecScope &dst_exec);
-    SyncBarrier(const SyncExecScope &src_exec, const SyncExecScope &dst_exec, const AllAccess &);
-    SyncBarrier(const SyncExecScope &src_exec, VkAccessFlags2 src_access_mask, const SyncExecScope &dst_exec,
-                VkAccessFlags2 dst_access_mask);
-    SyncBarrier(VkQueueFlags queue_flags, const VkSubpassDependency2 &barrier);
-    SyncBarrier(const std::vector<SyncBarrier> &barriers);
-};
-
-// Defines the source scope of the barrier.
-// ReadState and WriteState have InBarrierSourceScope() that checks if corresponding access is in the barrier source scope.
-struct BarrierScope {
-    VkPipelineStageFlagBits2 src_exec_scope;
-    SyncAccessFlags src_access_scope;
-
-    // Scope queue is used to include accesses only from the specific queue.
-    // The check against queue scope is unified for all cases. During record time the scope queue
-    // has default value (Invalid). This matches how the queue member of read/write accesses is
-    // initialized during recording, so (access_queue == scope_queue) evaluates to true during record time.
-    QueueId scope_queue = kQueueIdInvalid;
-
-    // The tag is needed for the event scope logic. The scope tag is defined by the "set event" command.
-    // The check against scope tag is unified for all cases. For non event code the scope tag is uint64-max
-    // value, so (access_tag < scope_tag) evaluates to true for non event code.
-    ResourceUsageTag scope_tag = kInvalidTag;
-
-    BarrierScope(const SyncBarrier &barrier, QueueId scope_queue = kQueueIdInvalid, ResourceUsageTag scope_tag = kInvalidTag);
-};
-
-struct ResourceFirstAccess {
+struct FirstAccess {
     const SyncAccessInfo *usage_info;
     ResourceUsageTag tag;
     uint32_t handle_index;
     SyncOrdering ordering_rule;
     SyncFlags flags;
 
-    ResourceFirstAccess(const SyncAccessInfo &usage_info, ResourceUsageTagEx tag_ex, SyncOrdering ordering_rule, SyncFlags flags)
+    FirstAccess(const SyncAccessInfo &usage_info, ResourceUsageTagEx tag_ex, SyncOrdering ordering_rule, SyncFlags flags)
         : usage_info(&usage_info), tag(tag_ex.tag), handle_index(tag_ex.handle_index), ordering_rule(ordering_rule), flags(flags) {}
-    bool operator==(const ResourceFirstAccess &rhs) const {
+    bool operator==(const FirstAccess &rhs) const {
         return (tag == rhs.tag) && (usage_info == rhs.usage_info) && (ordering_rule == rhs.ordering_rule) && flags == rhs.flags;
     }
     ResourceUsageTagEx TagEx() const { return {tag, handle_index}; }
 };
 
 using QueueId = uint32_t;
+
 struct OrderingBarrier {
-    VkPipelineStageFlags2 exec_scope;
+    VkPipelineStageFlags2 exec_scope = VK_PIPELINE_STAGE_2_NONE;
     SyncAccessFlags access_scope;
-    OrderingBarrier() = default;
-    OrderingBarrier(const OrderingBarrier &) = default;
-    OrderingBarrier(VkPipelineStageFlags2 es, SyncAccessFlags as) : exec_scope(es), access_scope(as) {}
-    OrderingBarrier &operator=(const OrderingBarrier &) = default;
-    OrderingBarrier &operator|=(const OrderingBarrier &rhs) {
-        exec_scope |= rhs.exec_scope;
-        access_scope |= rhs.access_scope;
-        return *this;
-    }
-    bool operator==(const OrderingBarrier &rhs) const {
-        return (exec_scope == rhs.exec_scope) && (access_scope == rhs.access_scope);
-    }
+
+    bool operator==(const OrderingBarrier &rhs) const;
+    size_t Hash() const;
 };
 
 const OrderingBarrier &GetOrderingRules(SyncOrdering ordering_enum);
@@ -250,19 +194,9 @@ struct ReadState {
         return (stage == rhs.stage) && (access_index == rhs.access_index) && (barriers == rhs.barriers) &&
                (sync_stages == rhs.sync_stages) && (tag == rhs.tag) && (queue == rhs.queue);
     }
-    bool IsReadBarrierHazard(VkPipelineStageFlags2 src_exec_scope) const {
-        // If the read stage is not in the src sync scope
-        // *AND* not execution chained with an existing sync barrier (that's the or)
-        // then the barrier access is unsafe (R/W after R)
-        return (src_exec_scope & (stage | barriers)) == 0;
-    }
+
     bool IsReadBarrierHazard(QueueId barrier_queue, VkPipelineStageFlags2 src_exec_scope,
                              const SyncAccessFlags &src_access_scope) const {
-        // If the read stage is not in the src sync scope
-        // *AND* not execution chained with an existing sync barrier (that's the or)
-        // then the barrier access is unsafe (R/W after R)
-        VkPipelineStageFlags2 queue_ordered_stage = (queue == barrier_queue) ? stage : VK_PIPELINE_STAGE_2_NONE;
-
         // Current implementation relies on TOP_OF_PIPE constant due to the fact that it's non-zero value
         // and AND-ing with it can create execution dependency when it's necessary. When NONE constant is
         // used, which equals to zero, then AND-ing with it always results in 0 which means "no barrier",
@@ -276,6 +210,10 @@ struct ReadState {
             src_exec_scope = VK_PIPELINE_STAGE_2_TOP_OF_PIPE_BIT;
         }
 
+        // If the read stage is not in the src sync scope
+        // *AND* not execution chained with an existing sync barrier (that's the or)
+        // then the barrier access is unsafe (R/W after R)
+        VkPipelineStageFlags2 queue_ordered_stage = (queue == barrier_queue) ? stage : VK_PIPELINE_STAGE_2_NONE;
         return (src_exec_scope & (queue_ordered_stage | barriers)) == 0;
     }
     bool ReadOrDependencyChainInSourceScope(QueueId queue, VkPipelineStageFlags2 src_exec_scope) const;
@@ -316,6 +254,7 @@ struct WriteState {
 
     bool IsOrdered(const OrderingBarrier &ordering, QueueId queue_id) const;
     bool IsLoadOp() const { return flags & SyncFlag::kLoadOp; }
+    bool IsStoreOp() const { return flags & SyncFlag::kStoreOp; }
     bool IsPresent() const { return flags & SyncFlag::kPresent; }
 
     ResourceUsageTagEx TagEx() const { return {tag, handle_index}; }
@@ -328,12 +267,12 @@ enum class PendingBarrierType : uint8_t { ReadAccessBarrier, WriteAccessBarrier,
 struct PendingBarrierInfo {
     PendingBarrierType type;
     uint32_t index;  // indexes array determined by 'type'
-    ResourceAccessState *access_state;
+    AccessState *access_state;
 };
 
 struct PendingReadBarrier {
     VkPipelineStageFlags2 barriers;
-    uint32_t last_reads_index;  // indexes ResourceAccessState::last_reads
+    uint32_t last_reads_index;  // indexes AccessState::last_reads
 };
 
 struct PendingWriteBarrier {
@@ -361,31 +300,43 @@ struct PendingBarriers {
     std::vector<PendingLayoutTransition> layout_transitions;
 
     // Store result of barrier application as PendingBarriers state
-    void AddReadBarrier(ResourceAccessState *access_state, uint32_t last_reads_index, const SyncBarrier &barrier);
-    void AddWriteBarrier(ResourceAccessState *access_state, const SyncBarrier &barrier);
-    void AddLayoutTransition(ResourceAccessState *access_state, const SyncBarrier &barrier,
+    void AddReadBarrier(AccessState *access_state, uint32_t last_reads_index, const SyncBarrier &barrier);
+    void AddWriteBarrier(AccessState *access_state, const SyncBarrier &barrier);
+    void AddLayoutTransition(AccessState *access_state, const OrderingBarrier &layout_transition_ordering_barrier,
                              uint32_t layout_transition_handle_index);
 
     // Update accesss state with collected barriers
     void Apply(const ResourceUsageTag exec_tag);
 };
 
-class ResourceAccessState {
+class AccessState {
   public:
+    ~AccessState();
+    AccessState(const AccessState &other);
+    AccessState(AccessState &&other);
+
+    // Use explicit default construction and assignment to control all places where this happens
+    static AccessState DefaultAccessState() { return {}; }
+    void Assign(const AccessState &other);
+    void Assign(AccessState &&other);
+    AccessState &operator=(const AccessState &other) = delete;
+    AccessState &operator=(AccessState &&other) = delete;
+
     HazardResult DetectHazard(const SyncAccessInfo &usage_info) const;
     HazardResult DetectMarkerHazard() const;
 
-    HazardResult DetectHazard(const SyncAccessInfo &usage_info, const OrderingBarrier &ordering, SyncFlags flags,
-                              QueueId queue_id) const;
-    HazardResult DetectHazard(const ResourceAccessState &recorded_use, QueueId queue_id, const ResourceUsageRange &tag_range) const;
+    HazardResult DetectHazard(const SyncAccessInfo &usage_info, const OrderingBarrier &ordering, SyncFlags flags, QueueId queue_id,
+                              bool detect_load_op_after_store_op_hazards) const;
+    HazardResult DetectHazard(const AccessState &recorded_use, QueueId queue_id, const ResourceUsageRange &tag_range,
+                              bool detect_load_op_after_store_op_hazards) const;
 
     HazardResult DetectAsyncHazard(const SyncAccessInfo &usage_info, ResourceUsageTag start_tag, QueueId queue_id) const;
-    HazardResult DetectAsyncHazard(const ResourceAccessState &recorded_use, const ResourceUsageRange &tag_range,
-                                   ResourceUsageTag start_tag, QueueId queue_id) const;
+    HazardResult DetectAsyncHazard(const AccessState &recorded_use, const ResourceUsageRange &tag_range, ResourceUsageTag start_tag,
+                                   QueueId queue_id) const;
 
     HazardResult DetectBarrierHazard(const SyncAccessInfo &usage_info, QueueId queue_id, VkPipelineStageFlags2 source_exec_scope,
                                      const SyncAccessFlags &source_access_scope) const;
-    HazardResult DetectBarrierHazard(const SyncAccessInfo &usage_info, const ResourceAccessState &scope_state,
+    HazardResult DetectBarrierHazard(const SyncAccessInfo &usage_info, const AccessState &scope_state,
                                      VkPipelineStageFlags2 source_exec_scope, const SyncAccessFlags &source_access_scope,
                                      QueueId event_queue, ResourceUsageTag event_tag) const;
 
@@ -394,7 +345,7 @@ class ResourceAccessState {
     void ClearWrite();
     void ClearRead();
     void ClearFirstUse();
-    void Resolve(const ResourceAccessState &other);
+    void Resolve(const AccessState &other);
 
     // Apply a single barrier to the access state
     void ApplyBarrier(const BarrierScope &barrier_scope, const SyncBarrier &barrier, bool layout_transition = false,
@@ -413,20 +364,20 @@ class ResourceAccessState {
     void ApplyPendingWriteBarrier(const PendingWriteBarrier &write_barrier);
     void ApplyPendingLayoutTransition(const PendingLayoutTransition &layout_transition, ResourceUsageTag layout_transition_tag);
 
-    void ApplySemaphore(const SemaphoreScope &signal, const SemaphoreScope wait);
+    void ApplySemaphore(const SemaphoreScope &signal, const SemaphoreScope &wait);
 
     struct WaitQueueTagPredicate {
         QueueId queue;
         ResourceUsageTag tag;
         bool operator()(const ReadState &read_access) const;       // Read access predicate
-        bool operator()(const ResourceAccessState &access) const;  // Write access predicate
+        bool operator()(const AccessState &access) const;  // Write access predicate
     };
     friend WaitQueueTagPredicate;
 
     struct WaitTagPredicate {
         ResourceUsageTag tag;
         bool operator()(const ReadState &read_access) const;       // Read access predicate
-        bool operator()(const ResourceAccessState &access) const;  // Write access predicate
+        bool operator()(const AccessState &access) const;  // Write access predicate
     };
     friend WaitTagPredicate;
 
@@ -434,7 +385,7 @@ class ResourceAccessState {
         ResourceUsageTag present_tag;
         ResourceUsageTag acquire_tag;
         bool operator()(const ReadState &read_access) const;       // Read access predicate
-        bool operator()(const ResourceAccessState &access) const;  // Write access predicate
+        bool operator()(const AccessState &access) const;  // Write access predicate
     };
     friend WaitAcquirePredicate;
 
@@ -444,10 +395,10 @@ class ResourceAccessState {
     template <typename Predicate>
     bool ClearPredicatedAccesses(Predicate &predicate);
 
+    ResourceUsageRange GetFirstAccessRange() const;
     bool FirstAccessInTagRange(const ResourceUsageRange &tag_range) const;
 
     void OffsetTag(ResourceUsageTag offset);
-    ResourceAccessState();
 
     bool HasWriteOp() const { return last_write.has_value(); }
     bool IsLastWriteOp(SyncAccessIndex access_index) const {
@@ -455,19 +406,26 @@ class ResourceAccessState {
     }
     ResourceUsageTag LastWriteTag() const { return last_write.has_value() ? last_write->tag : ResourceUsageTag(0); }
     const WriteState &LastWrite() const;
-    bool operator==(const ResourceAccessState &rhs) const {
+    bool operator==(const AccessState &rhs) const {
         const bool write_same = (read_execution_barriers == rhs.read_execution_barriers) &&
                                 (input_attachment_read == rhs.input_attachment_read) && (last_write == rhs.last_write);
 
-        const bool read_write_same = write_same && (last_read_stages == rhs.last_read_stages) && (last_reads == rhs.last_reads);
+        bool read_same = (last_read_stages == rhs.last_read_stages) && (last_read_count == rhs.last_read_count);
+        if (read_same) {
+            for (uint32_t i = 0; i < last_read_count; i++) {
+                read_same = read_same && (last_reads[i] == rhs.last_reads[i]);
+            }
+        }
+
+        const bool read_write_same = write_same && read_same;
 
         const bool same = read_write_same && (first_accesses_ == rhs.first_accesses_) &&
                           (first_read_stages_ == rhs.first_read_stages_) &&
-                          (first_write_layout_ordering_ == rhs.first_write_layout_ordering_);
+                          (first_write_layout_ordering_index == rhs.first_write_layout_ordering_index);
 
         return same;
     }
-    bool operator!=(const ResourceAccessState &rhs) const { return !(*this == rhs); }
+    bool operator!=(const AccessState &rhs) const { return !(*this == rhs); }
     VkPipelineStageFlags2 GetReadBarriers(SyncAccessIndex access_index) const;
     SyncAccessFlags GetWriteBarriers() const { return last_write.has_value() ? last_write->barriers : SyncAccessFlags(); }
     void SetQueueId(QueueId id);
@@ -477,71 +435,69 @@ class ResourceAccessState {
     void Normalize();
     void GatherReferencedTags(ResourceUsageTagSet &used) const;
 
-    void UpdateStats(syncval_stats::AccessContextStats &stats) const;
+    void UpdateStats(AccessContextStats &stats) const;
 
   private:
-    static constexpr VkPipelineStageFlags2 kInvalidAttachmentStage = ~VkPipelineStageFlags2(0);
+    AccessState() = default;  // not accessible, use DefaultAccessState instead
+    void CopySimpleMembers(const AccessState &other);
     bool IsRAWHazard(const SyncAccessInfo &usage_info) const;
 
-    static bool IsReadHazard(VkPipelineStageFlags2 stage_mask, const VkPipelineStageFlags2 barriers) {
-        return stage_mask != (stage_mask & barriers);
-    }
-
     bool IsReadHazard(VkPipelineStageFlags2 stage_mask, const ReadState &read_access) const {
-        return IsReadHazard(stage_mask, read_access.barriers);
+        return stage_mask != (stage_mask & read_access.barriers);
     }
     VkPipelineStageFlags2 GetOrderedStages(QueueId queue_id, const OrderingBarrier &ordering, SyncFlags flags) const;
 
     void UpdateFirst(ResourceUsageTagEx tag_ex, const SyncAccessInfo &usage_info, SyncOrdering ordering_rule, SyncFlags flags = 0);
     void TouchupFirstForLayoutTransition(ResourceUsageTag tag, const OrderingBarrier &layout_ordering);
-    void MergeReads(const ResourceAccessState &other);
 
-    // TODO: Add a NONE (zero) enum to SyncStageAccessFlags for input_attachment_read and last_write
+    bool HasReads() const { return last_read_count != 0; }
+    vvl::span<ReadState> GetReads() { return vvl::make_span(last_reads, last_read_count); }
+    vvl::span<ReadState> GetReads() const { return vvl::make_span(last_reads, last_read_count); }
+    void AddRead(const ReadState &read);
+    void MergeReads(const AccessState &other);
+    void ClearReadStates();
 
-    // With reads, each must be "safe" relative to it's prior write, so we need only
-    // save the most recent write operation (as anything *transitively* unsafe would arleady
-    // be included
-    // SyncStageAccessFlags write_barriers;              // union of applicable barrier masks since last write
-    // VkPipelineStageFlags2 write_dependency_chain;  // intiially zero, but accumulating the dstStages of barriers if they
-    // chain. ResourceUsageTag write_tag; QueueId write_queue;
-    std::optional<WriteState> last_write;  // only the most recent write
+  private:
+    // The most recent write.
+    // NOTE: For reads, each must be "safe" relative to its prior write, so we need
+    // only to save the most recent write operation, since anything *transitively*
+    // unsafe would already be included.
+    std::optional<WriteState> last_write;
 
-    VkPipelineStageFlags2 last_read_stages;
-    VkPipelineStageFlags2 read_execution_barriers;
+    uint32_t last_read_count = 0;
 
-    // NOTE: default capacity is set to 2. A single read is the most common case.
-    // Two reads occur sometimes and more than two is rare in practice.
-    // Syncval performance is sensitive to memory usage (there are many access objects).
-    // The early tests show that capacity of 1 can further improve performance and in some
-    // apps reduced memory bandwidth outweight the cost of additional allocations.
-    // More testing is needed.
-    using ReadStates = small_vector<ReadState, 2, uint32_t>;
+    // Points to the last_read_count read states
+    ReadState *last_reads = nullptr;
 
-    ReadStates last_reads;
+    // The common case is a single read. In that case, last_reads points to this object
+    // and last_read_count is 1.
+    ReadState single_last_read;
+
+    VkPipelineStageFlags2 last_read_stages = VK_PIPELINE_STAGE_2_NONE;
+    VkPipelineStageFlags2 read_execution_barriers = VK_PIPELINE_STAGE_2_NONE;
+
+    // NOTE: Reserve capacity for 2 first accesses, more than that is not very common
+    using FirstAccesses = small_vector<FirstAccess, 2>;
+    FirstAccesses first_accesses_;
+    VkPipelineStageFlags2 first_read_stages_ = VK_PIPELINE_STAGE_2_NONE;
+    uint32_t first_write_layout_ordering_index = vvl::kNoIndex32;
+    bool first_access_closed_ = false;
 
     // TODO Input Attachment cleanup for multiple reads in a given stage
     // Tracks whether the fragment shader read is input attachment read
-    bool input_attachment_read;
-
-    // Reserve capacity for 2 first accesses, more than that is not very common
-    using FirstAccesses = small_vector<ResourceFirstAccess, 2>;
-
-    FirstAccesses first_accesses_;
-    VkPipelineStageFlags2 first_read_stages_;
-    OrderingBarrier first_write_layout_ordering_;
-    bool first_access_closed_;
+    // TODO: Add a NONE (zero) enum to SyncStageAccessFlags for
+    // input_attachment_read and last_write
+    bool input_attachment_read = false;
 };
-using ResourceAccessStateFunction = std::function<void(ResourceAccessState *)>;
-using ResourceAccessRangeMap = sparse_container::range_map<ResourceAddress, ResourceAccessState>;
-using ResourceRangeMergeIterator = sparse_container::parallel_iterator<ResourceAccessRangeMap, const ResourceAccessRangeMap>;
+using AccessStateFunction = std::function<void(AccessState *)>;
 
 template <typename Predicate>
-bool ResourceAccessState::ClearPredicatedAccesses(Predicate &predicate) {
+bool AccessState::ClearPredicatedAccesses(Predicate &predicate) {
     VkPipelineStageFlags2 sync_reads = VK_PIPELINE_STAGE_2_NONE;
 
     // Use the predicate to build a mask of the read stages we are synchronizing
     // Use the sync_stages to also detect reads known to be before any synchronized reads (first pass)
-    for (const auto &read_access : last_reads) {
+    for (const auto &read_access : GetReads()) {
         if (predicate(read_access)) {
             // If we know this stage is before any stage we syncing, or if the predicate tells us that we are waited for..
             sync_reads |= read_access.stage;
@@ -551,7 +507,7 @@ bool ResourceAccessState::ClearPredicatedAccesses(Predicate &predicate) {
     // Now that we know the reads directly in scopejust need to go over the list again to pick up the "known earlier" stages.
     // NOTE: sync_stages is "deep" catching all stages synchronized after it because we forward barriers
     uint32_t unsync_count = 0;
-    for (const auto &read_access : last_reads) {
+    for (const auto &read_access : GetReads()) {
         if (0 != ((read_access.stage | read_access.sync_stages) & sync_reads)) {
             // This is redundant in the "stage" case, but avoids a second branch to get an accurate count
             sync_reads |= read_access.stage;
@@ -563,24 +519,27 @@ bool ResourceAccessState::ClearPredicatedAccesses(Predicate &predicate) {
     if (unsync_count) {
         if (sync_reads) {
             // When have some remaining unsynchronized reads, we have to rewrite the last_reads array.
-            ReadStates unsync_reads;
+            small_vector<ReadState, 4> unsync_reads;
             unsync_reads.reserve(unsync_count);
             VkPipelineStageFlags2 unsync_read_stages = VK_PIPELINE_STAGE_2_NONE;
-            for (auto &read_access : last_reads) {
-                if (0 == (read_access.stage & sync_reads)) {
+            for (auto &read_access : GetReads()) {
+                if ((read_access.stage & sync_reads) == 0) {
                     unsync_reads.emplace_back(read_access);
                     unsync_read_stages |= read_access.stage;
                 }
             }
             last_read_stages = unsync_read_stages;
-            last_reads = std::move(unsync_reads);
+            ClearReadStates();
+            for (const ReadState &read : unsync_reads) {
+                AddRead(read);
+            }
         }
     } else {
         // Nothing remains (or it was empty to begin with)
         ClearRead();
     }
 
-    bool all_clear = last_reads.empty();
+    bool all_clear = !HasReads();
     if (last_write.has_value()) {
         if (predicate(*this) || sync_reads) {
             // Clear any predicated write, or any the write from any any access with synchronized reads.
@@ -598,5 +557,17 @@ bool ResourceAccessState::ClearPredicatedAccesses(Predicate &predicate) {
 // A helper function to apply multiple barriers.
 // NOTE: That's for use cases when BarrierScope does not use queue id or tag (record time, not-event barriers).
 // This can be extended if necessary to provide BarrierScope for each barrier.
-void ApplyBarriers(ResourceAccessState &access_state, const std::vector<SyncBarrier> &barriers, bool layout_transition = false,
+void ApplyBarriers(AccessState &access_state, const std::vector<SyncBarrier> &barriers, bool layout_transition = false,
                    ResourceUsageTag layout_transition_tag = kInvalidTag);
+
+// Global registry of layout transition ordering barriers
+ThreadSafeLookupTable<OrderingBarrier> &GetLayoutOrderingBarrierLookup();
+
+}  // namespace syncval
+
+namespace std {
+template <>
+struct hash<syncval::OrderingBarrier> {
+    size_t operator()(const syncval::OrderingBarrier &ordering_barrier) const { return ordering_barrier.Hash(); }
+};
+}  // namespace std
