@@ -18,11 +18,13 @@
 #include "sync/sync_common.h"
 #include "state_tracker/buffer_state.h"
 
-const ResourceAccessRange kFullRange(0, std::numeric_limits<VkDeviceSize>::max());
+namespace syncval {
 
-ResourceAccessRange MakeRange(VkDeviceSize start, VkDeviceSize size) { return ResourceAccessRange(start, start + size); }
+const AccessRange kFullRange(0, std::numeric_limits<VkDeviceSize>::max());
 
-ResourceAccessRange MakeRange(const vvl::Buffer& buffer, VkDeviceSize offset, VkDeviceSize size) {
+AccessRange MakeRange(VkDeviceSize start, VkDeviceSize size) { return AccessRange(start, start + size); }
+
+AccessRange MakeRange(const vvl::Buffer& buffer, VkDeviceSize offset, VkDeviceSize size) {
     if (offset >= buffer.create_info.size) {
         return {};
     }
@@ -32,5 +34,7 @@ ResourceAccessRange MakeRange(const vvl::Buffer& buffer, VkDeviceSize offset, Vk
     } else {
         end = std::min(offset + size, buffer.create_info.size);
     }
-    return ResourceAccessRange(offset, end);
+    return AccessRange(offset, end);
 }
+
+}  // namespace syncval
