@@ -4,6 +4,7 @@
 # Copyright (c) 2015-2025 Valve Corporation
 # Copyright (c) 2015-2025 LunarG, Inc.
 # Copyright (c) 2015-2025 Google Inc.
+# Copyright (C) 2025 Arm Limited.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -52,6 +53,13 @@ class APISpecific:
                         'type': 'LayerObjectTypeParameterValidation',
                         'enabled': '!settings.disabled[stateless_checks]'
                     },
+                    {
+                        'include': 'generated/legacy.h',
+                        'device': 'legacy::Device',
+                        'instance': 'legacy::Instance',
+                        'type': 'LayerObjectTypeLegacy',
+                        'enabled': 'settings.enabled[legacy_detection]'
+                    },
                     {
                         'include': 'object_tracker/object_lifetime_validation.h',
                         'device': 'object_lifetimes::Device',
@@ -59,6 +67,19 @@ class APISpecific:
                         'type': 'LayerObjectTypeObjectTracker',
                         'enabled': '!settings.disabled[object_tracking]'
                     },
+                    {
+                        'include': 'state_tracker/state_tracker.h',
+                        'device': 'vvl::DeviceState',
+                        'instance': 'vvl::InstanceState',
+                        'type': 'LayerObjectTypeStateTracker',
+                        'enabled': '''
+                            !settings.disabled[core_checks] ||
+                            settings.enabled[best_practices] ||
+                            settings.enabled[gpu_validation] ||
+                            settings.enabled[debug_printf_validation] ||
+                            settings.enabled[sync_validation]
+                        '''
+                    },
                     {
                         'include': 'core_checks/core_validation.h',
                         'device': 'CoreChecks',
@@ -82,7 +103,7 @@ class APISpecific:
                     },
                     {
                         'include': 'sync/sync_validation.h',
-                        'device': 'SyncValidator',
+                        'device': 'syncval::SyncValidator',
                         'instance': 'syncval::Instance',
                         'type': 'LayerObjectTypeSyncValidation',
                         'enabled': 'settings.enabled[sync_validation]'
@@ -109,6 +130,9 @@ class DispatchObjectGenerator(BaseGenerator):
             'vkCreateComputePipelines',
             'vkCreateRayTracingPipelinesNV',
             'vkCreateRayTracingPipelinesKHR',
+            'vkCreateDataGraphPipelinesARM',
+            # Need to only wrap on certain cases
+            'vkCreateShadersEXT',
             # Need handle which pool descriptors were allocated from
             'vkResetDescriptorPool',
             'vkDestroyDescriptorPool',
@@ -320,7 +344,7 @@ class DispatchObjectGenerator(BaseGenerator):
             out.extend(guard_helper.add_guard(command.protect))
             out.append(f'\n{prototype}\n')
             out.append(f'auto dispatch = vvl::dispatch::GetData({command.params[0].name});\n')
-            returnResult = f'return ' if (command.returnType != 'void') else ''
+            returnResult = 'return ' if (command.returnType != 'void') else ''
             paramsList = ', '.join([param.name for param in command.params])
             out.append(f'{returnResult}{command.name.replace("vk", "dispatch->")}({paramsList}{call_extra});\n')
             out.append('}\n')
@@ -470,7 +494,7 @@ class DispatchObjectGenerator(BaseGenerator):
                     # Check for special case where multiple handles are returned
                     wrap_call = 'WrapNew' if handle_type != 'VkDisplayKHR' else 'MaybeWrapDisplay'
                     ndo_array = lastParam.length is not None
-                    create_ndo_code += 'if (VK_SUCCESS == result) {\n'
+                    create_ndo_code += 'if (result == VK_SUCCESS) {\n'
                     ndo_dest = f'*{lastParam.name}'
                     if ndo_array:
                         create_ndo_code += f'for (uint32_t index0 = 0; index0 < {lastParam.length}; index0++) {{\n'
@@ -595,6 +619,11 @@ class DispatchObjectGenerator(BaseGenerator):
                 if (count_name is not None) and not topLevel:
                     count_name = f'{prefix}{member.length}'
 
+                # Handle the case when the member specifying the count is a pointer
+                for count_member in members:
+                    if count_member.name == member.length and count_member.pointer:
+                        count_name = f'*{count_name}'
+
                 if (not topLevel) or (not isCreate) or (not member.pointer):
                     if count_name is not None:
                         if topLevel:
