@@ -89,6 +89,8 @@ class SpirvValidationHelperOutputGenerator(BaseGenerator):
         if enable.version is not None:
             # Version should be VK_VERSION_x_x as defined in header but need to get as VK_API_VERSION_x_x
             version = enable.version.replace('VK_VERSION', 'VK_API_VERSION')
+            if name == 'SPV_KHR_non_semantic_info':
+                version = 'VK_API_VERSION_1_0'
             out.append(f'{{{version}, nullptr, nullptr, ""}}')
         elif enable.feature is not None:
             out.append(f'{{0, &DeviceFeatures::{enable.feature}, nullptr, ""}}')
@@ -174,7 +176,10 @@ class SpirvValidationHelperOutputGenerator(BaseGenerator):
             #include <spirv/unified1/spirv.hpp>
             #include "vk_extension_helper.h"
             #include "state_tracker/shader_instruction.h"
+            #include "state_tracker/shader_module.h"
             #include "stateless/sl_spirv.h"
+            #include "generated/device_features.h"
+            #include "error_message/error_location.h"
             ''')
 
         #
@@ -207,7 +212,7 @@ class SpirvValidationHelperOutputGenerator(BaseGenerator):
             ''')
 
         #
-        # Build the struct with all the requirments for the spirv capabilities
+        # Build the struct with all the requirements for the spirv capabilities
         out.append('const std::unordered_multimap<uint32_t, RequiredSpirvInfo>& GetSpirvCapabilites() {\n')
         out.append('// clang-format off\n')
         out.append('    static const std::unordered_multimap<uint32_t, RequiredSpirvInfo> spirv_capabilities = {')
@@ -229,12 +234,17 @@ class SpirvValidationHelperOutputGenerator(BaseGenerator):
         out.append('\n')
 
         #
-        # Build the struct with all the requirments for the spirv extensions
+        # Build the struct with all the requirements for the spirv extensions
         out.append('const std::unordered_multimap<std::string_view, RequiredSpirvInfo>& GetSpirvExtensions() {\n')
         out.append('// clang-format off\n')
         out.append('    static const std::unordered_multimap<std::string_view, RequiredSpirvInfo> spirv_extensions = {')
         for spirv in [x for x in self.vk.spirv if x.extension]:
             for enable in spirv.enable:
+                if spirv.name == 'SPV_KHR_non_semantic_info' and enable.version is not None:
+                    out.append('\n        // Debug Printf is in charge of adding SPV_KHR_non_semantic_info and emitting an error if it is not available.\n')
+                    out.append('        // Given how things are wired, core check will not see the extension being added and would emit an error\n')
+                    out.append('        // if SPV_KHR_non_semantic_info is then seen in some spirv code. To prevent that, assume the extension is always available.\n')
+                    out.append('        // If Debug Printf is off, in practice even driver not supporting this extension will not crash.')
                 out.append(f'\n        {{"{spirv.name}", {self.createMapValue(spirv.name, enable, True)}}},')
         out.append('\n    };\n')
         out.append('// clang-format on\n')
@@ -317,7 +327,7 @@ static inline const char* SpvCapabilityRequirements(uint32_t capability) {
 
         out.append('''
 // clang-format off
-static inline std::string SpvExtensionRequirments(std::string_view extension) {
+static inline std::string SpvExtensionRequirements(std::string_view extension) {
     static const vvl::unordered_map<std::string_view, vvl::Requirements> table {
 ''')
         for spirv in [x for x in self.vk.spirv if x.extension]:
@@ -481,7 +491,7 @@ static inline std::string SpvExtensionRequirments(std::string_view extension) {
                 if (has_support == false) {
                     const char *vuid = pipeline ? "VUID-VkShaderModuleCreateInfo-pCode-08742" : "VUID-VkShaderCreateInfoEXT-pCode-08742";
                     skip |= LogError(vuid, module_state.handle(), loc,
-                        "SPIR-V Extension %s was declared, but one of the following requirements is required (%s).", extension_name.c_str(), SpvExtensionRequirments(extension_name).c_str());
+                        "SPIR-V Extension %s was declared, but one of the following requirements is required (%s).", extension_name.c_str(), SpvExtensionRequirements(extension_name).c_str());
                 }
             } //spv::OpExtension
             return skip;
