@@ -5,6 +5,7 @@
 # Copyright (c) 2015-2025 LunarG, Inc.
 # Copyright (c) 2015-2025 Google Inc.
 # Copyright (c) 2023-2025 RasterGrid Kft.
+# Copyright (C) 2025 Arm Limited.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -68,10 +69,13 @@ class StatelessValidationHelperOutputGenerator(BaseGenerator):
             'vkCreateBufferView',
             'vkCreateSemaphore',
             'vkCreateEvent',
+            'vkCreateTensorARM',
             'vkFreeDescriptorSets',
             'vkUpdateDescriptorSets',
             'vkBeginCommandBuffer',
             'vkFreeCommandBuffers',
+            'vkCmdDecompressMemoryEXT',
+            'vkCmdDecompressMemoryIndirectCountEXT',
             'vkCmdSetViewport',
             'vkCmdSetScissor',
             'vkCmdSetLineWidth',
@@ -216,6 +220,13 @@ class StatelessValidationHelperOutputGenerator(BaseGenerator):
             'vkReleaseSwapchainImagesKHR',
             'vkConvertCooperativeVectorMatrixNV',
             'vkCmdConvertCooperativeVectorMatrixNV',
+            'vkCmdBuildPartitionedAccelerationStructuresNV',
+            'vkCmdBuildClusterAccelerationStructureIndirectNV',
+            'vkGetClusterAccelerationStructureBuildSizesNV',
+            'vkCmdResolveImage',
+            'vkCmdResolveImage2',
+            'vkGetCalibratedTimestampsEXT',
+            'vkGetCalibratedTimestampsKHR',
         ]
 
         # Commands to ignore
@@ -278,6 +289,9 @@ class StatelessValidationHelperOutputGenerator(BaseGenerator):
             'VkAccelerationStructureGeometryInstancesDataKHR',
             'VkAccelerationStructureGeometryAabbsDataKHR',
             'VkIndirectExecutionSetPipelineInfoEXT', # VkIndirectExecutionSetShaderInfoEXT is done manually
+            'VkClusterAccelerationStructureTriangleClusterInputNV',
+            'VkClusterAccelerationStructureClustersBottomLevelInputNV',
+            'VkClusterAccelerationStructureMoveObjectsInputNV',
         ]
 
         # These functions entrypoints we as VVL expose
@@ -615,7 +629,8 @@ class StatelessValidationHelperOutputGenerator(BaseGenerator):
             out.append('    bool skip = false;\n')
             # Only generate validation code if the structure actually exists in the target API
             if struct_name in self.vk.structs:
-                out.extend(self.expandStructCode(struct_name, struct_name, 'loc', 'info.', '', [], 'context.'))
+                struct = self.vk.structs[struct_name]
+                out.extend(self.expandStructCode(struct_name, struct_name, 'loc', 'info.', '', [], 'context.', None))
             out.append('    return skip;\n')
             out.append('}\n')
 
@@ -662,6 +677,11 @@ class StatelessValidationHelperOutputGenerator(BaseGenerator):
     def makePointerCheck(self, valuePrefix, member: Member, lengthMember: Member, errorLoc, arrayRequired, counValueRequired, counPtrRequired, funcName, structTypeName, context):
         checkExpr = []
         callerName = structTypeName if structTypeName else funcName
+
+        # Workaround until https://gitlab.khronos.org/vulkan/vulkan/-/merge_requests/7886 is merged
+        if callerName == 'VkDeviceDeviceMemoryReportCreateInfoEXT' and member.name == 'pUserData':
+            return checkExpr
+
         if lengthMember:
             length_deref = '->' in member.length
             countRequiredVuid = self.GetVuid(callerName, f"{member.length}-arraylength")
@@ -728,7 +748,7 @@ class StatelessValidationHelperOutputGenerator(BaseGenerator):
         return checkExpr
 
     # Process struct member validation code, performing name substitution if required
-    def processStructMemberCode(self, line, funcName, errorLoc, memberNamePrefix, memberDisplayNamePrefix, context):
+    def processStructMemberCode(self, line, funcName, errorLoc, memberNamePrefix, memberDisplayNamePrefix, context, selector):
         # Build format specifier list
         kwargs = {}
         if '{funcName}' in line:
@@ -744,7 +764,9 @@ class StatelessValidationHelperOutputGenerator(BaseGenerator):
             else:
                 kwargs['displayNamePrefix'] = memberDisplayNamePrefix
         if '{context}' in line:
-                kwargs['context'] = context
+            kwargs['context'] = context
+        if '{selector}' in line:
+            kwargs['selector'] = f"{{valuePrefix}}{selector}"
 
         if kwargs:
             # Need to escape the C++ curly braces
@@ -765,7 +787,7 @@ class StatelessValidationHelperOutputGenerator(BaseGenerator):
         return scrubbed_lines
 
     # Process struct validation code for inclusion in function or parent struct validation code
-    def expandStructCode(self, item_type, funcName, errorLoc, memberNamePrefix, memberDisplayNamePrefix, output, context):
+    def expandStructCode(self, item_type, funcName, errorLoc, memberNamePrefix, memberDisplayNamePrefix, output, context, selector):
         if item_type not in self.validatedStructs:
             return ""
         lines = self.validatedStructs[item_type]
@@ -774,9 +796,9 @@ class StatelessValidationHelperOutputGenerator(BaseGenerator):
                 output[-1] += '\n'
             if isinstance(line, list):
                 for sub in line:
-                    output.append(self.processStructMemberCode(sub, funcName, errorLoc, memberNamePrefix, memberDisplayNamePrefix, context))
+                    output.append(self.processStructMemberCode(sub, funcName, errorLoc, memberNamePrefix, memberDisplayNamePrefix, context, selector))
             else:
-                output.append(self.processStructMemberCode(line, funcName, errorLoc, memberNamePrefix, memberDisplayNamePrefix, context))
+                output.append(self.processStructMemberCode(line, funcName, errorLoc, memberNamePrefix, memberDisplayNamePrefix, context, selector))
         return output
 
     # Generate the parameter checking code
@@ -800,6 +822,9 @@ class StatelessValidationHelperOutputGenerator(BaseGenerator):
             usedLines = []
             lengthMember = None
             condition = None
+            # Incase the member is actually a Param
+            selector = getattr(member, 'selector', None)
+
             #
             # Generate the full name of the value, which will be printed in the error message, by adding the variable prefix to the value name
             valueDisplayName = f'{displayNamePrefix}{member.name}'
@@ -852,7 +877,10 @@ class StatelessValidationHelperOutputGenerator(BaseGenerator):
                                 # Handle edge case where XML expresses a non-optional non-pointer value length with noautovalidity
                                 # ex: <param noautovalidity="true"len="commandBufferCount">
                                 countRequiredVuid = self.GetVuid(callerName, f"{lengthMember.name}-arraylength")
-                                if countRequiredVuid in duplicateCountVuid:
+                                if ' ' in member.length:
+                                    # For things like altlen="(rasterizationSamples + 31) / 32" we want to skip these
+                                    countRequiredVuid = None
+                                elif countRequiredVuid in duplicateCountVuid:
                                     countRequiredVuid = None
                                 else:
                                     duplicateCountVuid.append(countRequiredVuid)
@@ -878,7 +906,7 @@ class StatelessValidationHelperOutputGenerator(BaseGenerator):
                 else:
                     if member.type in self.vk.structs and self.vk.structs[member.type].sType:
                         # If this is a pointer to a struct with an sType field, verify the type
-                        struct = self.vk.structs[member.type]
+                        member_struct = self.vk.structs[member.type]
                         sTypeVuid = self.GetVuid(member.type, "sType-sType")
                         paramVuid = self.GetVuid(callerName, f"{member.name}-parameter")
                         if lengthMember:
@@ -888,18 +916,18 @@ class StatelessValidationHelperOutputGenerator(BaseGenerator):
                                 paramVuid = 'kVUIDUndefined'
                             # This is an array of struct pointers
                             if member.cDeclaration.count('*') == 2:
-                                usedLines.append(f'skip |= {context}ValidateStructPointerTypeArray({errorLoc}.dot(Field::{lengthMember.name}), {errorLoc}.dot(Field::{member.name}), {valuePrefix}{lengthMember.name}, {valuePrefix}{member.name}, {struct.sType}, {counValueRequired}, {arrayRequired}, {sTypeVuid}, {paramVuid}, {countRequiredVuid});\n')
+                                usedLines.append(f'skip |= {context}ValidateStructPointerTypeArray({errorLoc}.dot(Field::{lengthMember.name}), {errorLoc}.dot(Field::{member.name}), {valuePrefix}{lengthMember.name}, {valuePrefix}{member.name}, {member_struct.sType}, {counValueRequired}, {arrayRequired}, {sTypeVuid}, {paramVuid}, {countRequiredVuid});\n')
                             # This is an array with a pointer to a count value
                             elif lengthMember.pointer:
                                 # When the length parameter is a pointer, there is an extra Boolean parameter in the function call to indicate if it is required
                                 countPtrRequiredVuid = self.GetVuid(callerName, f"{member.length}-parameter")
-                                usedLines.append(f'skip |= {context}ValidateStructTypeArray({errorLoc}.dot(Field::{member.length}), {errorLoc}.dot(Field::{member.name}), {valuePrefix}{member.length}, {valuePrefix}{member.name}, {struct.sType}, {counPtrRequired}, {counValueRequired}, {arrayRequired}, {sTypeVuid}, {paramVuid}, {countPtrRequiredVuid}, {countRequiredVuid});\n')
+                                usedLines.append(f'skip |= {context}ValidateStructTypeArray({errorLoc}.dot(Field::{member.length}), {errorLoc}.dot(Field::{member.name}), {valuePrefix}{member.length}, {valuePrefix}{member.name}, {member_struct.sType}, {counPtrRequired}, {counValueRequired}, {arrayRequired}, {sTypeVuid}, {paramVuid}, {countPtrRequiredVuid}, {countRequiredVuid});\n')
                             # This is an array with an integer count value
                             else:
-                                usedLines.append(f'skip |= {context}ValidateStructTypeArray({errorLoc}.dot(Field::{member.length}), {errorLoc}.dot(Field::{member.name}), {valuePrefix}{member.length}, {valuePrefix}{member.name}, {struct.sType}, {counValueRequired}, {arrayRequired}, {sTypeVuid}, {paramVuid}, {countRequiredVuid});\n')
+                                usedLines.append(f'skip |= {context}ValidateStructTypeArray({errorLoc}.dot(Field::{member.length}), {errorLoc}.dot(Field::{member.name}), {valuePrefix}{member.length}, {valuePrefix}{member.name}, {member_struct.sType}, {counValueRequired}, {arrayRequired}, {sTypeVuid}, {paramVuid}, {countRequiredVuid});\n')
                         # This is an individual struct
                         else:
-                            usedLines.append(f'skip |= {context}ValidateStructType({errorLoc}.dot(Field::{member.name}), {valuePrefix}{member.name}, {struct.sType}, {arrayRequired}, {paramVuid}, {sTypeVuid});\n')
+                            usedLines.append(f'skip |= {context}ValidateStructType({errorLoc}.dot(Field::{member.name}), {valuePrefix}{member.name}, {member_struct.sType}, {arrayRequired}, {paramVuid}, {sTypeVuid});\n')
                     # If this is an input handle array that is not allowed to contain NULL handles, verify that none of the handles are VK_NULL_HANDLE
                     elif member.type in self.vk.handles and member.const and not self.isHandleOptional(member, lengthMember):
                         if not lengthMember:
@@ -977,7 +1005,7 @@ class StatelessValidationHelperOutputGenerator(BaseGenerator):
                             memberDisplayNamePrefix = f'{valueDisplayName}->'
 
                         # Expand the struct validation lines
-                        expr = self.expandStructCode(member.type, funcName, newErrorLoc, memberNamePrefix, memberDisplayNamePrefix, expr, context)
+                        expr = self.expandStructCode(member.type, funcName, newErrorLoc, memberNamePrefix, memberDisplayNamePrefix, expr, context, selector)
                         # If only 4 lines and no "skip" then this is an empty check
                         hasChecks = len(expr) > 4 or 'skip' in expr[3]
                         hasChecks = hasChecks if member.type != 'VkRect2D' else False # exception that doesn't have check actually
@@ -1033,11 +1061,23 @@ class StatelessValidationHelperOutputGenerator(BaseGenerator):
                             invalidVuid = self.GetVuid(callerName, f"{member.name}-parameter")
                             zeroVuid = invalidVuid
                         allFlagsName = 'All' + flagBitsName
-                        zeroVuidArg = '' if member.optional else ', ' + zeroVuid
+                        zeroVuidArg = ', nullptr' if member.optional else ', ' + zeroVuid
                         condition = [item for item in self.structMemberValidationConditions if (item['struct'] == structTypeName and item['field'] == flagBitsName)]
-                        usedLines.append(f'skip |= {context}ValidateFlags({errorLoc}.dot(Field::{member.name}), vvl::FlagBitmask::{flagBitsName}, {allFlagsName}, {valuePrefix}{member.name}, {flagsType}, {invalidVuid}{zeroVuidArg});\n')
+                        isInstanceFunction = 'false'
+                        if (struct):
+                            extensions = self.vk.structs[struct.name].extensions
+                            for extension in extensions:
+                                if self.vk.extensions[extension].instance:
+                                    isInstanceFunction = 'true'
+                        else:
+                            if self.vk.commands[funcName].instance:
+                                isInstanceFunction = 'true'
+                        usedLines.append(f'skip |= {context}ValidateFlags({errorLoc}.dot(Field::{member.name}), vvl::FlagBitmask::{flagBitsName}, {allFlagsName}, {valuePrefix}{member.name}, {flagsType}, {invalidVuid}{zeroVuidArg}, {isInstanceFunction});\n')
                     elif member.type == 'VkBool32':
                         usedLines.append(f'skip |= {context}ValidateBool32({errorLoc}.dot(Field::{member.name}), {valuePrefix}{member.name});\n')
+                    elif member.type == 'VkDeviceAddress' and not member.optional:
+                        vuid = self.GetVuid(callerName, f"{member.name}-parameter")
+                        usedLines.append(f'skip |= {context}ValidateNotZero({valuePrefix}{member.name} == 0, {vuid}, {errorLoc}.dot(Field::{member.name}));\n')
                     elif member.type in self.vk.enums and member.type != 'VkStructureType':
                         vuid = self.GetVuid(callerName, f"{member.name}-parameter")
                         usedLines.append(f'skip |= {context}ValidateRangedEnum({errorLoc}.dot(Field::{member.name}), vvl::Enum::{member.type}, {valuePrefix}{member.name}, {vuid});\n')
@@ -1045,7 +1085,7 @@ class StatelessValidationHelperOutputGenerator(BaseGenerator):
                     if member.type in self.validatedStructs:
                         memberNamePrefix = f'{valuePrefix}{member.name}.'
                         memberDisplayNamePrefix = f'{valueDisplayName}.'
-                        usedLines.append(self.expandStructCode(member.type, funcName, errorLoc, memberNamePrefix, memberDisplayNamePrefix, [], context))
+                        usedLines.append(self.expandStructCode(member.type, funcName, errorLoc, memberNamePrefix, memberDisplayNamePrefix, [], context, selector))
             # Append the parameter check to the function body for the current command
             if usedLines:
                 # Apply special conditional checks
@@ -1058,6 +1098,16 @@ class StatelessValidationHelperOutputGenerator(BaseGenerator):
                     checkedExpr.append('}\n')
                     usedLines = [checkedExpr]
 
+                if (struct is not None and struct.union):
+                    checkedExpr = []
+                    condExpr = " || ".join(f"{{selector}} == {v}" for v in member.selection)
+                    checkedExpr.append(f'if ({condExpr})')
+                    checkedExpr.append('{\n')
+                    for expr in usedLines:
+                        checkedExpr.append(expr)
+                    checkedExpr.append('}\n')
+                    usedLines = checkedExpr
+
                 lines += usedLines
 
         return lines
@@ -1084,7 +1134,7 @@ class StatelessValidationHelperOutputGenerator(BaseGenerator):
         if struct.name == 'VkPhysicalDeviceLayeredApiPropertiesListKHR':
             return ""
 
-        expr = self.expandStructCode(struct.name, struct.name, 'pNext_loc', 'structure->', '', [], '')
+        expr = self.expandStructCode(struct.name, struct.name, 'pNext_loc', 'structure->', '', [], '', None)
         structValidationSource = self.ScrubStructCode(expr)
         if structValidationSource != '':
             # Only reasonable to validate content of structs if const as otherwise the date inside has not been writen to yet
