@@ -1,8 +1,8 @@
 #!/usr/bin/env python3
 
 # Copyright 2017 The Glslang Authors. All rights reserved.
-# Copyright (c) 2018-2024 Valve Corporation
-# Copyright (c) 2018-2024 LunarG, Inc.
+# Copyright (c) 2018-2025 Valve Corporation
+# Copyright (c) 2018-2025 LunarG, Inc.
 # Copyright (c) 2023-2023 RasterGrid Kft.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -108,6 +108,9 @@ a description of its elements.  All elements are required except those
 marked as optional.  Please see the "known_good.json" file for
 examples of all of these elements.
 
+Required fields
+---------------
+
 - name
 
 The name of the dependent repository.  This field can be referenced
@@ -137,10 +140,28 @@ directory.
 The directory used to store the installed build artifacts, relative
 to the "top" directory.
 
+Required Exclusive Option Group: [commit, release]
+-----------------
+
 - commit
 
 The commit used to checkout the repository.  This can be a SHA-1
-object name or a refname used with the remote name "origin".
+object name or a refname used with the remote name "origin". This
+option is for dependencies that need to be built from source.
+
+- release
+
+Tells update_deps to download binary artifacts instead of checking out
+a git repository. The value of release can be used as a format specifier
+in the url json field. Because different platforms require different binaries,
+the url *can* contain the following format specifiers:
+    release - replaced with the release json field
+    system - replaced with platform.system() (EG. Windows, Linux, Darwin)
+    arch - replaced with the arch command line argument
+
+
+Optional fields
+---------------
 
 - deps (optional)
 
@@ -222,6 +243,20 @@ Legal options include:
 
 Builds on all platforms by default.
 
+- url_format_map (optional)
+
+Used with the --release option. Contains a list of key-string pairs that are used
+to fixup the platform.system() and args.arch names in the url. It is needed
+to account for differences between the arch and system names python uses and the
+naming scheme of the repository. For example if the binary artifact for the current
+system uses "macOS", we need to add "darwin" : "macos" to the url_format_map so that
+the url is formatted with macOS instead of darwin.
+
+- build_architectures (optional)
+
+A list of architectures supported by the dependency. If the value of args.arch isn't
+present in this list, then the dependency is skipped.
+
 Note
 ----
 
@@ -244,6 +279,9 @@ import shlex
 import shutil
 import stat
 import time
+import urllib.request
+import zipfile
+import tarfile
 
 KNOWN_GOOD_FILE_NAME = 'known_good.json'
 
@@ -301,7 +339,7 @@ def run_cmake_command(cmake_cmd):
     # errors='replace' affects only how the text output is decoded, and indicates that
     # 8-bit characters that aren't recognized by the UTF decoder will be replaced with
     # an "unknown character" glyph instead of crashing.
-    result = subprocess.run(cmake_cmd, 
+    result = subprocess.run(cmake_cmd,
         stdout = subprocess.PIPE,
         stderr = subprocess.STDOUT,
         text = True,
@@ -333,9 +371,18 @@ class GoodRepo(object):
         self._args = args
         # Required JSON elements
         self.name = json['name']
+
         self.url = json['url']
         self.sub_dir = json['sub_dir']
-        self.commit = json['commit']
+
+        # commit/release Exclusive Option Group
+        self.commit = json.get('commit', None)
+        self.release = json.get('release', None)
+        self.is_release = True if self.release is not None else False
+
+        if self.commit is None and self.release is None:
+            raise RuntimeError(f'Repo {self.name} is missing required exclusive option group! Either "commit" or "release" must be specified.')
+
         # Optional JSON elements
         self.build_dir = None
         self.install_dir = None
@@ -357,6 +404,9 @@ class GoodRepo(object):
         self.build_platforms = json['build_platforms'] if ('build_platforms' in json) else []
         self.optional = set(json.get('optional', []))
         self.api = json['api'] if ('api' in json) else None
+        self.url_format_map = json.get('url_format_map')
+        self.build_architectures = json.get('build_architectures') if ('build_architectures' in json) else []
+
         # Absolute paths for a repo's directories
         dir_top = os.path.abspath(args.dir)
         self.repo_dir = os.path.join(dir_top, self.sub_dir)
@@ -376,6 +426,10 @@ class GoodRepo(object):
         if self.build_platforms == [] or target_platform in self.build_platforms:
             self.on_build_platform = True
 
+        self.on_build_architecture = False
+        if self.build_architectures == [] or self._args.arch in self.build_architectures:
+            self.on_build_architecture = True
+
     def Clone(self, retries=10, retry_seconds=60):
         if VERBOSE:
             print('Cloning {n} into {d}'.format(n=self.name, d=self.repo_dir))
@@ -443,6 +497,46 @@ class GoodRepo(object):
         if VERBOSE:
             print(command_output(['git', 'status'], self.repo_dir))
 
+
+    def DownloadAndExtractArtifact(self):
+        """Download and extract file located at given url."""
+
+        system = platform.system()
+        arch = self._args.arch
+
+        if system in self.url_format_map:
+            system = self.url_format_map[system]
+        if arch in self.url_format_map:
+            arch = self.url_format_map[arch]
+
+        formatted_url = self.url.format(release=self.release, system=system, arch=arch)
+
+        # use the last part of the url after the slash as filename to download into
+        download_path = os.path.join(self.build_dir, formatted_url[formatted_url.rfind('/') + 1:])
+
+        make_or_exist_dirs(self.build_dir)
+        print(f"Downloading {self.name} from {formatted_url} to {download_path}")
+
+        if not os.path.isfile(download_path):
+            if VERBOSE:
+                print(f"Downloading {self.name} from {formatted_url} to {download_path}")
+            try:
+                urllib.request.urlretrieve(formatted_url, download_path)
+            except Exception as e:
+                print(f"Failed to download {self.name}: {e}")
+                return False
+
+        make_or_exist_dirs(self.install_dir)
+
+        if download_path.endswith('.zip'):
+            with zipfile.ZipFile(download_path, 'r') as zip_ref:
+                zip_ref.extractall(self.install_dir)
+        elif download_path.endswith('.tar') or download_path.endswith('.tar.gz'):
+            with tarfile.open(download_path) as tar_ref:
+                tar_ref.extractall(self.install_dir)
+
+        return True
+
     def CustomPreProcess(self, cmd_str, repo_dict):
         return cmd_str.format(repo_dict, self._args, CONFIG_MAP[self._args.config])
 
@@ -491,7 +585,7 @@ class GoodRepo(object):
         # repo's install dir.
         for d in self.deps:
             dep_commit = [r for r in repos if r.name == d['repo_name']]
-            if len(dep_commit) and dep_commit[0].on_build_platform:
+            if len(dep_commit) and dep_commit[0].on_build_platform and dep_commit[0].on_build_architecture:
                 cmake_cmd.append('-D{var_name}={install_dir}'.format(
                     var_name=d['var_name'],
                     install_dir=dep_commit[0].install_dir))
@@ -580,6 +674,23 @@ class GoodRepo(object):
 
         print(f"Installed {self.name} ({self.commit}) in {total_time:.3f} seconds", flush=True)
 
+    def AcquireReleaseArtifacts(self):
+        """Process acquiring the release artifacts and time how long it took"""
+        if VERBOSE:
+            print('Processing release artifacts {n}'.format(n=self.name))
+            print('Download dir = {b}'.format(b=self.build_dir))
+            print('Install dir = {i}\n'.format(i=self.install_dir))
+
+        start = time.time()
+
+        success = self.DownloadAndExtractArtifact()
+        if not success:
+            return
+
+        total_time = time.time() - start
+
+        print(f"Installed {self.name} ({self.release}) in {total_time:.3f} seconds", flush=True)
+
     def IsOptional(self, opts):
         return len(self.optional.intersection(opts)) > 0
 
@@ -623,7 +734,6 @@ def GetInstallNames(args):
         else:
             return None
 
-
 def CreateHelper(args, repos, filename):
     """Create a CMake config helper file.
 
@@ -642,19 +752,22 @@ def CreateHelper(args, repos, filename):
             # the target API then skip it
             if repo.api is not None and repo.api != args.api:
                 continue
-            if install_names and repo.name in install_names and repo.on_build_platform:
+            if install_names and repo.name in install_names and repo.on_build_platform and repo.on_build_architecture:
                 helper_file.write('set({var} "{dir}" CACHE STRING "")\n'
                                   .format(
                                       var=install_names[repo.name],
                                       dir=escape(repo.install_dir)))
 
-
 def GetDefaultArch():
     machine = platform.machine().lower()
     if  machine == "arm64" or machine == "aarch64":
         return 'arm64'
-    else:
-        return '64'
+    system = platform.system()
+    # vcvarsall.bat sets the Platform env-var, which CMake uses to determine the default build architecture
+    Platform_env_var = os.getenv('Platform')
+    if system == 'Windows' and Platform_env_var is not None:
+        return Platform_env_var
+    return '64'
 
 
 def main():
@@ -779,9 +892,9 @@ def main():
         if repo.api is not None and repo.api != args.api:
             continue
 
-        # If the repo has a platform whitelist, skip the repo
-        # unless we are building on a whitelisted platform.
-        if not repo.on_build_platform:
+        # If the repo has a platform or architecture whitelist, skip the repo
+        # unless we are building on a whitelisted platform and architecture.
+        if not repo.on_build_platform or not repo.on_build_architecture:
             continue
 
         # Skip building the repo if its install directory already exists
@@ -827,12 +940,17 @@ def main():
             if not do_build:
                 continue
 
-        # Clone/update the repository
-        repo.Checkout()
+        # Handle binary dependencies differently
+        if repo.is_release:
+            if args.do_build:
+                repo.AcquireReleaseArtifacts()
+        else:
+            # Clone/update the repository
+            repo.Checkout()
 
-        # Build the repository
-        if args.do_build and repo.build_step != 'skip':
-            repo.Build(repos, repo_dict)
+            # Build the repository
+            if args.do_build and repo.build_step != 'skip':
+                repo.Build(repos, repo_dict)
 
     # Need to restore original cwd in order for CreateHelper to find json file
     os.chdir(save_cwd)
@@ -843,4 +961,3 @@ def main():
 
 if __name__ == '__main__':
     main()
-
