@@ -2,6 +2,7 @@
  * Copyright (c) 2015-2016, 2020-2025 The Khronos Group Inc.
  * Copyright (c) 2015-2016, 2020-2025 Valve Corporation
  * Copyright (c) 2015-2016, 2020-2025 LunarG, Inc.
+ * Copyright (C) 2025 Arm Limited.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -56,6 +57,7 @@ class CommandPool;
 class Swapchain;
 class IndirectCommandsLayout;
 class IndirectExecutionSet;
+class Tensor;
 
 std::vector<VkLayerProperties> GetGlobalLayers();
 std::vector<VkExtensionProperties> GetGlobalExtensions();
@@ -134,7 +136,7 @@ class NonDispHandle : public Handle<T> {
 
     void SetDevice(VkDevice device) { dev_handle_ = device; }
 
-    void destroy() noexcept { dev_handle_ = VK_NULL_HANDLE; }
+    void Destroy() noexcept { dev_handle_ = VK_NULL_HANDLE; }
 
   public:
     void SetName(VkObjectType object_type, const char *name) { Handle<T>::SetName(dev_handle_, object_type, name); }
@@ -239,6 +241,7 @@ class Device : public internal::Handle<VkDevice> {
     const std::vector<Queue *> &QueuesWithComputeCapability() const { return queues_[COMPUTE]; }
     const std::vector<Queue *> &QueuesWithTransferCapability() const { return queues_[TRANSFER]; }
     const std::vector<Queue *> &QueuesWithSparseCapability() const { return queues_[SPARSE]; }
+    const std::vector<Queue *> &QueuesWithDataGraphCapability() const { return queues_[DATA_GRAPH]; }
 
     using QueueFamilyQueues = std::vector<std::unique_ptr<Queue>>;
     const QueueFamilyQueues &QueuesFromFamily(uint32_t queue_family) const;
@@ -262,7 +265,7 @@ class Device : public internal::Handle<VkDevice> {
     std::optional<uint32_t> NonGraphicsQueueFamily() const;
     Queue *NonGraphicsQueue() const;
 
-    uint32_t graphics_queue_node_index_ = vvl::kU32Max;
+    uint32_t graphics_queue_node_index_ = vvl::kNoIndex32;
 
     const PhysicalDevice physical_device_;
 
@@ -305,7 +308,8 @@ class Device : public internal::Handle<VkDevice> {
         SPARSE = 3,
         VIDEO_DECODE = 4,
         VIDEO_ENCODE = 5,
-        QUEUE_CAPABILITY_COUNT = 6,
+        DATA_GRAPH = 6,
+        QUEUE_CAPABILITY_COUNT = 7,
     };
 
     void InitQueues(const VkDeviceCreateInfo &info);
@@ -322,7 +326,7 @@ class DeviceMemory : public internal::NonDispHandle<VkDeviceMemory> {
     DeviceMemory() = default;
     DeviceMemory(const Device &dev, const VkMemoryAllocateInfo &info) { Init(dev, info); }
     ~DeviceMemory() noexcept;
-    void destroy() noexcept;
+    void Destroy() noexcept;
     DeviceMemory &operator=(DeviceMemory &&) = default;
 
     // vkAllocateMemory()
@@ -356,7 +360,7 @@ class Fence : public internal::NonDispHandle<VkFence> {
     Fence(Fence &&rhs) noexcept : NonDispHandle(std::move(rhs)) {}
     Fence &operator=(Fence &&) noexcept;
     ~Fence() noexcept;
-    void destroy() noexcept;
+    void Destroy() noexcept;
 
     // vkCreateFence()
     void Init(const Device &dev, const VkFenceCreateInfo &info);
@@ -387,7 +391,7 @@ class Semaphore : public internal::NonDispHandle<VkSemaphore> {
     Semaphore(Semaphore &&rhs) noexcept : NonDispHandle(std::move(rhs)) {}
     Semaphore &operator=(Semaphore &&) noexcept;
     ~Semaphore() noexcept;
-    void destroy() noexcept;
+    void Destroy() noexcept;
 
     // vkCreateSemaphore()
     void Init(const Device &dev, const VkSemaphoreCreateInfo &info);
@@ -415,7 +419,7 @@ class Event : public internal::NonDispHandle<VkEvent> {
     Event(const Device &dev) { Init(dev, CreateInfo(0)); }
     Event(const Device &dev, const VkEventCreateInfo &info) { Init(dev, info); }
     ~Event() noexcept;
-    void destroy() noexcept;
+    void Destroy() noexcept;
 
     // vkCreateEvent()
     void Init(const Device &dev, const VkEventCreateInfo &info);
@@ -532,7 +536,7 @@ class QueryPool : public internal::NonDispHandle<VkQueryPool> {
         Init(dev, info);
     }
     ~QueryPool() noexcept;
-    void destroy() noexcept;
+    void Destroy() noexcept;
 
     // vkCreateQueryPool()
     void Init(const Device &dev, const VkQueryPoolCreateInfo &info);
@@ -573,6 +577,14 @@ class Buffer : public internal::NonDispHandle<VkBuffer> {
              &allocate_flag_info);
     }
 
+    explicit Buffer(const Device& dev, VkDeviceSize size, VkBufferUsageFlags2CreateInfo usage_ci, DeviceAddressT) {
+        usage_ci.usage |= VK_BUFFER_USAGE_2_SHADER_DEVICE_ADDRESS_BIT;
+        VkMemoryAllocateFlagsInfo allocate_flag_info = vku::InitStructHelper();
+        allocate_flag_info.flags = VK_MEMORY_ALLOCATE_DEVICE_ADDRESS_BIT;
+        Init(dev, CreateInfo(size, 0, {}, &usage_ci), VK_MEMORY_PROPERTY_HOST_VISIBLE_BIT | VK_MEMORY_PROPERTY_HOST_COHERENT_BIT,
+             &allocate_flag_info);
+    }
+
     explicit Buffer(const Device &dev, VkBufferUsageFlags usage, const void *data, size_t data_size,
                     const vvl::span<uint32_t> &queue_families = {}) {
         InitHostVisibleWithData(dev, usage, data, data_size, queue_families);
@@ -581,7 +593,7 @@ class Buffer : public internal::NonDispHandle<VkBuffer> {
     Buffer(Buffer &&rhs) noexcept;
     Buffer &operator=(Buffer &&rhs) noexcept;
     ~Buffer() noexcept;
-    void destroy() noexcept;
+    void Destroy() noexcept;
 
     // vkCreateBuffer()
     void Init(const Device &dev, const VkBufferCreateInfo &info, VkMemoryPropertyFlags mem_props = 0,
@@ -663,7 +675,7 @@ class BufferView : public internal::NonDispHandle<VkBufferView> {
         Init(dev, buffer_view_ci);
     }
     ~BufferView() noexcept;
-    void destroy() noexcept;
+    void Destroy() noexcept;
 
     // vkCreateBufferView()
     void Init(const Device &dev, const VkBufferViewCreateInfo &info);
@@ -695,7 +707,7 @@ class Image : public internal::NonDispHandle<VkImage> {
     Image &operator=(Image &&rhs) noexcept;
 
     ~Image() noexcept;
-    void destroy() noexcept;
+    void Destroy() noexcept;
 
     void Init(const Device &dev, const VkImageCreateInfo &info, VkMemoryPropertyFlags mem_props = 0,
               void *alloc_info_pnext = nullptr);
@@ -793,7 +805,7 @@ class ImageView : public internal::NonDispHandle<VkImageView> {
         return *this;
     }
     ~ImageView() noexcept;
-    void destroy() noexcept;
+    void Destroy() noexcept;
 
     // vkCreateImageView()
     void Init(const Device &dev, const VkImageViewCreateInfo &info);
@@ -806,7 +818,7 @@ class AccelerationStructureNV : public internal::NonDispHandle<VkAccelerationStr
         Init(dev, info, init_memory);
     }
     ~AccelerationStructureNV() noexcept;
-    void destroy() noexcept;
+    void Destroy() noexcept;
 
     // vkCreateAccelerationStructureNV
     void Init(const Device &dev, const VkAccelerationStructureCreateInfoNV &info, bool init_memory = true);
@@ -837,7 +849,7 @@ class ShaderModule : public internal::NonDispHandle<VkShaderModule> {
     ShaderModule(ShaderModule &&rhs) noexcept : NonDispHandle(std::move(rhs)) {}
     ShaderModule &operator=(ShaderModule &&rhs) noexcept;
     ~ShaderModule() noexcept;
-    void destroy() noexcept;
+    void Destroy() noexcept;
 
     // vkCreateShaderModule()
     void Init(const Device &dev, const VkShaderModuleCreateInfo &info);
@@ -859,7 +871,7 @@ class Shader : public internal::NonDispHandle<VkShaderEXT> {
     Shader(const Device &dev, const VkShaderStageFlagBits stage, const std::vector<uint8_t> &binary,
            const VkDescriptorSetLayout *descriptorSetLayout = nullptr, const VkPushConstantRange *pushConstRange = nullptr);
     ~Shader() noexcept;
-    void destroy() noexcept;
+    void Destroy() noexcept;
 
     // vkCreateShaderModule()
     void Init(const Device &dev, const VkShaderCreateInfoEXT &info);
@@ -872,7 +884,7 @@ class PipelineCache : public internal::NonDispHandle<VkPipelineCache> {
     PipelineCache() = default;
     PipelineCache(const Device &dev, const VkPipelineCacheCreateInfo &info) { Init(dev, info); }
     ~PipelineCache() noexcept;
-    void destroy() noexcept;
+    void Destroy() noexcept;
 
     void Init(const Device &dev, const VkPipelineCacheCreateInfo &info);
     void SetName(const char *name) { NonDispHandle<VkPipelineCache>::SetName(VK_OBJECT_TYPE_PIPELINE_CACHE, name); }
@@ -887,8 +899,9 @@ class Pipeline : public internal::NonDispHandle<VkPipeline> {
     }
     Pipeline(const Device &dev, const VkComputePipelineCreateInfo &info) { Init(dev, info); }
     Pipeline(const Device &dev, const VkRayTracingPipelineCreateInfoKHR &info) { Init(dev, info); }
+    Pipeline(const Device &dev, const VkDataGraphPipelineCreateInfoARM &info) { Init(dev, info); }
     ~Pipeline() noexcept;
-    void destroy() noexcept;
+    void Destroy() noexcept;
 
     // vkCreateGraphicsPipeline()
     void Init(const Device &dev, const VkGraphicsPipelineCreateInfo &info);
@@ -900,6 +913,8 @@ class Pipeline : public internal::NonDispHandle<VkPipeline> {
     void Init(const Device &dev, const VkRayTracingPipelineCreateInfoKHR &info);
     // vkCreateRayTracingPipelinesKHR with deferredOperation
     void InitDeferred(const Device &dev, const VkRayTracingPipelineCreateInfoKHR &info, VkDeferredOperationKHR deferred_op);
+    // vkCreateDataGraphPipelinesARM
+    void Init(const Device &dev, const VkDataGraphPipelineCreateInfoARM &info);
     // vkLoadPipeline()
     void Init(const Device &dev, size_t size, const void *data);
     // vkLoadPipelineDerivative()
@@ -928,7 +943,7 @@ class PipelineLayout : public internal::NonDispHandle<VkPipelineLayout> {
         Init(dev, info, layouts);
     }
     ~PipelineLayout() noexcept;
-    void destroy() noexcept;
+    void Destroy() noexcept;
 
     // Move constructor for Visual Studio 2013
     PipelineLayout(PipelineLayout &&src) noexcept : NonDispHandle(std::move(src)){};
@@ -950,7 +965,7 @@ class Sampler : public internal::NonDispHandle<VkSampler> {
     Sampler() = default;
     Sampler(const Device &dev, const VkSamplerCreateInfo &info) { Init(dev, info); }
     ~Sampler() noexcept;
-    void destroy() noexcept;
+    void Destroy() noexcept;
 
     // vkCreateSampler()
     void Init(const Device &dev, const VkSamplerCreateInfo &info);
@@ -978,12 +993,12 @@ class DescriptorSetLayout : public internal::NonDispHandle<VkDescriptorSetLayout
         Init(dev, info);
     }
     ~DescriptorSetLayout() noexcept;
-    void destroy() noexcept;
+    void Destroy() noexcept;
 
     DescriptorSetLayout(DescriptorSetLayout &&src) noexcept : NonDispHandle(std::move(src)){};
 
     DescriptorSetLayout &operator=(DescriptorSetLayout &&src) noexcept {
-        destroy();
+        Destroy();
         this->NonDispHandle::operator=(std::move(src));
         return *this;
     }
@@ -1002,7 +1017,7 @@ class DescriptorPool : public internal::NonDispHandle<VkDescriptorPool> {
     DescriptorPool() = default;
     DescriptorPool(const Device &dev, const VkDescriptorPoolCreateInfo &info) { Init(dev, info); }
     ~DescriptorPool() noexcept;
-    void destroy() noexcept;
+    void Destroy() noexcept;
 
     // vkCreateDescriptorPool()
     void Init(const Device &dev, const VkDescriptorPoolCreateInfo &info);
@@ -1035,7 +1050,7 @@ inline VkDescriptorPoolCreateInfo DescriptorPool::CreateInfo(VkDescriptorPoolCre
 class DescriptorSet : public internal::NonDispHandle<VkDescriptorSet> {
   public:
     ~DescriptorSet() noexcept;
-    void destroy() noexcept;
+    void Destroy() noexcept;
 
     explicit DescriptorSet() : NonDispHandle() {}
     explicit DescriptorSet(const Device &dev, DescriptorPool *pool, VkDescriptorSet set) : NonDispHandle(dev.handle(), set) {
@@ -1050,7 +1065,7 @@ class DescriptorSet : public internal::NonDispHandle<VkDescriptorSet> {
 class DescriptorUpdateTemplate : public internal::NonDispHandle<VkDescriptorUpdateTemplate> {
   public:
     ~DescriptorUpdateTemplate() noexcept;
-    void destroy() noexcept;
+    void Destroy() noexcept;
 
     explicit DescriptorUpdateTemplate() : NonDispHandle() {}
     explicit DescriptorUpdateTemplate(const Device &dev, const VkDescriptorUpdateTemplateCreateInfo &info) { Init(dev, info); }
@@ -1063,7 +1078,7 @@ class DescriptorUpdateTemplate : public internal::NonDispHandle<VkDescriptorUpda
 class CommandPool : public internal::NonDispHandle<VkCommandPool> {
   public:
     ~CommandPool() noexcept;
-    void destroy() noexcept;
+    void Destroy() noexcept;
 
     explicit CommandPool() : NonDispHandle() {}
     explicit CommandPool(const Device &dev, const VkCommandPoolCreateInfo &info) { Init(dev, info); }
@@ -1076,12 +1091,11 @@ class CommandPool : public internal::NonDispHandle<VkCommandPool> {
 class CommandBuffer : public internal::Handle<VkCommandBuffer> {
   public:
     ~CommandBuffer() noexcept;
-    void destroy() noexcept;
+    void Destroy() noexcept;
 
     explicit CommandBuffer() : Handle() {}
-    explicit CommandBuffer(const Device &dev, const VkCommandBufferAllocateInfo &info) { Init(dev, info); }
-    explicit CommandBuffer(const Device &dev, const CommandPool &pool,
-                           VkCommandBufferLevel level = VK_COMMAND_BUFFER_LEVEL_PRIMARY) {
+    CommandBuffer(const Device &dev, const VkCommandBufferAllocateInfo &info) { Init(dev, info); }
+    CommandBuffer(const Device &dev, const CommandPool &pool, VkCommandBufferLevel level = VK_COMMAND_BUFFER_LEVEL_PRIMARY) {
         Init(dev, pool, level);
     }
     CommandBuffer(CommandBuffer &&rhs) noexcept : Handle(std::move(rhs)) {
@@ -1177,7 +1191,7 @@ class RenderPass : public internal::NonDispHandle<VkRenderPass> {
     // vkCreateRenderPass2()
     RenderPass(const Device &dev, const VkRenderPassCreateInfo2 &info) { Init(dev, info); }
     ~RenderPass() noexcept;
-    void destroy() noexcept;
+    void Destroy() noexcept;
 
     // vkCreateRenderPass()
     void Init(const Device &dev, const VkRenderPassCreateInfo &info);
@@ -1203,7 +1217,7 @@ class Framebuffer : public internal::NonDispHandle<VkFramebuffer> {
         Init(dev, info);
     }
     ~Framebuffer() noexcept;
-    void destroy() noexcept;
+    void Destroy() noexcept;
 
     // vkCreateFramebuffer()
     void Init(const Device &dev, const VkFramebufferCreateInfo &info);
@@ -1216,7 +1230,7 @@ class SamplerYcbcrConversion : public internal::NonDispHandle<VkSamplerYcbcrConv
     SamplerYcbcrConversion(const Device &dev, VkFormat format) { Init(dev, DefaultConversionInfo(format)); }
     SamplerYcbcrConversion(const Device &dev, const VkSamplerYcbcrConversionCreateInfo &info) { Init(dev, info); }
     ~SamplerYcbcrConversion() noexcept;
-    void destroy() noexcept;
+    void Destroy() noexcept;
 
     void Init(const Device &dev, const VkSamplerYcbcrConversionCreateInfo &info);
     void SetName(const char *name) {
@@ -1242,6 +1256,74 @@ inline VkBufferCreateInfo Buffer::CreateInfo(VkDeviceSize size, VkFlags usage, c
     return info;
 }
 
+class Tensor : public internal::NonDispHandle<VkTensorARM> {
+  public:
+    explicit Tensor();
+    explicit Tensor(const Device &dev, const bool is_copy_tensor = false);
+    explicit Tensor(const Device &dev, const VkTensorDescriptionARM &desc);
+    explicit Tensor(const Device &dev, const VkTensorCreateInfoARM &info);
+    ~Tensor() noexcept;
+    void Destroy() noexcept;
+
+    // vkCreateTensor()
+    void InitNoMem(const Device &dev, const VkTensorCreateInfoARM &info);
+
+    const VkMemoryRequirements2 &GetMemoryReqs();
+    void BindToMem(VkFlags required_flags = 0, VkFlags forbidden_flags = 0);
+    VkFormat Format() const { return description_.format; }
+    uint32_t DimensionCount() const { return description_.dimensionCount; }
+    const VkTensorDescriptionARM &Description() const { return description_; }
+
+  private:
+    const Device *device_ = nullptr;
+    VkTensorCreateInfoARM create_info_;
+    VkTensorDescriptionARM description_;
+    VkTensorMemoryRequirementsInfoARM mem_req_info_;
+    VkMemoryRequirements2 mem_reqs_;
+    std::vector<int64_t> dimensions_;
+    std::vector<int64_t> strides_;
+    vkt::DeviceMemory memory_;
+};
+
+class TensorView : public internal::NonDispHandle<VkTensorViewARM> {
+  public:
+    TensorView() = default;
+    explicit TensorView(const Device &dev, const VkTensorViewCreateInfoARM &info);
+    ~TensorView() noexcept;
+    void Destroy() noexcept;
+
+    // vkCreateTensorViewARM
+    void Init(const Device &dev, const VkTensorViewCreateInfoARM &info);
+
+  private:
+    const Device *device_ = nullptr;
+    VkTensorViewCreateInfoARM create_info_;
+};
+
+class DataGraphPipelineSession : public internal::NonDispHandle<VkDataGraphPipelineSessionARM> {
+  public:
+    explicit DataGraphPipelineSession(const Device &dev, const VkDataGraphPipelineSessionCreateInfoARM &info);
+    ~DataGraphPipelineSession() noexcept;
+    void Destroy() noexcept;
+
+    // CreateDataGraphPipelineSessionARM
+    void Init(const Device &dev);
+
+    void GetMemoryReqs();
+    void AllocSessionMem(std::vector<vkt::DeviceMemory> &device_mem, bool is_protected = false, size_t scale_factor = 1,
+                         int32_t size_modifier = 0);
+    size_t BindPointsCount() const { return bind_point_reqs_.size(); }
+
+    const std::vector<VkDataGraphPipelineSessionBindPointRequirementARM> &BindPointReqs() const { return bind_point_reqs_; }
+    const std::vector<VkMemoryRequirements2> &MemReqs() const { return mem_reqs_; }
+
+  private:
+    const Device *device_ = nullptr;
+    VkDataGraphPipelineSessionCreateInfoARM create_info_;
+    std::vector<VkDataGraphPipelineSessionBindPointRequirementARM> bind_point_reqs_;
+    std::vector<VkMemoryRequirements2> mem_reqs_;
+};
+
 inline VkEventCreateInfo Event::CreateInfo(VkFlags flags) {
     VkEventCreateInfo info = vku::InitStructHelper();
     info.flags = flags;
@@ -1326,7 +1408,7 @@ class Swapchain : public internal::NonDispHandle<VkSwapchainKHR> {
     Swapchain(Swapchain &&rhs) noexcept : NonDispHandle(std::move(rhs)) {}
     Swapchain &operator=(Swapchain &&) = default;
     ~Swapchain() noexcept;
-    void destroy() noexcept;
+    void Destroy() noexcept;
 
     void Init(const Device &dev, const VkSwapchainCreateInfoKHR &info);
     void SetName(const char *name) { NonDispHandle<VkSwapchainKHR>::SetName(VK_OBJECT_TYPE_SWAPCHAIN_KHR, name); }
@@ -1341,7 +1423,7 @@ class Swapchain : public internal::NonDispHandle<VkSwapchainKHR> {
 class IndirectCommandsLayout : public internal::NonDispHandle<VkIndirectCommandsLayoutEXT> {
   public:
     ~IndirectCommandsLayout() noexcept;
-    void destroy() noexcept;
+    void Destroy() noexcept;
 
     explicit IndirectCommandsLayout() : NonDispHandle() {}
     explicit IndirectCommandsLayout(const Device &dev, const VkIndirectCommandsLayoutCreateInfoEXT &info) { Init(dev, info); }
@@ -1351,7 +1433,7 @@ class IndirectCommandsLayout : public internal::NonDispHandle<VkIndirectCommands
 class IndirectExecutionSet : public internal::NonDispHandle<VkIndirectExecutionSetEXT> {
   public:
     ~IndirectExecutionSet() noexcept;
-    void destroy() noexcept;
+    void Destroy() noexcept;
 
     explicit IndirectExecutionSet() : NonDispHandle() {}
     explicit IndirectExecutionSet(const Device &dev, const VkIndirectExecutionSetCreateInfoEXT &info) { Init(dev, info); }
@@ -1414,4 +1496,7 @@ class Surface {
     VkInstance instance_ = VK_NULL_HANDLE;
     VkSurfaceKHR handle_ = VK_NULL_HANDLE;
 };
+
+const VkAllocationCallbacks *DefaultAllocator();
+
 }  // namespace vkt
