@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 2023-2024 Valve Corporation
- * Copyright (c) 2023-2024 LunarG, Inc.
+ * Copyright (c) 2023-2025 Valve Corporation
+ * Copyright (c) 2023-2025 LunarG, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -80,7 +80,7 @@ class RayTracingPipelineHelper {
     }
 
     template <typename Test, typename OverrideFunc>
-    static void OneshotPositiveTest(Test& test, const OverrideFunc& info_override, const VkFlags message_flag_mask = kErrorBit) {
+    static void OneshotPositiveTest(Test& test, const OverrideFunc& info_override) {
         RayTracingPipelineHelper helper(test);
         info_override(helper);
 
