@@ -21,22 +21,20 @@
 #include "../framework/thread_helper.h"
 
 void VkBestPracticesLayerTest::InitBestPracticesFramework(const char *vendor_checks_to_enable) {
-    // Enable the vendor-specific checks spcified by vendor_checks_to_enable
-    const char *input_values[] = {vendor_checks_to_enable};
-    const VkLayerSettingEXT settings[] = {{OBJECT_LAYER_NAME, "enables", VK_LAYER_SETTING_TYPE_STRING_EXT,
-                                           static_cast<uint32_t>(std::size(input_values)), input_values}};
+    const VkLayerSettingEXT settings = {OBJECT_LAYER_NAME, vendor_checks_to_enable, VK_LAYER_SETTING_TYPE_BOOL32_EXT, 1, &kVkTrue};
+    const VkLayerSettingsCreateInfoEXT layer_settings_create_info{VK_STRUCTURE_TYPE_LAYER_SETTINGS_CREATE_INFO_EXT, nullptr, 1,
+                                                                  &settings};
 
-    const VkLayerSettingsCreateInfoEXT layer_settings_create_info{VK_STRUCTURE_TYPE_LAYER_SETTINGS_CREATE_INFO_EXT, nullptr,
-                                                                  static_cast<uint32_t>(std::size(settings)), settings};
-
-    features_.pNext = &layer_settings_create_info;
+    if (vendor_checks_to_enable) {
+        features_.pNext = &layer_settings_create_info;
+    }
 
     AddRequiredExtensions(VK_EXT_VALIDATION_FEATURES_EXTENSION_NAME);
     InitFramework(&features_);
 }
 
-void VkBestPracticesLayerTest::InitBestPractices(const char *ValidationChecksToEnable) {
-    RETURN_IF_SKIP(InitBestPracticesFramework(ValidationChecksToEnable));
+void VkBestPracticesLayerTest::InitBestPractices(const char* vendor_checks_to_enable) {
+    RETURN_IF_SKIP(InitBestPracticesFramework(vendor_checks_to_enable));
     RETURN_IF_SKIP(InitState());
 }
 
@@ -84,97 +82,30 @@ TEST_F(VkBestPracticesLayerTest, ReturnCodes) {
     m_errorMonitor->VerifyFound();
 }
 
-TEST_F(VkBestPracticesLayerTest, UseDeprecatedInstanceExtensions) {
-    TEST_DESCRIPTION("Create an instance with a deprecated extension.");
-
-    // We need to explicitly allow promoted extensions to be enabled as this test relies on this behavior
-    AllowPromotedExtensions();
-
+TEST_F(VkBestPracticesLayerTest, SpecialUseExtensionsInstance) {
     SetTargetApiVersion(VK_API_VERSION_1_1);
-    AddRequiredExtensions(VK_KHR_GET_PHYSICAL_DEVICE_PROPERTIES_2_EXTENSION_NAME);
-    RETURN_IF_SKIP(InitBestPracticesFramework());
-    if (IsPlatformMockICD()) {
-        GTEST_SKIP() << "Test not supported by MockICD - currently can't create 2 concurrent instances";
+    if (!InstanceExtensionSupported(VK_GOOGLE_SURFACELESS_QUERY_EXTENSION_NAME)) {
+        GTEST_SKIP() << "Did not find required instance extension";
     }
+    m_instance_extension_names.emplace_back(VK_KHR_SURFACE_EXTENSION_NAME);
+    m_instance_extension_names.emplace_back(VK_GOOGLE_SURFACELESS_QUERY_EXTENSION_NAME);
 
-    // Create a 1.1 vulkan instance and request an extension promoted to core in 1.1
-    if (IsExtensionsEnabled(VK_EXT_DEBUG_REPORT_EXTENSION_NAME)) {
-        // Extra error if VK_EXT_debug_report is used on Android still
-        m_errorMonitor->SetDesiredWarning("BestPractices-deprecated-extension");
-    }
+    const VkLayerSettingEXT settings = {OBJECT_LAYER_NAME, "validate_best_practices", VK_LAYER_SETTING_TYPE_BOOL32_EXT, 1,
+                                        &kVkTrue};
+    const VkLayerSettingsCreateInfoEXT layer_settings_create_info{VK_STRUCTURE_TYPE_LAYER_SETTINGS_CREATE_INFO_EXT, nullptr, 1,
+                                                                  &settings};
 
-    m_errorMonitor->SetDesiredWarning("BestPractices-deprecated-extension");  // VK_KHR_get_physical_device_properties2
-    m_errorMonitor->SetDesiredWarning("BestPractices-deprecated-extension");  // VK_EXT_validation_features
-    m_errorMonitor->SetDesiredWarning("BestPractices-specialuse-extension");  // VK_EXT_debug_utils
-    m_errorMonitor->SetDesiredWarning("BestPractices-specialuse-extension");  // VK_EXT_validation_features
+    // GetDebugCreateInfo() is last pNext chain set in GetInstanceCreateInfo()
+    const_cast<VkDebugUtilsMessengerCreateInfoEXT*>(m_errorMonitor->GetDebugCreateInfo())->pNext = &layer_settings_create_info;
 
-    VkInstance dummy = VK_NULL_HANDLE;
-    auto features = features_;
+    Monitor().SetDesiredWarning("BestPractices-specialuse-extension");
+    VkInstance dummy_instance;
     auto ici = GetInstanceCreateInfo();
-    features.pNext = ici.pNext;
-    ici.pNext = &features;
-    vk::CreateInstance(&ici, nullptr, &dummy);
-    m_errorMonitor->VerifyFound();
-
-    VkApplicationInfo new_info{};
-    new_info.apiVersion = VK_API_VERSION_1_0;
-    new_info.pApplicationName = ici.pApplicationInfo->pApplicationName;
-    new_info.applicationVersion = ici.pApplicationInfo->applicationVersion;
-    new_info.pEngineName = ici.pApplicationInfo->pEngineName;
-    new_info.engineVersion = ici.pApplicationInfo->engineVersion;
-    ici.pApplicationInfo = &new_info;
-
-    // Create a 1.0 vulkan instance and request an extension promoted to core in 1.1
-    if (IsExtensionsEnabled(VK_EXT_DEBUG_REPORT_EXTENSION_NAME)) {
-        // Extra error if VK_EXT_debug_report is used on Android still
-        m_errorMonitor->SetDesiredWarning("BestPractices-deprecated-extension");
-    }
-    m_errorMonitor->SetUnexpectedError("khronos-Validation-debug-build-warning-message");
-    m_errorMonitor->SetUnexpectedError("khronos-Validation-fine-grained-locking-warning-message");
-    m_errorMonitor->SetDesiredWarning("BestPractices-deprecated-extension");  // VK_EXT_validation_features
-    m_errorMonitor->SetDesiredWarning("BestPractices-specialuse-extension");  // VK_EXT_debug_utils
-    m_errorMonitor->SetDesiredWarning("BestPractices-specialuse-extension");  // VK_EXT_validation_features
-    vk::CreateInstance(&ici, nullptr, &dummy);
-    m_errorMonitor->VerifyFound();
-    if (dummy != VK_NULL_HANDLE) {
-        vk::DestroyInstance(dummy, nullptr);
-    }
+    vk::CreateInstance(&ici, nullptr, &dummy_instance);
+    Monitor().VerifyFound();
 }
 
-TEST_F(VkBestPracticesLayerTest, UseDeprecatedDeviceExtensions) {
-    TEST_DESCRIPTION("Create a device with a deprecated extension.");
-
-    // We need to explicitly allow promoted extensions to be enabled as this test relies on this behavior
-    AllowPromotedExtensions();
-
-    SetTargetApiVersion(VK_API_VERSION_1_2);
-    AddRequiredExtensions(VK_KHR_BUFFER_DEVICE_ADDRESS_EXTENSION_NAME);
-    RETURN_IF_SKIP(InitBestPracticesFramework());
-
-    VkDevice local_device;
-    VkDeviceCreateInfo dev_info = vku::InitStructHelper();
-    VkDeviceQueueCreateInfo queue_info = vku::InitStructHelper();
-    queue_info.queueFamilyIndex = 0;
-    queue_info.queueCount = 1;
-    float qp = 1;
-    queue_info.pQueuePriorities = &qp;
-    dev_info.queueCreateInfoCount = 1;
-    dev_info.pQueueCreateInfos = &queue_info;
-    dev_info.enabledLayerCount = 0;
-    dev_info.ppEnabledLayerNames = NULL;
-    dev_info.enabledExtensionCount = m_device_extension_names.size();
-    dev_info.ppEnabledExtensionNames = m_device_extension_names.data();
-
-    // One for VK_KHR_buffer_device_address
-    // One for the dependency extension VK_KHR_device_group
-    m_errorMonitor->SetDesiredWarning("BestPractices-deprecated-extension", 2);
-    vk::CreateDevice(this->Gpu(), &dev_info, NULL, &local_device);
-    m_errorMonitor->VerifyFound();
-}
-
-TEST_F(VkBestPracticesLayerTest, SpecialUseExtensions) {
-    TEST_DESCRIPTION("Create a device with a 'specialuse' extension.");
-
+TEST_F(VkBestPracticesLayerTest, SpecialUseExtensionsDevice) {
     AddRequiredExtensions(VK_EXT_DEPTH_CLIP_ENABLE_EXTENSION_NAME);
     RETURN_IF_SKIP(InitBestPracticesFramework());
 
@@ -414,17 +345,9 @@ TEST_F(VkBestPracticesLayerTest, MSImageRequiresMemory) {
 
 TEST_F(VkBestPracticesLayerTest, AttachmentShouldNotBeTransient) {
     TEST_DESCRIPTION("Test for non-lazy multisampled images");
-
     RETURN_IF_SKIP(InitBestPracticesFramework());
     RETURN_IF_SKIP(InitState());
 
-    m_errorMonitor->SetDesiredFailureMsg(kPerformanceWarningBit,
-                                         "BestPractices-vkCreateFramebuffer-attachment-should-not-be-transient");
-
-    m_errorMonitor->SetAllowedFailureMsg("BestPractices-vkBindImageMemory-small-dedicated-allocation");
-    m_errorMonitor->SetAllowedFailureMsg("BestPractices-vkBindImageMemory-non-lazy-transient-image");
-    m_errorMonitor->SetAllowedFailureMsg("BestPractices-AMD-vkImage-AvoidGeneral");
-
     VkAttachmentDescription attachment{};
     attachment.samples = VK_SAMPLE_COUNT_1_BIT;
     attachment.loadOp = VK_ATTACHMENT_LOAD_OP_LOAD;
@@ -447,8 +370,9 @@ TEST_F(VkBestPracticesLayerTest, AttachmentShouldNotBeTransient) {
     vkt::Image image(*m_device, image_info, vkt::set_layout);
     vkt::ImageView image_view = image.CreateView();
 
+    m_errorMonitor->SetDesiredFailureMsg(kPerformanceWarningBit,
+                                         "BestPractices-vkCreateFramebuffer-attachment-should-not-be-transient");
     vkt::Framebuffer fb(*m_device, rp, 1, &image_view.handle(), 1920, 1080);
-
     m_errorMonitor->VerifyFound();
 }
 
@@ -592,8 +516,7 @@ TEST_F(VkBestPracticesLayerTest, ClearAttachmentsAfterLoadSecondary) {
     render_pass_begin_info.renderPass = rp;
     render_pass_begin_info.framebuffer = fb;
     // need full clear
-    render_pass_begin_info.renderArea.extent.width = m_width;
-    render_pass_begin_info.renderArea.extent.height = m_height;
+    render_pass_begin_info.renderArea.extent = {m_width, m_height};
 
     // Plain clear after load.
     m_command_buffer.BeginRenderPass(render_pass_begin_info);
@@ -688,8 +611,6 @@ TEST_F(VkBestPracticesLayerTest, TripleBufferingTest) {
     AddSurfaceExtension();
     RETURN_IF_SKIP(InitBestPracticesFramework());
     RETURN_IF_SKIP(InitState());
-    m_errorMonitor->SetDesiredFailureMsg(kPerformanceWarningBit,
-                                         "BestPractices-vkCreateSwapchainKHR-suboptimal-swapchain-image-count");
     RETURN_IF_SKIP(InitSurface());
     InitSwapchainInfo();
 
@@ -728,10 +649,14 @@ TEST_F(VkBestPracticesLayerTest, TripleBufferingTest) {
     swapchain_create_info.clipped = VK_FALSE;
     swapchain_create_info.oldSwapchain = 0;
 
+    m_errorMonitor->SetDesiredFailureMsg(kPerformanceWarningBit,
+                                         "BestPractices-vkCreateSwapchainKHR-suboptimal-swapchain-image-count");
+    // skip core checks
     m_errorMonitor->SetAllowedFailureMsg("VUID-VkSwapchainCreateInfoKHR-presentMode-02839");
     m_swapchain.Init(*m_device, swapchain_create_info);
     m_errorMonitor->VerifyFound();
 
+    // Lazy way to not query
     m_errorMonitor->SetAllowedFailureMsg("VUID-VkSwapchainCreateInfoKHR-presentMode-02839");
     swapchain_create_info.minImageCount = 3;
     m_swapchain.Init(*m_device, swapchain_create_info);
@@ -790,14 +715,14 @@ TEST_F(VkBestPracticesLayerTest, SwapchainCreationTest) {
 
     // present mode might not be supported
     m_errorMonitor->SetAllowedFailureMsg("VUID-VkSwapchainCreateInfoKHR-presentMode-01281");
-
+    // Lazy way to not query
     m_errorMonitor->SetAllowedFailureMsg("VUID-VkSwapchainCreateInfoKHR-presentMode-02839");
     m_swapchain.Init(*m_device, swapchain_create_info);
     m_errorMonitor->VerifyFound();
 
-    swapchain_create_info.presentMode = VK_PRESENT_MODE_FIFO_KHR;
-
+    // Lazy way to not query
     m_errorMonitor->SetAllowedFailureMsg("VUID-VkSwapchainCreateInfoKHR-presentMode-02839");
+    swapchain_create_info.presentMode = VK_PRESENT_MODE_FIFO_KHR;
     m_swapchain.Init(*m_device, swapchain_create_info);
 }
 
@@ -1037,17 +962,13 @@ TEST_F(VkBestPracticesLayerTest, TransitionFromUndefinedToReadOnly) {
     img_barrier.oldLayout = VK_IMAGE_LAYOUT_UNDEFINED;
     img_barrier.newLayout = VK_IMAGE_LAYOUT_SHADER_READ_ONLY_OPTIMAL;
     img_barrier.image = image;
-    img_barrier.subresourceRange.aspectMask = VK_IMAGE_ASPECT_COLOR_BIT;
-    img_barrier.subresourceRange.baseArrayLayer = 0;
-    img_barrier.subresourceRange.baseMipLevel = 0;
-    img_barrier.subresourceRange.layerCount = 1;
-    img_barrier.subresourceRange.levelCount = 1;
+    img_barrier.subresourceRange = {VK_IMAGE_ASPECT_COLOR_BIT, 0, 1, 0, 1};
 
     m_command_buffer.Begin();
 
     vk::CmdClearColorImage(m_command_buffer, image, VK_IMAGE_LAYOUT_GENERAL, &color_clear_value, 1, &clear_range);
 
-    m_errorMonitor->SetAllowedFailureMsg("VUID-vkCmdPipelineBarrier-pImageMemoryBarriers-02820");
+    m_errorMonitor->SetAllowedFailureMsg("VUID-vkCmdPipelineBarrier-pImageMemoryBarriers-02820");  // skip core checks
     m_errorMonitor->SetDesiredWarning("BestPractices-ImageMemoryBarrier-TransitionUndefinedToReadOnly");
     vk::CmdPipelineBarrier(m_command_buffer, VK_PIPELINE_STAGE_BOTTOM_OF_PIPE_BIT, VK_PIPELINE_STAGE_TOP_OF_PIPE_BIT, 0, 0, nullptr,
                            0, nullptr, 1, &img_barrier);
@@ -1058,16 +979,12 @@ TEST_F(VkBestPracticesLayerTest, TransitionFromUndefinedToReadOnly) {
 
 TEST_F(VkBestPracticesLayerTest, OverAllocateFromDescriptorPool) {
     TEST_DESCRIPTION("Attempt to allocate more sets and descriptors than descriptor pool has available.");
-
     SetTargetApiVersion(VK_API_VERSION_1_1);
     RETURN_IF_SKIP(InitBestPracticesFramework());
     RETURN_IF_SKIP(InitState());
     InitRenderTarget();
 
-    VkDescriptorPoolSize ds_type_count = {};
-    ds_type_count.type = VK_DESCRIPTOR_TYPE_SAMPLER;
-    ds_type_count.descriptorCount = 8;
-
+    VkDescriptorPoolSize ds_type_count = {VK_DESCRIPTOR_TYPE_SAMPLER, 8};
     VkDescriptorPoolCreateInfo ds_pool_ci = vku::InitStructHelper();
     ds_pool_ci.flags = 0;
     ds_pool_ci.maxSets = 3;
@@ -1075,15 +992,7 @@ TEST_F(VkBestPracticesLayerTest, OverAllocateFromDescriptorPool) {
     ds_pool_ci.pPoolSizes = &ds_type_count;
 
     vkt::DescriptorPool ds_pool(*m_device, ds_pool_ci);
-
-    VkDescriptorSetLayoutBinding dsl_binding_samp = {};
-    dsl_binding_samp.binding = 0;
-    dsl_binding_samp.descriptorType = VK_DESCRIPTOR_TYPE_SAMPLER;
-    dsl_binding_samp.descriptorCount = 1;
-    dsl_binding_samp.stageFlags = VK_SHADER_STAGE_ALL;
-    dsl_binding_samp.pImmutableSamplers = NULL;
-
-    const vkt::DescriptorSetLayout ds_layout_samp(*m_device, {dsl_binding_samp});
+    const vkt::DescriptorSetLayout ds_layout_samp(*m_device, {0, VK_DESCRIPTOR_TYPE_SAMPLER, 1, VK_SHADER_STAGE_ALL, nullptr});
 
     // Try to allocate 2 sets when pool only has 1 set
     VkDescriptorSet descriptor_sets[4];
@@ -1100,16 +1009,12 @@ TEST_F(VkBestPracticesLayerTest, OverAllocateFromDescriptorPool) {
 
 TEST_F(VkBestPracticesLayerTest, OverAllocateTypeFromDescriptorPool) {
     TEST_DESCRIPTION("Attempt to allocate more sets and descriptors than descriptor pool has available.");
-
     SetTargetApiVersion(VK_API_VERSION_1_1);
     RETURN_IF_SKIP(InitBestPracticesFramework());
     RETURN_IF_SKIP(InitState());
     InitRenderTarget();
 
-    VkDescriptorPoolSize ds_type_count = {};
-    ds_type_count.type = VK_DESCRIPTOR_TYPE_SAMPLER;
-    ds_type_count.descriptorCount = 3;
-
+    VkDescriptorPoolSize ds_type_count = {VK_DESCRIPTOR_TYPE_SAMPLER, 3};
     VkDescriptorPoolCreateInfo ds_pool_ci = vku::InitStructHelper();
     ds_pool_ci.flags = 0;
     ds_pool_ci.maxSets = 4;
@@ -1118,14 +1023,7 @@ TEST_F(VkBestPracticesLayerTest, OverAllocateTypeFromDescriptorPool) {
 
     vkt::DescriptorPool ds_pool(*m_device, ds_pool_ci);
 
-    VkDescriptorSetLayoutBinding dsl_binding_samp = {};
-    dsl_binding_samp.binding = 0;
-    dsl_binding_samp.descriptorType = VK_DESCRIPTOR_TYPE_SAMPLER;
-    dsl_binding_samp.descriptorCount = 1;
-    dsl_binding_samp.stageFlags = VK_SHADER_STAGE_ALL;
-    dsl_binding_samp.pImmutableSamplers = NULL;
-
-    const vkt::DescriptorSetLayout ds_layout_samp(*m_device, {dsl_binding_samp});
+    const vkt::DescriptorSetLayout ds_layout_samp(*m_device, {0, VK_DESCRIPTOR_TYPE_SAMPLER, 1, VK_SHADER_STAGE_ALL, nullptr});
 
     // Try to allocate 2 sets when pool only has 1 set
     VkDescriptorSet descriptor_sets[4];
@@ -1193,8 +1091,7 @@ TEST_F(VkBestPracticesLayerTest, RenderPassClearWithoutLoadOpClear) {
     begin_info.clearValueCount = 1;  // Pass one clearValue, in conflict with attachment.loadOp = VK_ATTACHMENT_LOAD_OP_DONT_CARE
     begin_info.pClearValues = &cv;
     begin_info.renderPass = rp;
-    begin_info.renderArea.extent.width = w;
-    begin_info.renderArea.extent.height = h;
+    begin_info.renderArea.extent = {w, h};
     begin_info.framebuffer = fb;
 
     m_errorMonitor->SetDesiredWarning("BestPractices-ClearValueWithoutLoadOpClear");
@@ -1265,8 +1162,7 @@ TEST_F(VkBestPracticesLayerTest, RenderPassClearValueCountHigherThanAttachmentCo
                                      // second clearValue will be ignored
     begin_info.pClearValues = cv;
     begin_info.renderPass = rp;
-    begin_info.renderArea.extent.width = w;
-    begin_info.renderArea.extent.height = h;
+    begin_info.renderArea.extent = {w, h};
     begin_info.framebuffer = fb;
 
     m_errorMonitor->SetDesiredWarning("BestPractices-ClearValueCountHigherThanAttachmentCount");
@@ -1352,45 +1248,6 @@ TEST_F(VkBestPracticesLayerTest, DontCareThenLoad) {
     m_default_queue->Wait();
 }
 
-TEST_F(VkBestPracticesLayerTest, LoadDeprecatedExtension) {
-    TEST_DESCRIPTION("Test for loading a vk1.3 deprecated extension with a 1.3 instance on a 1.2 or less device");
-
-    SetTargetApiVersion(VK_API_VERSION_1_3);
-
-    RETURN_IF_SKIP(InitBestPracticesFramework());
-
-    const char *extension = VK_KHR_SYNCHRONIZATION_2_EXTENSION_NAME;
-
-    if (!DeviceExtensionSupported(extension)) {
-        GTEST_SKIP() << extension << " not supported.";
-    }
-
-    VkDeviceQueueCreateInfo qci = vku::InitStructHelper();
-    qci.queueFamilyIndex = 0;
-    float priority = 1;
-    qci.pQueuePriorities = &priority;
-    qci.queueCount = 1;
-
-    VkDeviceCreateInfo dev_info = vku::InitStructHelper();
-    dev_info.queueCreateInfoCount = 1;
-    dev_info.pQueueCreateInfos = &qci;
-    dev_info.enabledExtensionCount = 1;
-    dev_info.ppEnabledExtensionNames = &extension;
-
-    m_errorMonitor->SetDesiredWarning("BestPractices-deprecated-extension");
-    // api version != device version
-    m_errorMonitor->SetAllowedFailureMsg("BestPractices-vkCreateDevice-API-version-mismatch");
-
-    VkDevice device = VK_NULL_HANDLE;
-    vk::CreateDevice(Gpu(), &dev_info, nullptr, &device);
-
-    if (DeviceValidationVersion() >= VK_API_VERSION_1_3) {
-        m_errorMonitor->VerifyFound();
-    }
-
-    if (device) vk::DestroyDevice(device, nullptr);
-}
-
 TEST_F(VkBestPracticesLayerTest, ExclusiveImageMultiQueueUsage) {
     TEST_DESCRIPTION("Test for using a queue exclusive image on multiple queues");
 
@@ -1460,8 +1317,7 @@ TEST_F(VkBestPracticesLayerTest, ExclusiveImageMultiQueueUsage) {
     begin_info.clearValueCount = 1;
     begin_info.pClearValues = &cv;
     begin_info.renderPass = rp;
-    begin_info.renderArea.extent.width = w;
-    begin_info.renderArea.extent.height = h;
+    begin_info.renderArea.extent = {w, h};
     begin_info.framebuffer = fb;
 
     // Prepare compute
@@ -1602,11 +1458,7 @@ TEST_F(VkBestPracticesLayerTest, ImageMemoryBarrierAccessLayoutCombinations) {
     img_barrier.oldLayout = VK_IMAGE_LAYOUT_UNDEFINED;
     img_barrier.newLayout = VK_IMAGE_LAYOUT_TRANSFER_DST_OPTIMAL;
     img_barrier.image = image;
-    img_barrier.subresourceRange.aspectMask = VK_IMAGE_ASPECT_COLOR_BIT;
-    img_barrier.subresourceRange.baseArrayLayer = 0;
-    img_barrier.subresourceRange.baseMipLevel = 0;
-    img_barrier.subresourceRange.layerCount = 1;
-    img_barrier.subresourceRange.levelCount = 1;
+    img_barrier.subresourceRange = {VK_IMAGE_ASPECT_COLOR_BIT, 0, 1, 0, 1};
 
     m_command_buffer.Begin();
 
@@ -1648,15 +1500,11 @@ TEST_F(VkBestPracticesLayerTest, ImageMemoryBarrierAccessLayoutCombinations) {
         img_barrier2.srcAccessMask = 0;
         img_barrier2.oldLayout = VK_IMAGE_LAYOUT_UNDEFINED;
         img_barrier2.image = image;
-        img_barrier2.subresourceRange.aspectMask = VK_IMAGE_ASPECT_COLOR_BIT;
-        img_barrier2.subresourceRange.baseArrayLayer = 0;
-        img_barrier2.subresourceRange.baseMipLevel = 0;
-        img_barrier2.subresourceRange.layerCount = 1;
-        img_barrier2.subresourceRange.levelCount = 1;
+        img_barrier2.subresourceRange = {VK_IMAGE_ASPECT_COLOR_BIT, 0, 1, 0, 1};
 
         img_barrier2.dstAccessMask = VK_ACCESS_INPUT_ATTACHMENT_READ_BIT;
         img_barrier2.newLayout = VK_IMAGE_LAYOUT_TRANSFER_DST_OPTIMAL;
-        m_errorMonitor->SetAllowedFailureMsg("VUID-VkImageMemoryBarrier2-dstAccessMask-03903");
+        m_errorMonitor->SetAllowedFailureMsg("VUID-VkImageMemoryBarrier2-dstAccessMask-03903");  // skip core checks
         m_errorMonitor->SetDesiredWarning("BestPractices-ImageBarrierAccessLayout");
         m_command_buffer.BarrierKHR(img_barrier2);
         m_errorMonitor->VerifyFound();
@@ -1664,7 +1512,7 @@ TEST_F(VkBestPracticesLayerTest, ImageMemoryBarrierAccessLayoutCombinations) {
         // make sure bits above UINT32_MAX are detected
         img_barrier2.newLayout = VK_IMAGE_LAYOUT_PRESENT_SRC_KHR;
         img_barrier2.dstAccessMask = VK_ACCESS_2_SHADER_STORAGE_WRITE_BIT;
-        m_errorMonitor->SetAllowedFailureMsg("VUID-VkImageMemoryBarrier2-dstAccessMask-03907");
+        m_errorMonitor->SetAllowedFailureMsg("VUID-VkImageMemoryBarrier2-dstAccessMask-03907");  // skip core checks
         m_errorMonitor->SetDesiredWarning("BestPractices-ImageBarrierAccessLayout");
         m_command_buffer.BarrierKHR(img_barrier2);
         m_errorMonitor->VerifyFound();
@@ -1710,7 +1558,7 @@ TEST_F(VkBestPracticesLayerTest, NoCreateSwapchainPresentModes) {
 
     RETURN_IF_SKIP(InitState());
     RETURN_IF_SKIP(InitSurface());
-    m_errorMonitor->SetAllowedFailureMsg("VUID-VkSwapchainCreateInfoKHR-presentMode-02839");
+    m_errorMonitor->SetAllowedFailureMsg("VUID-VkSwapchainCreateInfoKHR-presentMode-02839");  // skip core checks
     m_errorMonitor->SetDesiredWarning("BestPractices-vkCreateSwapchainKHR-no-VkSwapchainPresentModesCreateInfoKHR-provided");
     m_swapchain = CreateSwapchain(m_surface.Handle(), VK_IMAGE_USAGE_COLOR_ATTACHMENT_BIT, VK_SURFACE_TRANSFORM_IDENTITY_BIT_KHR);
     m_errorMonitor->VerifyFound();
@@ -2267,20 +2115,13 @@ TEST_F(VkBestPracticesLayerTest, MutableDescriptors) {
     RETURN_IF_SKIP(InitBestPractices());
     VkDescriptorType descriptor_types[] = {VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER, VK_DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER};
 
-    VkMutableDescriptorTypeListEXT mutable_descriptor_type_lists[2] = {};
-    mutable_descriptor_type_lists[0].descriptorTypeCount = 1;
-    mutable_descriptor_type_lists[0].pDescriptorTypes = &descriptor_types[0];
-    mutable_descriptor_type_lists[1].descriptorTypeCount = 1;
-    mutable_descriptor_type_lists[1].pDescriptorTypes = &descriptor_types[1];
+    VkMutableDescriptorTypeListEXT mutable_descriptor_type_lists[2] = {{1, &descriptor_types[0]}, {1, &descriptor_types[1]}};
 
     VkMutableDescriptorTypeCreateInfoEXT mdtci = vku::InitStructHelper();
     mdtci.mutableDescriptorTypeListCount = 2;
     mdtci.pMutableDescriptorTypeLists = mutable_descriptor_type_lists;
 
-    VkDescriptorPoolSize pool_size = {};
-    pool_size.type = VK_DESCRIPTOR_TYPE_MUTABLE_EXT;
-    pool_size.descriptorCount = 4;
-
+    VkDescriptorPoolSize pool_size = {VK_DESCRIPTOR_TYPE_MUTABLE_EXT, 4};
     VkDescriptorPoolCreateInfo ds_pool_ci = vku::InitStructHelper(&mdtci);
     ds_pool_ci.maxSets = 2;
     ds_pool_ci.poolSizeCount = 1;
