@@ -14,7 +14,6 @@
 #include "../framework/layer_validation_tests.h"
 #include "../framework/pipeline_helper.h"
 #include "../framework/buffer_helper.h"
-#include "../framework/ray_tracing_objects.h"
 
 class NegativeGpuAVIndirectBuffer : public GpuAVTest {};
 
@@ -410,7 +409,7 @@ TEST_F(NegativeGpuAVIndirectBuffer, Mesh) {
     m_command_buffer.EndRenderPass();
     m_command_buffer.End();
 
-    if (mesh_shader_props.maxMeshWorkGroupCount[0] < std::numeric_limits<uint32_t>::max()) {
+    if (mesh_shader_props.maxMeshWorkGroupCount[0] < vvl::kU32Max) {
         draw_ptr[8] = mesh_shader_props.maxMeshWorkGroupCount[0] + 1;
         m_errorMonitor->SetDesiredError("VUID-VkDrawMeshTasksIndirectCommandEXT-TaskEXT-07326");
         if (mesh_shader_props.maxMeshWorkGroupCount[0] + 1 >= mesh_shader_props.maxMeshWorkGroupTotalCount) {
@@ -422,7 +421,7 @@ TEST_F(NegativeGpuAVIndirectBuffer, Mesh) {
     }
 
     // Set y in second draw
-    if (mesh_shader_props.maxMeshWorkGroupCount[1] < std::numeric_limits<uint32_t>::max()) {
+    if (mesh_shader_props.maxMeshWorkGroupCount[1] < vvl::kU32Max) {
         draw_ptr[5] = mesh_shader_props.maxMeshWorkGroupCount[1] + 1;
         m_errorMonitor->SetDesiredError("VUID-VkDrawMeshTasksIndirectCommandEXT-TaskEXT-07327");
         if (mesh_shader_props.maxMeshWorkGroupCount[1] + 1 >= mesh_shader_props.maxMeshWorkGroupTotalCount) {
@@ -434,7 +433,7 @@ TEST_F(NegativeGpuAVIndirectBuffer, Mesh) {
     }
 
     // Set z in first draw
-    if (mesh_shader_props.maxMeshWorkGroupCount[2] < std::numeric_limits<uint32_t>::max()) {
+    if (mesh_shader_props.maxMeshWorkGroupCount[2] < vvl::kU32Max) {
         draw_ptr[2] = mesh_shader_props.maxMeshWorkGroupCount[2] + 1;
         m_errorMonitor->SetDesiredError("VUID-VkDrawMeshTasksIndirectCommandEXT-TaskEXT-07328");
         if (mesh_shader_props.maxMeshWorkGroupCount[2] + 1 >= mesh_shader_props.maxMeshWorkGroupTotalCount) {
@@ -446,7 +445,7 @@ TEST_F(NegativeGpuAVIndirectBuffer, Mesh) {
     }
 // total count can end up being really high, draw takes too long and times out
 #if 0
-    if (mesh_shader_props.maxMeshWorkGroupTotalCount < std::numeric_limits<uint32_t>::max()) {
+    if (mesh_shader_props.maxMeshWorkGroupTotalCount < vvl::kU32Max) {
         const uint32_t half_total = (mesh_shader_props.maxMeshWorkGroupTotalCount + 2) / 2;
         if (half_total < mesh_shader_props.maxMeshWorkGroupCount[0]) {
             draw_ptr[2] = 1;
@@ -465,6 +464,10 @@ TEST_F(NegativeGpuAVIndirectBuffer, Mesh) {
 #endif
 }
 
+// https://github.com/KhronosGroup/Vulkan-ValidationLayers/issues/11028
+// This test will get invalid vkCmdDrawMeshTasksIndirectCountEXT
+// to be executed by the driver, likely killing it.
+// Until GPU-AV manages to stop commands, this will stay disabled.
 TEST_F(NegativeGpuAVIndirectBuffer, DISABLED_MeshTask) {
     TEST_DESCRIPTION("GPU validation: Validate DrawMeshTasksIndirect* DrawBuffer contents");
     SetTargetApiVersion(VK_API_VERSION_1_3);
@@ -534,7 +537,7 @@ TEST_F(NegativeGpuAVIndirectBuffer, DISABLED_MeshTask) {
     m_command_buffer.End();
 
     // Set x in second draw
-    if (mesh_shader_props.maxTaskWorkGroupCount[0] < std::numeric_limits<uint32_t>::max()) {
+    if (mesh_shader_props.maxTaskWorkGroupCount[0] < vvl::kU32Max) {
         draw_ptr[4] = mesh_shader_props.maxTaskWorkGroupCount[0] + 1;
         m_errorMonitor->SetDesiredError("VUID-VkDrawMeshTasksIndirectCommandEXT-TaskEXT-07322");
         m_default_queue->SubmitAndWait(m_command_buffer);
@@ -543,7 +546,7 @@ TEST_F(NegativeGpuAVIndirectBuffer, DISABLED_MeshTask) {
     }
 
     // Set y in first draw
-    if (mesh_shader_props.maxTaskWorkGroupCount[1] < std::numeric_limits<uint32_t>::max()) {
+    if (mesh_shader_props.maxTaskWorkGroupCount[1] < vvl::kU32Max) {
         draw_ptr[1] = mesh_shader_props.maxTaskWorkGroupCount[1] + 1;
         m_errorMonitor->SetDesiredError("VUID-VkDrawMeshTasksIndirectCommandEXT-TaskEXT-07323");
         m_default_queue->SubmitAndWait(m_command_buffer);
@@ -552,7 +555,7 @@ TEST_F(NegativeGpuAVIndirectBuffer, DISABLED_MeshTask) {
     }
 
     // Set z in third draw
-    if (mesh_shader_props.maxTaskWorkGroupCount[2] < std::numeric_limits<uint32_t>::max()) {
+    if (mesh_shader_props.maxTaskWorkGroupCount[2] < vvl::kU32Max) {
         draw_ptr[10] = mesh_shader_props.maxTaskWorkGroupCount[2] + 1;
         m_errorMonitor->SetDesiredError("VUID-VkDrawMeshTasksIndirectCommandEXT-TaskEXT-07324");
         m_default_queue->SubmitAndWait(m_command_buffer);
@@ -560,7 +563,7 @@ TEST_F(NegativeGpuAVIndirectBuffer, DISABLED_MeshTask) {
         draw_ptr[10] = 1;
     }
 
-    if (mesh_shader_props.maxTaskWorkGroupTotalCount < std::numeric_limits<uint32_t>::max()) {
+    if (mesh_shader_props.maxTaskWorkGroupTotalCount < vvl::kU32Max) {
         const uint32_t half_total = (mesh_shader_props.maxTaskWorkGroupTotalCount + 2) / 2;
         if (half_total < mesh_shader_props.maxTaskWorkGroupCount[0]) {
             draw_ptr[2] = 1;
@@ -842,100 +845,6 @@ TEST_F(NegativeGpuAVIndirectBuffer, DispatchWorkgroupSizeShaderObjects) {
     m_errorMonitor->VerifyFound();
 }
 
-TEST_F(NegativeGpuAVIndirectBuffer, BasicTraceRaysMultipleStages) {
-    TEST_DESCRIPTION(
-        "Setup a ray tracing pipeline (ray generation, miss and closest hit shaders) and acceleration structure, and trace one "
-        "ray");
-
-    SetTargetApiVersion(VK_API_VERSION_1_2);
-    // TODO: Refacto ray tracing extensions listing.
-    // Originally from RayTracingTest::InitFrameworkForRayTracingTest
-    AddRequiredExtensions(VK_KHR_GET_PHYSICAL_DEVICE_PROPERTIES_2_EXTENSION_NAME);
-    AddRequiredExtensions(VK_KHR_GET_MEMORY_REQUIREMENTS_2_EXTENSION_NAME);
-    AddRequiredExtensions(VK_KHR_ACCELERATION_STRUCTURE_EXTENSION_NAME);
-    AddRequiredExtensions(VK_KHR_RAY_TRACING_PIPELINE_EXTENSION_NAME);
-    AddRequiredExtensions(VK_KHR_RAY_QUERY_EXTENSION_NAME);
-    AddRequiredExtensions(VK_EXT_DESCRIPTOR_INDEXING_EXTENSION_NAME);
-    AddRequiredExtensions(VK_KHR_BUFFER_DEVICE_ADDRESS_EXTENSION_NAME);
-    AddRequiredExtensions(VK_KHR_SPIRV_1_4_EXTENSION_NAME);
-    AddRequiredExtensions(VK_KHR_DEFERRED_HOST_OPERATIONS_EXTENSION_NAME);
-    AddRequiredExtensions(VK_KHR_PIPELINE_LIBRARY_EXTENSION_NAME);
-    AddRequiredFeature(vkt::Feature::rayTracingPipeline);
-    AddRequiredFeature(vkt::Feature::accelerationStructure);
-    AddRequiredFeature(vkt::Feature::bufferDeviceAddress);
-    RETURN_IF_SKIP(InitGpuAvFramework());
-    RETURN_IF_SKIP(InitState());
-
-    vkt::rt::Pipeline pipeline(*this, m_device);
-
-    // Set shaders
-
-    const char *ray_gen = R"glsl(
-        #version 460
-        #extension GL_EXT_ray_tracing : require // Requires SPIR-V 1.5 (Vulkan 1.2)
-
-        layout(binding = 0, set = 0) uniform accelerationStructureEXT tlas;
-
-        layout(location = 0) rayPayloadEXT vec3 hit;
-
-        void main() {
-        traceRayEXT(tlas, gl_RayFlagsOpaqueEXT, 0xff, 0, 0, 0, vec3(0,0,1), 0.1, vec3(0,0,1), 1000.0, 0);
-        }
-        )glsl";
-    pipeline.SetGlslRayGenShader(ray_gen);
-
-    const char *miss = R"glsl(
-        #version 460
-        #extension GL_EXT_ray_tracing : require
-
-        layout(binding = 0, set = 0) uniform accelerationStructureEXT tlas;
-
-        layout(location = 0) rayPayloadInEXT vec3 hit;
-
-        void main() {
-        hit = vec3(0.1, 0.2, 0.3);
-        }
-        )glsl";
-    pipeline.AddGlslMissShader(miss);
-
-    const char *closest_hit = R"glsl(
-        #version 460
-        #extension GL_EXT_ray_tracing : require
-
-        layout(binding = 0, set = 0) uniform accelerationStructureEXT tlas;
-
-        layout(location = 0) rayPayloadInEXT vec3 hit;
-        hitAttributeEXT vec2 baryCoord;
-
-        void main() {
-        const vec3 barycentricCoords = vec3(1.0f - baryCoord.x - baryCoord.y, baryCoord.x, baryCoord.y);
-        hit = barycentricCoords;
-        }
-        )glsl";
-    pipeline.AddGlslClosestHitShader(closest_hit);
-
-    // Descriptor set
-    pipeline.AddBinding(VK_DESCRIPTOR_TYPE_ACCELERATION_STRUCTURE_KHR, 0);
-    pipeline.CreateDescriptorSet();
-    vkt::as::BuildGeometryInfoKHR tlas(vkt::as::blueprint::BuildOnDeviceTopLevel(*m_device, *m_default_queue, m_command_buffer));
-    pipeline.GetDescriptorSet().WriteDescriptorAccelStruct(0, 1, &tlas.GetDstAS()->handle());
-    pipeline.GetDescriptorSet().UpdateDescriptorSets();
-
-    pipeline.Build();
-
-    // Bind descriptor set, pipeline, and trace rays
-    m_command_buffer.Begin();
-    vk::CmdBindDescriptorSets(m_command_buffer, VK_PIPELINE_BIND_POINT_RAY_TRACING_KHR, pipeline.GetPipelineLayout(), 0, 1,
-                              &pipeline.GetDescriptorSet().set_, 0, nullptr);
-    vk::CmdBindPipeline(m_command_buffer, VK_PIPELINE_BIND_POINT_RAY_TRACING_KHR, pipeline);
-    vkt::rt::TraceRaysSbt trace_rays_sbt = pipeline.GetTraceRaysSbt();
-    vk::CmdTraceRaysKHR(m_command_buffer, &trace_rays_sbt.ray_gen_sbt, &trace_rays_sbt.miss_sbt, &trace_rays_sbt.hit_sbt,
-                        &trace_rays_sbt.callable_sbt, 1, 1, 1);
-    m_command_buffer.End();
-    m_default_queue->Submit(m_command_buffer);
-    m_device->Wait();
-}
-
 TEST_F(NegativeGpuAVIndirectBuffer, BufferUsageFlags2) {
     TEST_DESCRIPTION("test VkBufferUsageFlags2CreateInfo");
     AddRequiredExtensions(VK_KHR_MAINTENANCE_5_EXTENSION_NAME);
