@@ -15,7 +15,7 @@
 #include "../framework/layer_validation_tests.h"
 #include "../framework/pipeline_helper.h"
 
-class NegativeMesh : public VkLayerTest {};
+class NegativeMesh : public MeshTest {};
 
 TEST_F(NegativeMesh, BasicUsage) {
     TEST_DESCRIPTION("Test VK_EXT_mesh_shader.");
@@ -39,7 +39,7 @@ TEST_F(NegativeMesh, BasicUsage) {
     RETURN_IF_SKIP(Init());
     InitRenderTarget();
 
-    static const char vert_shader_text[] = R"glsl(
+    const char vert_shader_text[] = R"glsl(
         #version 460
         vec2 vertices[3];
         void main() {
@@ -51,7 +51,7 @@ TEST_F(NegativeMesh, BasicUsage) {
         }
     )glsl";
 
-    static const char mesh_shader_text[] = R"glsl(
+    const char mesh_shader_text[] = R"glsl(
         #version 460
         #extension GL_EXT_mesh_shader : require
         layout(local_size_x = 1, local_size_y = 1, local_size_z = 1) in;
@@ -67,7 +67,7 @@ TEST_F(NegativeMesh, BasicUsage) {
         }
     )glsl";
 
-    static const char mesh_shader_xfb_text[] = R"(
+    const char mesh_shader_xfb_text[] = R"(
                OpCapability MeshShadingEXT
                OpCapability TransformFeedback
                OpExtension "SPV_EXT_mesh_shader"
@@ -133,7 +133,7 @@ TEST_F(NegativeMesh, BasicUsage) {
             helper.shader_stages_ = {ms_xfb.GetStageCreateInfo(), fs.GetStageCreateInfo()};
         };
         CreatePipelineHelper::OneshotTest(*this, break_vp4, kErrorBit,
-                                          std::vector<std::string>({" VUID-VkGraphicsPipelineCreateInfo-None-02322"}));
+                                          std::vector<std::string>({"VUID-VkGraphicsPipelineCreateInfo-None-02322"}));
 
         // invalid dynamic state with mesh shader
         std::vector<VkDynamicState> dyn_states[] = {
@@ -176,94 +176,14 @@ TEST_F(NegativeMesh, BasicUsage) {
 
 TEST_F(NegativeMesh, ExtensionDisabled) {
     TEST_DESCRIPTION("Test VK_EXT_mesh_shader VUs with EXT_mesh_shader disabled.");
-
     SetTargetApiVersion(VK_API_VERSION_1_3);
     AddRequiredExtensions(VK_EXT_MESH_SHADER_EXTENSION_NAME);
     AddRequiredFeature(vkt::Feature::maintenance4);
-
     RETURN_IF_SKIP(Init());
     InitRenderTarget();
 
-    VkShaderObj vs(this, kVertexMinimalGlsl, VK_SHADER_STAGE_VERTEX_BIT);
-
-    // #version 450
-    // #extension GL_EXT_mesh_shader : require
-    // layout(local_size_x = 1, local_size_y = 1, local_size_z = 1) in;
-    // struct Task {
-    //     uint baseID;
-    // };
-    // taskPayloadSharedEXT Task OUT;
-    // void main() {
-    //     OUT.baseID = 1;
-    // }
-    const char *task_src = R"(
-               OpCapability MeshShadingEXT
-               OpExtension "SPV_EXT_mesh_shader"
-          %1 = OpExtInstImport "GLSL.std.450"
-               OpMemoryModel Logical GLSL450
-               OpEntryPoint TaskEXT %main "main" %OUT
-               OpExecutionModeId %main LocalSizeId %uint_1 %uint_1 %uint_1
-               OpMemberName %Task 0 "baseID"
-               OpName %OUT "OUT"
-       %void = OpTypeVoid
-          %3 = OpTypeFunction %void
-       %uint = OpTypeInt 32 0
-     %uint_1 = OpConstant %uint 1
-       %Task = OpTypeStruct %uint
-%_ptr_TaskPayloadWorkgroupEXT_Task = OpTypePointer TaskPayloadWorkgroupEXT %Task
-        %OUT = OpVariable %_ptr_TaskPayloadWorkgroupEXT_Task TaskPayloadWorkgroupEXT
-        %int = OpTypeInt 32 1
-      %int_0 = OpConstant %int 0
-%_ptr_TaskPayloadWorkgroupEXT_uint = OpTypePointer TaskPayloadWorkgroupEXT %uint
-     %v3uint = OpTypeVector %uint 3
-         %16 = OpConstantComposite %v3uint %uint_1 %uint_1 %uint_1
-       %main = OpFunction %void None %3
-          %5 = OpLabel
-         %14 = OpAccessChain %_ptr_TaskPayloadWorkgroupEXT_uint %OUT %int_0
-               OpStore %14 %uint_1
-               OpReturn
-               OpFunctionEnd
-    )";
-
-    // #version 450
-    // #extension GL_EXT_mesh_shader : enable
-    // layout (triangles) out;
-    // layout (max_vertices = 3, max_primitives = 1) out;
-    // struct Task {
-    //     uint baseID;
-    // };
-    // taskPayloadSharedEXT Task IN;
-    // void main() {}
-    const char *mesh_src = R"(
-               OpCapability MeshShadingEXT
-               OpExtension "SPV_EXT_mesh_shader"
-          %1 = OpExtInstImport "GLSL.std.450"
-               OpMemoryModel Logical GLSL450
-               OpEntryPoint MeshEXT %main "main" %IN
-               OpExecutionMode %main LocalSize 1 1 1
-               OpExecutionMode %main OutputVertices 3
-               OpExecutionMode %main OutputPrimitivesEXT 1
-               OpExecutionMode %main OutputTrianglesEXT
-               OpSource GLSL 450
-               OpSourceExtension "GL_EXT_mesh_shader"
-               OpName %main "main"
-               OpName %Task "Task"
-               OpMemberName %Task 0 "baseID"
-               OpName %IN "IN"
-       %void = OpTypeVoid
-          %3 = OpTypeFunction %void
-       %uint = OpTypeInt 32 0
-       %Task = OpTypeStruct %uint
-%_ptr_TaskPayloadWorkgroupEXT_Task = OpTypePointer TaskPayloadWorkgroupEXT %Task
-         %IN = OpVariable %_ptr_TaskPayloadWorkgroupEXT_Task TaskPayloadWorkgroupEXT
-       %main = OpFunction %void None %3
-          %5 = OpLabel
-               OpReturn
-               OpFunctionEnd
-    )";
-
-    VkShaderObj task_shader(this, task_src, VK_SHADER_STAGE_TASK_BIT_EXT, SPV_ENV_VULKAN_1_3, SPV_SOURCE_ASM);
-    VkShaderObj mesh_shader(this, mesh_src, VK_SHADER_STAGE_MESH_BIT_EXT, SPV_ENV_VULKAN_1_3, SPV_SOURCE_ASM);
+    VkShaderObj task_shader(this, kTaskMinimalGlsl, VK_SHADER_STAGE_TASK_BIT_EXT, SPV_ENV_VULKAN_1_3);
+    VkShaderObj mesh_shader(this, kMeshMinimalGlsl, VK_SHADER_STAGE_MESH_BIT_EXT, SPV_ENV_VULKAN_1_3);
     VkShaderObj fs(this, kFragmentMinimalGlsl, VK_SHADER_STAGE_FRAGMENT_BIT);
 
     // mesh and task shaders not supported
@@ -277,14 +197,8 @@ TEST_F(NegativeMesh, ExtensionDisabled) {
 
 TEST_F(NegativeMesh, RuntimeSpirv) {
     TEST_DESCRIPTION("Test VK_EXT_mesh_shader spirv related VUIDs.");
-
-    SetTargetApiVersion(VK_API_VERSION_1_3);
-    AddRequiredExtensions(VK_EXT_MESH_SHADER_EXTENSION_NAME);
     AddRequiredFeature(vkt::Feature::maintenance4);
-    AddRequiredFeature(vkt::Feature::meshShader);
-    AddRequiredFeature(vkt::Feature::taskShader);
-
-    RETURN_IF_SKIP(Init());
+    RETURN_IF_SKIP(InitBasicMeshAndTask());
     InitRenderTarget();
 
     VkPhysicalDeviceMeshShaderPropertiesEXT mesh_shader_properties = vku::InitStructHelper();
@@ -299,33 +213,33 @@ TEST_F(NegativeMesh, RuntimeSpirv) {
     uint32_t max_mesh_workgroup_size_y = mesh_shader_properties.maxMeshWorkGroupSize[1];
     uint32_t max_mesh_workgroup_size_z = mesh_shader_properties.maxMeshWorkGroupSize[2];
 
-    if (max_task_workgroup_size_x < vvl::MaxTypeValue(max_task_workgroup_size_x)) {
+    if (max_task_workgroup_size_x < vvl::kU32Max) {
         error_vuids.emplace_back("VUID-RuntimeSpirv-TaskEXT-07291");
         max_task_workgroup_size_x += 1;
     }
 
-    if (max_task_workgroup_size_y < vvl::MaxTypeValue(max_task_workgroup_size_y)) {
+    if (max_task_workgroup_size_y < vvl::kU32Max) {
         error_vuids.emplace_back("VUID-RuntimeSpirv-TaskEXT-07292");
         max_task_workgroup_size_y += 1;
     }
 
-    if (max_task_workgroup_size_z < vvl::MaxTypeValue(max_task_workgroup_size_z)) {
+    if (max_task_workgroup_size_z < vvl::kU32Max) {
         error_vuids.emplace_back("VUID-RuntimeSpirv-TaskEXT-07293");
         max_task_workgroup_size_z += 1;
     }
     error_vuids.emplace_back("VUID-RuntimeSpirv-TaskEXT-07294");
 
-    if (max_mesh_workgroup_size_x < vvl::MaxTypeValue(max_mesh_workgroup_size_x)) {
+    if (max_mesh_workgroup_size_x < vvl::kU32Max) {
         error_vuids.emplace_back("VUID-RuntimeSpirv-MeshEXT-07295");
         max_mesh_workgroup_size_x += 1;
     }
 
-    if (max_mesh_workgroup_size_y < vvl::MaxTypeValue(max_mesh_workgroup_size_y)) {
+    if (max_mesh_workgroup_size_y < vvl::kU32Max) {
         error_vuids.emplace_back("VUID-RuntimeSpirv-MeshEXT-07296");
         max_mesh_workgroup_size_y += 1;
     }
 
-    if (max_mesh_workgroup_size_z < vvl::MaxTypeValue(max_mesh_workgroup_size_z)) {
+    if (max_mesh_workgroup_size_z < vvl::kU32Max) {
         error_vuids.emplace_back("VUID-RuntimeSpirv-MeshEXT-07297");
         max_mesh_workgroup_size_z += 1;
     }
@@ -387,6 +301,7 @@ TEST_F(NegativeMesh, RuntimeSpirv) {
                OpFunctionEnd
     )";
 
+    // lazy way to not check limits
     m_errorMonitor->SetAllowedFailureMsg("VUID-RuntimeSpirv-MeshEXT-07115");
     m_errorMonitor->SetAllowedFailureMsg("VUID-RuntimeSpirv-MeshEXT-07116");
     VkShaderObj task_shader(this, task_src.c_str(), VK_SHADER_STAGE_TASK_BIT_EXT, SPV_ENV_VULKAN_1_3, SPV_SOURCE_ASM);
@@ -402,13 +317,8 @@ TEST_F(NegativeMesh, RuntimeSpirv) {
 
 TEST_F(NegativeMesh, RuntimeSpirv2) {
     TEST_DESCRIPTION("Test VK_EXT_mesh_shader spirv related VUIDs.");
-
-    SetTargetApiVersion(VK_API_VERSION_1_3);
-    AddRequiredExtensions(VK_EXT_MESH_SHADER_EXTENSION_NAME);
     AddRequiredFeature(vkt::Feature::maintenance4);
-    AddRequiredFeature(vkt::Feature::meshShader);
-
-    RETURN_IF_SKIP(Init());
+    RETURN_IF_SKIP(InitBasicMeshAndTask());
     InitRenderTarget();
 
     VkPhysicalDeviceMeshShaderPropertiesEXT mesh_shader_properties = vku::InitStructHelper();
@@ -418,13 +328,13 @@ TEST_F(NegativeMesh, RuntimeSpirv2) {
     uint32_t max_mesh_output_primitives = mesh_shader_properties.maxMeshOutputPrimitives;
 
     bool skip = true;
-    if (max_mesh_output_vertices < vvl::MaxTypeValue(max_mesh_output_vertices)) {
+    if (max_mesh_output_vertices < vvl::kU32Max) {
         m_errorMonitor->SetDesiredError("VUID-RuntimeSpirv-MeshEXT-07115");
         skip = false;
         max_mesh_output_vertices += 1;
     }
 
-    if (max_mesh_output_primitives < vvl::MaxTypeValue(max_mesh_output_primitives)) {
+    if (max_mesh_output_primitives < vvl::kU32Max) {
         m_errorMonitor->SetDesiredError("VUID-RuntimeSpirv-MeshEXT-07116");
         skip = false;
         max_mesh_output_primitives += 1;
@@ -536,7 +446,7 @@ TEST_F(NegativeMesh, BasicUsageNV) {
     RETURN_IF_SKIP(InitState(nullptr, &features2));
     InitRenderTarget();
 
-    static const char vertShaderText[] = R"glsl(
+    const char vertShaderText[] = R"glsl(
         #version 450
         vec2 vertices[3];
         void main() {
@@ -548,7 +458,7 @@ TEST_F(NegativeMesh, BasicUsageNV) {
         }
     )glsl";
 
-    static const char meshShaderText[] = R"glsl(
+    const char meshShaderText[] = R"glsl(
         #version 450
         #extension GL_NV_mesh_shader : require
         layout(local_size_x = 1) in;
@@ -743,7 +653,7 @@ TEST_F(NegativeMesh, ExtensionDisabledNV) {
                OpFunctionEnd
     )";
 
-    static const char mesh_src[] = R"glsl(
+    const char mesh_src[] = R"glsl(
         #version 450
 
         #extension GL_NV_mesh_shader : require
@@ -776,13 +686,8 @@ TEST_F(NegativeMesh, ExtensionDisabledNV) {
 
 TEST_F(NegativeMesh, DrawCmds) {
     TEST_DESCRIPTION("Test VK_EXT_mesh_shader draw commands.");
-
-    SetTargetApiVersion(VK_API_VERSION_1_3);
-    AddRequiredExtensions(VK_EXT_MESH_SHADER_EXTENSION_NAME);
     AddRequiredFeature(vkt::Feature::maintenance4);
-    AddRequiredFeature(vkt::Feature::meshShader);
-
-    RETURN_IF_SKIP(Init());
+    RETURN_IF_SKIP(InitBasicMeshAndTask());
     InitRenderTarget();
 
     VkPhysicalDeviceMeshShaderPropertiesEXT mesh_shader_properties = vku::InitStructHelper();
@@ -845,19 +750,19 @@ TEST_F(NegativeMesh, DrawCmds) {
     uint32_t max_group_count_Y = mesh_shader_properties.maxTaskWorkGroupCount[1];
     uint32_t max_group_count_Z = mesh_shader_properties.maxTaskWorkGroupCount[2];
 
-    if (max_group_count_X < vvl::MaxTypeValue(max_group_count_X)) {
+    if (max_group_count_X < vvl::kU32Max) {
         m_errorMonitor->SetDesiredError("VUID-vkCmdDrawMeshTasksEXT-TaskEXT-07322");
-        max_group_count_X = vvl::MaxTypeValue(max_group_count_X);
+        max_group_count_X = vvl::kU32Max;
     }
 
-    if (max_group_count_Y < vvl::MaxTypeValue(max_group_count_Y)) {
+    if (max_group_count_Y < vvl::kU32Max) {
         m_errorMonitor->SetDesiredError("VUID-vkCmdDrawMeshTasksEXT-TaskEXT-07323");
-        max_group_count_Y = vvl::MaxTypeValue(max_group_count_Y);
+        max_group_count_Y = vvl::kU32Max;
     }
 
-    if (max_group_count_Z < vvl::MaxTypeValue(max_group_count_Z)) {
+    if (max_group_count_Z < vvl::kU32Max) {
         m_errorMonitor->SetDesiredError("VUID-vkCmdDrawMeshTasksEXT-TaskEXT-07324");
-        max_group_count_Z = vvl::MaxTypeValue(max_group_count_Z);
+        max_group_count_Z = vvl::kU32Max;
     }
 
     m_errorMonitor->SetDesiredError("VUID-vkCmdDrawMeshTasksEXT-TaskEXT-07325");
@@ -868,7 +773,7 @@ TEST_F(NegativeMesh, DrawCmds) {
     vk::CmdDrawMeshTasksIndirectEXT(m_command_buffer, buffer, 0, 2, sizeof(VkDrawMeshTasksIndirectCommandEXT));
     m_errorMonitor->VerifyFound();
 
-    if (m_device->Physical().limits_.maxDrawIndirectCount < vvl::MaxTypeValue(m_device->Physical().limits_.maxDrawIndirectCount)) {
+    if (m_device->Physical().limits_.maxDrawIndirectCount < vvl::kU32Max) {
         m_errorMonitor->SetUnexpectedError("VUID-vkCmdDrawMeshTasksIndirectEXT-drawCount-02718");
         m_errorMonitor->SetDesiredError("VUID-vkCmdDrawMeshTasksIndirectEXT-drawCount-07090");
         m_errorMonitor->SetDesiredError("VUID-vkCmdDrawMeshTasksIndirectEXT-drawCount-02719");
@@ -909,14 +814,9 @@ TEST_F(NegativeMesh, DrawCmds) {
 
 TEST_F(NegativeMesh, MultiDrawIndirect) {
     TEST_DESCRIPTION("Test VK_EXT_mesh_shader indirect draw command.");
-
-    SetTargetApiVersion(VK_API_VERSION_1_3);
-    AddRequiredExtensions(VK_EXT_MESH_SHADER_EXTENSION_NAME);
     AddRequiredFeature(vkt::Feature::maintenance4);
-    AddRequiredFeature(vkt::Feature::meshShader);
     AddRequiredFeature(vkt::Feature::multiDrawIndirect);
-
-    RETURN_IF_SKIP(Init());
+    RETURN_IF_SKIP(InitBasicMeshAndTask());
     InitRenderTarget();
 
     VkPhysicalDeviceMeshShaderPropertiesEXT mesh_shader_properties = vku::InitStructHelper();
@@ -932,7 +832,7 @@ TEST_F(NegativeMesh, MultiDrawIndirect) {
     // };
     // taskPayloadSharedEXT Task IN;
     // void main() {}
-    static const char mesh_src[] = R"(
+    const char mesh_src[] = R"(
                OpCapability MeshShadingEXT
                OpExtension "SPV_EXT_mesh_shader"
           %1 = OpExtInstImport "GLSL.std.450"
@@ -1040,7 +940,7 @@ TEST_F(NegativeMesh, DrawCmdsNV) {
 
     InitRenderTarget();
 
-    static const char mesh_src[] = R"glsl(
+    const char mesh_src[] = R"glsl(
         #version 450
         #extension GL_NV_mesh_shader : require
         layout(local_size_x = 1) in;
@@ -1101,16 +1001,10 @@ TEST_F(NegativeMesh, DrawCmdsNV) {
 
 TEST_F(NegativeMesh, MeshTasksWorkgroupCount) {
     TEST_DESCRIPTION("Test mesh tasks emitted from task shader.");
-
-    SetTargetApiVersion(VK_API_VERSION_1_3);
     AddRequiredExtensions(VK_KHR_SPIRV_1_4_EXTENSION_NAME);
-    AddRequiredExtensions(VK_EXT_MESH_SHADER_EXTENSION_NAME);
     AddRequiredExtensions(VK_KHR_MAINTENANCE_4_EXTENSION_NAME);
     AddRequiredFeature(vkt::Feature::maintenance4);
-    AddRequiredFeature(vkt::Feature::meshShader);
-    AddRequiredFeature(vkt::Feature::taskShader);
-
-    RETURN_IF_SKIP(Init());
+    RETURN_IF_SKIP(InitBasicMeshAndTask());
     InitRenderTarget();
 
     VkPhysicalDeviceMeshShaderPropertiesEXT mesh_shader_properties = vku::InitStructHelper();
@@ -1149,7 +1043,7 @@ TEST_F(NegativeMesh, MeshTasksWorkgroupCount) {
         }
     )glsl";
 
-    static const char mesh_src[] = R"glsl(
+    const char mesh_src[] = R"glsl(
         #version 460
         #extension GL_EXT_mesh_shader : require
         layout(local_size_x = 1, local_size_y = 1, local_size_z = 1) in;
@@ -1165,7 +1059,7 @@ TEST_F(NegativeMesh, MeshTasksWorkgroupCount) {
         }
     )glsl";
 
-    static const char frag_src[] = R"glsl(
+    const char frag_src[] = R"glsl(
         #version 460
         layout(location = 0) out vec4 uFragColor;
         void main(){
@@ -1196,32 +1090,27 @@ TEST_F(NegativeMesh, MeshTasksWorkgroupCount) {
     if (mesh_shader_properties.maxMeshWorkGroupCount[0] == mesh_shader_properties.maxMeshWorkGroupTotalCount) {
         vuids.emplace_back("VUID-RuntimeSpirv-TaskEXT-07302");
     }
-    if (mesh_shader_properties.maxMeshWorkGroupCount[0] != std::numeric_limits<uint32_t>::max()) {
+    if (mesh_shader_properties.maxMeshWorkGroupCount[0] != vvl::kU32Max) {
         CreatePipelineHelper::OneshotTest(*this, mesh_tasks_x, kErrorBit, vuids);
     }
-    if (mesh_shader_properties.maxMeshWorkGroupCount[1] != std::numeric_limits<uint32_t>::max()) {
+    if (mesh_shader_properties.maxMeshWorkGroupCount[1] != vvl::kU32Max) {
         CreatePipelineHelper::OneshotTest(*this, mesh_tasks_y, kErrorBit, "VUID-RuntimeSpirv-TaskEXT-07300");
     }
-    if (mesh_shader_properties.maxMeshWorkGroupCount[2] != std::numeric_limits<uint32_t>::max()) {
+    if (mesh_shader_properties.maxMeshWorkGroupCount[2] != vvl::kU32Max) {
         CreatePipelineHelper::OneshotTest(*this, mesh_tasks_z, kErrorBit, "VUID-RuntimeSpirv-TaskEXT-07301");
     }
 }
 
 TEST_F(NegativeMesh, MeshShaderConservativeRasterization) {
     TEST_DESCRIPTION("Use mesh shader with invalid conservative rasterization mode");
-
-    SetTargetApiVersion(VK_API_VERSION_1_3);
     AddRequiredExtensions(VK_KHR_SPIRV_1_4_EXTENSION_NAME);
-    AddRequiredExtensions(VK_EXT_MESH_SHADER_EXTENSION_NAME);
     AddRequiredExtensions(VK_KHR_MAINTENANCE_4_EXTENSION_NAME);
     AddRequiredExtensions(VK_EXT_CONSERVATIVE_RASTERIZATION_EXTENSION_NAME);
     AddRequiredFeature(vkt::Feature::maintenance4);
-    AddRequiredFeature(vkt::Feature::meshShader);
-
-    RETURN_IF_SKIP(Init());
+    RETURN_IF_SKIP(InitBasicMeshAndTask());
     InitRenderTarget();
 
-    static const char meshShaderText[] = R"glsl(
+    const char meshShaderText[] = R"glsl(
         #version 460
         #extension GL_EXT_mesh_shader : require
         layout(local_size_x = 1, local_size_y = 1, local_size_z = 1) in;
@@ -1246,26 +1135,21 @@ TEST_F(NegativeMesh, MeshShaderConservativeRasterization) {
     CreatePipelineHelper pipe(*this);
     pipe.rs_state_ci_.pNext = &conservative_state;
     pipe.shader_stages_ = {ms.GetStageCreateInfo(), pipe.fs_->GetStageCreateInfo()};
-    m_errorMonitor->SetDesiredError("VUID-VkGraphicsPipelineCreateInfo-conservativePointAndLineRasterization-06761");
+    m_errorMonitor->SetDesiredError("VUID-VkGraphicsPipelineCreateInfo-conservativePointAndLineRasterization-08892");
     pipe.CreateGraphicsPipeline();
     m_errorMonitor->VerifyFound();
 }
 
 TEST_F(NegativeMesh, MeshIncompatibleActiveQueries) {
     TEST_DESCRIPTION("Draw with mesh shaders when xfb or primitives generated queries are enabled");
-
-    SetTargetApiVersion(VK_API_VERSION_1_3);
     AddRequiredExtensions(VK_KHR_SPIRV_1_4_EXTENSION_NAME);
-    AddRequiredExtensions(VK_EXT_MESH_SHADER_EXTENSION_NAME);
     AddRequiredExtensions(VK_KHR_MAINTENANCE_4_EXTENSION_NAME);
     AddRequiredExtensions(VK_EXT_TRANSFORM_FEEDBACK_EXTENSION_NAME);
     AddRequiredExtensions(VK_EXT_PRIMITIVES_GENERATED_QUERY_EXTENSION_NAME);
     AddRequiredFeature(vkt::Feature::maintenance4);
-    AddRequiredFeature(vkt::Feature::meshShader);
     AddRequiredFeature(vkt::Feature::primitivesGeneratedQuery);
     AddRequiredFeature(vkt::Feature::transformFeedback);
-
-    RETURN_IF_SKIP(Init());
+    RETURN_IF_SKIP(InitBasicMeshAndTask());
     InitRenderTarget();
 
     VkShaderObj ms(this, kMeshMinimalGlsl, VK_SHADER_STAGE_MESH_BIT_EXT, SPV_ENV_VULKAN_1_3);
@@ -1299,13 +1183,8 @@ TEST_F(NegativeMesh, MeshIncompatibleActiveQueries) {
 
 TEST_F(NegativeMesh, DrawIndexMesh) {
     TEST_DESCRIPTION("use DrawIndex in Mesh shader but there is a Task Shader.");
-    SetTargetApiVersion(VK_API_VERSION_1_2);
-    AddRequiredExtensions(VK_EXT_MESH_SHADER_EXTENSION_NAME);
-    AddRequiredFeature(vkt::Feature::meshShader);
-    AddRequiredFeature(vkt::Feature::taskShader);
     AddRequiredFeature(vkt::Feature::shaderDrawParameters);
-
-    RETURN_IF_SKIP(Init());
+    RETURN_IF_SKIP(InitBasicMeshAndTask());
     InitRenderTarget();
 
     const char *task_source = R"glsl(
@@ -1342,15 +1221,10 @@ TEST_F(NegativeMesh, DrawIndexMesh) {
 
 TEST_F(NegativeMesh, DrawIndexMeshShaderObject) {
     TEST_DESCRIPTION("use DrawIndex in Mesh shader but there is a Task Shader.");
-    SetTargetApiVersion(VK_API_VERSION_1_2);
-    AddRequiredExtensions(VK_EXT_MESH_SHADER_EXTENSION_NAME);
     AddRequiredExtensions(VK_EXT_SHADER_OBJECT_EXTENSION_NAME);
     AddRequiredFeature(vkt::Feature::shaderObject);
     AddRequiredFeature(vkt::Feature::shaderDrawParameters);
-    AddRequiredFeature(vkt::Feature::meshShader);
-    AddRequiredFeature(vkt::Feature::taskShader);
-
-    RETURN_IF_SKIP(Init());
+    RETURN_IF_SKIP(InitBasicMeshAndTask());
     InitRenderTarget();
 
     const char *mesh_src = R"glsl(
@@ -1369,3 +1243,120 @@ TEST_F(NegativeMesh, DrawIndexMeshShaderObject) {
                                  GLSLToSPV(VK_SHADER_STAGE_MESH_BIT_EXT, mesh_src, SPV_ENV_VULKAN_1_2));
     m_errorMonitor->VerifyFound();
 }
+
+TEST_F(NegativeMesh, TaskPayloadSharedMissing) {
+    RETURN_IF_SKIP(InitBasicMeshAndTask());
+    InitRenderTarget();
+
+    const char *task_source = R"glsl(
+        #version 460
+        #extension GL_EXT_mesh_shader : enable
+        void main() {
+            EmitMeshTasksEXT(1u, 1u, 1u);
+        }
+    )glsl";
+
+    const char *mesh_source = R"glsl(
+        #version 460
+        #extension GL_EXT_mesh_shader : enable
+        layout(max_vertices = 32, max_primitives = 32, triangles) out;
+        taskPayloadSharedEXT uint payload;
+        void main() {
+            uint x = payload;
+            SetMeshOutputsEXT(3,1);
+        }
+    )glsl";
+
+    VkShaderObj ts(this, task_source, VK_SHADER_STAGE_TASK_BIT_EXT, SPV_ENV_VULKAN_1_2);
+    VkShaderObj ms(this, mesh_source, VK_SHADER_STAGE_MESH_BIT_EXT, SPV_ENV_VULKAN_1_2);
+    VkShaderObj fs(this, kFragmentMinimalGlsl, VK_SHADER_STAGE_FRAGMENT_BIT, SPV_ENV_VULKAN_1_2);
+
+    CreatePipelineHelper pipe(*this);
+    pipe.shader_stages_ = {ts.GetStageCreateInfo(), ms.GetStageCreateInfo(), fs.GetStageCreateInfo()};
+    m_errorMonitor->SetDesiredError("VUID-RuntimeSpirv-MeshEXT-10883");
+    pipe.CreateGraphicsPipeline();
+    m_errorMonitor->VerifyFound();
+}
+
+TEST_F(NegativeMesh, TaskPayloadSharedDifferent) {
+    RETURN_IF_SKIP(InitBasicMeshAndTask());
+    InitRenderTarget();
+
+    const char *task_source = R"glsl(
+        #version 460
+        #extension GL_EXT_mesh_shader : enable
+        struct Foo {
+            uint a[3];
+            uint b;
+        };
+        taskPayloadSharedEXT Foo payload;
+        void main() {
+            payload.b = 4;
+            EmitMeshTasksEXT(1u, 1u, 1u);
+        }
+    )glsl";
+
+    const char *mesh_source = R"glsl(
+        #version 460
+        #extension GL_EXT_mesh_shader : enable
+        layout(max_vertices = 32, max_primitives = 32, triangles) out;
+        taskPayloadSharedEXT uint payload;
+        void main() {
+            uint x = payload;
+            SetMeshOutputsEXT(3,1);
+        }
+    )glsl";
+
+    VkShaderObj ts(this, task_source, VK_SHADER_STAGE_TASK_BIT_EXT, SPV_ENV_VULKAN_1_2);
+    VkShaderObj ms(this, mesh_source, VK_SHADER_STAGE_MESH_BIT_EXT, SPV_ENV_VULKAN_1_2);
+    VkShaderObj fs(this, kFragmentMinimalGlsl, VK_SHADER_STAGE_FRAGMENT_BIT, SPV_ENV_VULKAN_1_2);
+
+    CreatePipelineHelper pipe(*this);
+    pipe.shader_stages_ = {ts.GetStageCreateInfo(), ms.GetStageCreateInfo(), fs.GetStageCreateInfo()};
+    m_errorMonitor->SetDesiredError("VUID-RuntimeSpirv-MeshEXT-10883");
+    pipe.CreateGraphicsPipeline();
+    m_errorMonitor->VerifyFound();
+}
+
+TEST_F(NegativeMesh, TaskPayloadSharedMissingShaderObject) {
+    AddRequiredExtensions(VK_EXT_SHADER_OBJECT_EXTENSION_NAME);
+    AddRequiredExtensions(VK_KHR_DYNAMIC_RENDERING_EXTENSION_NAME);
+    AddRequiredFeature(vkt::Feature::dynamicRendering);
+    AddRequiredFeature(vkt::Feature::maintenance4);
+    AddRequiredFeature(vkt::Feature::shaderObject);
+    RETURN_IF_SKIP(InitBasicMeshAndTask());
+    InitDynamicRenderTarget();
+
+    const char *task_source = R"glsl(
+        #version 460
+        #extension GL_EXT_mesh_shader : enable
+        void main() {
+            EmitMeshTasksEXT(1u, 1u, 1u);
+        }
+    )glsl";
+
+    const char *mesh_source = R"glsl(
+        #version 460
+        #extension GL_EXT_mesh_shader : enable
+        layout(max_vertices = 32, max_primitives = 32, triangles) out;
+        taskPayloadSharedEXT uint mesh_payload;
+        void main() {
+            uint x = mesh_payload;
+            SetMeshOutputsEXT(3,1);
+        }
+    )glsl";
+
+    const vkt::Shader task_shader(*m_device, VK_SHADER_STAGE_TASK_BIT_EXT, task_source);
+    const vkt::Shader mesh_shader(*m_device, VK_SHADER_STAGE_MESH_BIT_EXT, mesh_source);
+    const vkt::Shader frag_shader(*m_device, VK_SHADER_STAGE_FRAGMENT_BIT, kFragmentMinimalGlsl);
+
+    m_command_buffer.Begin();
+    m_command_buffer.BeginRenderingColor(GetDynamicRenderTarget(), GetRenderTargetArea());
+    m_command_buffer.BindMeshShaders(task_shader, mesh_shader, frag_shader);
+    SetDefaultDynamicStatesExclude();
+    m_errorMonitor->SetDesiredError("VUID-RuntimeSpirv-MeshEXT-10883");
+    vk::CmdDrawMeshTasksEXT(m_command_buffer, 1, 1, 1);
+    m_errorMonitor->VerifyFound();
+    m_command_buffer.EndRendering();
+    m_command_buffer.End();
+}
