@@ -12,6 +12,7 @@
  *     http://www.apache.org/licenses/LICENSE-2.0
  */
 
+#include <vulkan/vulkan_core.h>
 #include "utils/cast_utils.h"
 #include "../framework/layer_validation_tests.h"
 #include "../framework/pipeline_helper.h"
@@ -391,10 +392,7 @@ TEST_F(NegativeObjectLifetime, DISABLED_DescriptorSetMutableBufferDestroyed) {
         VK_DESCRIPTOR_TYPE_STORAGE_BUFFER,
     };
 
-    VkMutableDescriptorTypeListEXT type_list = {};
-    type_list.descriptorTypeCount = 2;
-    type_list.pDescriptorTypes = desc_types;
-
+    VkMutableDescriptorTypeListEXT type_list = {2, desc_types};
     VkMutableDescriptorTypeCreateInfoEXT mdtci = vku::InitStructHelper();
     mdtci.mutableDescriptorTypeListCount = 1;
     mdtci.pMutableDescriptorTypeLists = &type_list;
@@ -443,10 +441,7 @@ TEST_F(NegativeObjectLifetime, DISABLED_DescriptorSetMutableBufferArrayDestroyed
         VK_DESCRIPTOR_TYPE_STORAGE_BUFFER,
     };
 
-    VkMutableDescriptorTypeListEXT type_list = {};
-    type_list.descriptorTypeCount = 2;
-    type_list.pDescriptorTypes = desc_types;
-
+    VkMutableDescriptorTypeListEXT type_list = {2, desc_types};
     VkMutableDescriptorTypeCreateInfoEXT mdtci = vku::InitStructHelper();
     mdtci.mutableDescriptorTypeListCount = 1;
     mdtci.pMutableDescriptorTypeLists = &type_list;
@@ -570,8 +565,7 @@ TEST_F(NegativeObjectLifetime, CmdBufferFramebufferImageDestroyed) {
 
         // Just use default renderpass with our framebuffer
         m_renderPassBeginInfo.framebuffer = fb;
-        m_renderPassBeginInfo.renderArea.extent.width = 32;
-        m_renderPassBeginInfo.renderArea.extent.height = 32;
+        m_renderPassBeginInfo.renderArea.extent = {32, 32};
         // Create Null cmd buffer for submit
         m_command_buffer.Begin();
         m_command_buffer.BeginRenderPass(m_renderPassBeginInfo);
@@ -638,7 +632,7 @@ TEST_F(NegativeObjectLifetime, FramebufferAttachmentMemoryFreed) {
     m_errorMonitor->VerifyFound();
 }
 
-TEST_F(NegativeObjectLifetime, DescriptorPoolInUseDestroyedSignaled) {
+TEST_F(NegativeObjectLifetime, DescriptorPoolInUseDestroyed) {
     TEST_DESCRIPTION("Delete a DescriptorPool with a DescriptorSet that is in use.");
     RETURN_IF_SKIP(Init());
     InitRenderTarget();
@@ -686,7 +680,7 @@ TEST_F(NegativeObjectLifetime, DescriptorPoolInUseDestroyedSignaled) {
     // TODO : It seems Validation layers think ds_pool was already destroyed, even though it wasn't?
 }
 
-TEST_F(NegativeObjectLifetime, FramebufferInUseDestroyedSignaled) {
+TEST_F(NegativeObjectLifetime, FramebufferInUseDestroyed) {
     TEST_DESCRIPTION("Delete in-use framebuffer.");
     RETURN_IF_SKIP(Init());
     VkFormatProperties format_properties;
@@ -725,18 +719,11 @@ TEST_F(NegativeObjectLifetime, FramebufferInUseDestroyedSignaled) {
 
 TEST_F(NegativeObjectLifetime, PushDescriptorUniformDestroySignaled) {
     TEST_DESCRIPTION("Destroy a uniform buffer in use by a push descriptor set");
-
     AddRequiredExtensions(VK_KHR_PUSH_DESCRIPTOR_EXTENSION_NAME);
     RETURN_IF_SKIP(Init());
     InitRenderTarget();
 
-    VkDescriptorSetLayoutBinding dsl_binding = {};
-    dsl_binding.binding = 2;
-    dsl_binding.descriptorType = VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER;
-    dsl_binding.descriptorCount = 1;
-    dsl_binding.stageFlags = VK_SHADER_STAGE_FRAGMENT_BIT;
-    dsl_binding.pImmutableSamplers = NULL;
-
+    VkDescriptorSetLayoutBinding dsl_binding = {2, VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER, 1, VK_SHADER_STAGE_FRAGMENT_BIT, nullptr};
     const vkt::DescriptorSetLayout ds_layout(*m_device, {dsl_binding});
     // Create push descriptor set layout
     const vkt::DescriptorSetLayout push_ds_layout(*m_device, {dsl_binding}, VK_DESCRIPTOR_SET_LAYOUT_CREATE_PUSH_DESCRIPTOR_BIT);
@@ -749,10 +736,7 @@ TEST_F(NegativeObjectLifetime, PushDescriptorUniformDestroySignaled) {
     const uint32_t data_size = sizeof(float) * 3;
     vkt::Buffer vbo(*m_device, data_size, VK_BUFFER_USAGE_UNIFORM_BUFFER_BIT);
 
-    VkDescriptorBufferInfo buff_info;
-    buff_info.buffer = vbo;
-    buff_info.offset = 0;
-    buff_info.range = data_size;
+    VkDescriptorBufferInfo buff_info{vbo, 0, data_size};
     VkWriteDescriptorSet descriptor_write = vku::InitStructHelper();
     descriptor_write.dstBinding = 2;
     descriptor_write.descriptorCount = 1;
@@ -779,7 +763,7 @@ TEST_F(NegativeObjectLifetime, PushDescriptorUniformDestroySignaled) {
     m_default_queue->Wait();
 }
 
-TEST_F(NegativeObjectLifetime, FramebufferImageInUseDestroyedSignaled) {
+TEST_F(NegativeObjectLifetime, FramebufferImageInUseDestroyed) {
     TEST_DESCRIPTION("Delete in-use image that's child of framebuffer.");
     RETURN_IF_SKIP(Init());
 
@@ -813,7 +797,7 @@ TEST_F(NegativeObjectLifetime, FramebufferImageInUseDestroyedSignaled) {
     fb.Destroy();
 }
 
-TEST_F(NegativeObjectLifetime, EventInUseDestroyedSignaled) {
+TEST_F(NegativeObjectLifetime, EventInUseDestroyed) {
     RETURN_IF_SKIP(Init());
     InitRenderTarget();
 
@@ -832,11 +816,7 @@ TEST_F(NegativeObjectLifetime, EventInUseDestroyedSignaled) {
     m_errorMonitor->VerifyFound();
 }
 
-TEST_F(NegativeObjectLifetime, InUseDestroyedSignaled) {
-    TEST_DESCRIPTION(
-        "Use vkCmdExecuteCommands with invalid state in primary and secondary command buffers. Delete objects that are in use. "
-        "Call VkQueueSubmit with an event that has been deleted.");
-
+TEST_F(NegativeObjectLifetime, MiscInUseDestroyed) {
     RETURN_IF_SKIP(Init());
     InitRenderTarget();
 
@@ -900,7 +880,7 @@ TEST_F(NegativeObjectLifetime, InUseDestroyedSignaled) {
     vk::DestroyEvent(device(), event, nullptr);
 }
 
-TEST_F(NegativeObjectLifetime, PipelineInUseDestroyedSignaled) {
+TEST_F(NegativeObjectLifetime, PipelineInUseDestroyed) {
     TEST_DESCRIPTION("Delete in-use pipeline.");
 
     RETURN_IF_SKIP(Init());
@@ -936,7 +916,7 @@ TEST_F(NegativeObjectLifetime, PipelineInUseDestroyedSignaled) {
     vk::DestroyPipeline(*m_device, delete_this_pipeline, nullptr);
 }
 
-TEST_F(NegativeObjectLifetime, ImageViewInUseDestroyedSignaled) {
+TEST_F(NegativeObjectLifetime, ImageViewInUseDestroyed) {
     TEST_DESCRIPTION("Delete in-use imageView.");
 
     RETURN_IF_SKIP(Init());
@@ -992,7 +972,7 @@ TEST_F(NegativeObjectLifetime, ImageViewInUseDestroyedSignaled) {
     vk::DestroySampler(device(), sampler, nullptr);
 }
 
-TEST_F(NegativeObjectLifetime, BufferViewInUseDestroyedSignaled) {
+TEST_F(NegativeObjectLifetime, BufferViewInUseDestroyed) {
     TEST_DESCRIPTION("Delete in-use bufferView.");
 
     RETURN_IF_SKIP(Init());
@@ -1055,7 +1035,7 @@ TEST_F(NegativeObjectLifetime, BufferViewInUseDestroyedSignaled) {
     vk::DestroyBufferView(device(), view, NULL);
 }
 
-TEST_F(NegativeObjectLifetime, SamplerInUseDestroyedSignaled) {
+TEST_F(NegativeObjectLifetime, SamplerInUseDestroyed) {
     TEST_DESCRIPTION("Delete in-use sampler.");
 
     RETURN_IF_SKIP(Init());
@@ -1109,6 +1089,50 @@ TEST_F(NegativeObjectLifetime, SamplerInUseDestroyedSignaled) {
     vk::DestroySampler(device(), sampler, NULL);  // Destroyed for real
 }
 
+TEST_F(NegativeObjectLifetime, ImmutableSamplerInUseDestroyed) {
+    TEST_DESCRIPTION("Delete in-use immutable sampler.");
+    RETURN_IF_SKIP(Init());
+
+    vkt::Sampler sampler(*m_device, SafeSaneSamplerCreateInfo());
+
+    const VkDescriptorSetLayoutBinding binding = {0, VK_DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER, 1, VK_SHADER_STAGE_ALL,
+                                                  &sampler.handle()};
+    vkt::DescriptorSetLayout set_layout(*m_device, binding);
+    vkt::PipelineLayout pipeline_layout(*m_device, {&set_layout});
+
+    sampler.Destroy();
+
+    CreateComputePipelineHelper pipe(*this);
+    pipe.cp_ci_.layout = pipeline_layout;
+    m_errorMonitor->SetDesiredError("VUID-VkComputePipelineCreateInfo-layout-parameter");
+    pipe.CreateComputePipeline();
+    m_errorMonitor->VerifyFound();
+}
+
+TEST_F(NegativeObjectLifetime, ImmutableSamplerInUseDestroyed2) {
+    TEST_DESCRIPTION("Delete in-use immutable sampler.");
+    RETURN_IF_SKIP(Init());
+
+    vkt::Sampler sampler(*m_device, SafeSaneSamplerCreateInfo());
+
+    OneOffDescriptorSet descriptor_set(
+        m_device, {
+                      {0, VK_DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER, 1, VK_SHADER_STAGE_COMPUTE_BIT, &sampler.handle()},
+                  });
+
+    CreateComputePipelineHelper pipe(*this);
+    pipe.pipeline_layout_ = vkt::PipelineLayout(*m_device, {&descriptor_set.layout_});
+    pipe.CreateComputePipeline();
+
+    sampler.Destroy();
+
+    m_command_buffer.Begin();
+    m_errorMonitor->SetDesiredError("VUID-vkCmdBindPipeline-pipeline-parameter");
+    vk::CmdBindPipeline(m_command_buffer, VK_PIPELINE_BIND_POINT_COMPUTE, pipe);
+    m_errorMonitor->VerifyFound();
+    m_command_buffer.End();
+}
+
 TEST_F(NegativeObjectLifetime, CmdBufferEventDestroyed) {
     TEST_DESCRIPTION("Attempt to draw with a command buffer that is invalid due to an event dependency being destroyed.");
     RETURN_IF_SKIP(Init());
