@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 2023-2024 Valve Corporation
- * Copyright (c) 2023-2024 LunarG, Inc.
+ * Copyright (c) 2023-2025 Valve Corporation
+ * Copyright (c) 2023-2025 LunarG, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -287,4 +287,91 @@ TEST_F(NegativeShaderMesh, MeshAndTaskShaderDerivatives) {
     m_errorMonitor->SetDesiredError("VUID-RuntimeSpirv-meshAndTaskShaderDerivatives-10153");
     VkShaderObj ms(this, ms_source, VK_SHADER_STAGE_MESH_BIT_EXT, SPV_ENV_VULKAN_1_2, SPV_SOURCE_ASM);
     m_errorMonitor->VerifyFound();
-}
\ No newline at end of file
+}
+
+TEST_F(NegativeShaderMesh, MeshShaderPayloadMemoryOverLimit) {
+    TEST_DESCRIPTION("Validate Mesh shader shared memory limit");
+
+    SetTargetApiVersion(VK_API_VERSION_1_2);
+    AddRequiredExtensions(VK_EXT_MESH_SHADER_EXTENSION_NAME);
+    AddRequiredFeature(vkt::Feature::meshShader);
+
+    RETURN_IF_SKIP(Init());
+    InitRenderTarget();
+
+    VkPhysicalDeviceMeshShaderPropertiesEXT mesh_shader_properties = vku::InitStructHelper();
+    GetPhysicalDeviceProperties2(mesh_shader_properties);
+
+    const uint32_t max_mesh_payload_and_shared_memory_size = mesh_shader_properties.maxMeshPayloadAndSharedMemorySize;
+    const uint32_t max_mesh_payload_and_shared_ints = max_mesh_payload_and_shared_memory_size / 4;
+
+    std::stringstream mesh_source;
+    mesh_source << R"glsl(
+            #version 460
+            #extension GL_EXT_mesh_shader : require
+            layout(max_vertices = 3, max_primitives=1) out;
+            layout(triangles) out;
+            struct Task {
+                uint baseID[)glsl";
+    mesh_source << (max_mesh_payload_and_shared_ints / 2 + 1);
+    mesh_source << R"glsl(];
+            };
+            taskPayloadSharedEXT Task IN;
+            shared int a[)glsl";
+    mesh_source << (max_mesh_payload_and_shared_ints / 2 + 1);
+    mesh_source << R"glsl(];
+            void main(){}
+        )glsl";
+
+    VkShaderObj mesh(this, mesh_source.str().c_str(), VK_SHADER_STAGE_MESH_BIT_EXT, SPV_ENV_VULKAN_1_2);
+    const auto set_info = [&](CreatePipelineHelper &helper) { helper.shader_stages_ = {mesh.GetStageCreateInfo()}; };
+    CreatePipelineHelper::OneshotTest(*this, set_info, kErrorBit, "VUID-RuntimeSpirv-maxMeshPayloadAndSharedMemorySize-08755");
+}
+
+TEST_F(NegativeShaderMesh, MeshShaderPayloadSpecConstantSet) {
+    TEST_DESCRIPTION("Validate Mesh shader shared memory limit");
+
+    SetTargetApiVersion(VK_API_VERSION_1_2);
+    AddRequiredExtensions(VK_EXT_MESH_SHADER_EXTENSION_NAME);
+    AddRequiredFeature(vkt::Feature::meshShader);
+
+    RETURN_IF_SKIP(Init());
+    InitRenderTarget();
+
+    VkPhysicalDeviceMeshShaderPropertiesEXT mesh_shader_properties = vku::InitStructHelper();
+    GetPhysicalDeviceProperties2(mesh_shader_properties);
+
+    const uint32_t max_mesh_payload_and_shared_memory_size = mesh_shader_properties.maxMeshPayloadAndSharedMemorySize;
+    const uint32_t max_mesh_payload_and_shared_ints = max_mesh_payload_and_shared_memory_size / 4;
+
+    const char *mesh_source = R"glsl(
+            #version 460
+            #extension GL_EXT_mesh_shader : require
+            layout(max_vertices = 3, max_primitives=1) out;
+            layout(triangles) out;
+            layout(constant_id = 0) const int SIZE = 64;
+            struct Task {
+                uint baseID[SIZE];
+            };
+            taskPayloadSharedEXT Task IN;
+            shared int a[SIZE];
+            void main(){}
+        )glsl";
+
+    uint32_t size = max_mesh_payload_and_shared_ints / 2 + 1;
+
+    VkSpecializationMapEntry map_entry;
+    map_entry.constantID = 0u;
+    map_entry.offset = 0u;
+    map_entry.size = sizeof(uint32_t);
+
+    VkSpecializationInfo spec_info;
+    spec_info.mapEntryCount = 1u;
+    spec_info.pMapEntries = &map_entry;
+    spec_info.dataSize = sizeof(uint32_t);
+    spec_info.pData = &size;
+
+    VkShaderObj mesh(this, mesh_source, VK_SHADER_STAGE_MESH_BIT_EXT, SPV_ENV_VULKAN_1_2, SPV_SOURCE_GLSL, &spec_info);
+    const auto set_info = [&](CreatePipelineHelper &helper) { helper.shader_stages_ = {mesh.GetStageCreateInfo()};};
+    CreatePipelineHelper::OneshotTest(*this, set_info, kErrorBit, "VUID-RuntimeSpirv-maxMeshPayloadAndSharedMemorySize-08755");
+}
