@@ -424,6 +424,40 @@ TEST_F(NegativeVideo, CreateSessionInvalidStdHeaderVersion) {
     m_errorMonitor->VerifyFound();
 }
 
+TEST_F(NegativeVideo, CreateSessionUnsupportedCodecOp) {
+    TEST_DESCRIPTION("vkCreateVideoSessionKHR - unsupported video codec operation");
+
+    RETURN_IF_SKIP(Init());
+
+    VideoConfig config = GetConfig();
+    if (!config) {
+        GTEST_SKIP() << "Test requires video support";
+    }
+
+    uint32_t queue_family_index = VK_QUEUE_FAMILY_IGNORED;
+    for (uint32_t qfi = 0; qfi < QueueFamilyCount(); ++qfi) {
+        if ((QueueFamilyFlags(qfi) & (VK_QUEUE_VIDEO_DECODE_BIT_KHR | VK_QUEUE_VIDEO_ENCODE_BIT_KHR)) &&
+            ((QueueFamilyVideoCodecOps(qfi) & config.Profile()->videoCodecOperation) == 0)) {
+            queue_family_index = qfi;
+            break;
+        }
+    }
+
+    if (queue_family_index == VK_QUEUE_FAMILY_IGNORED) {
+        GTEST_SKIP() << "Test requires a queue family that supports video but not the specific codec op";
+    }
+
+    VkVideoSessionKHR session;
+    VkVideoSessionCreateInfoKHR create_info = *config.SessionCreateInfo();
+    create_info.pVideoProfile = config.Profile();
+    create_info.pStdHeaderVersion = config.StdVersion();
+    create_info.queueFamilyIndex = queue_family_index;
+
+    m_errorMonitor->SetDesiredError("VUID-VkVideoSessionCreateInfoKHR-pVideoProfile-11759");
+    vk::CreateVideoSessionKHR(device(), &create_info, nullptr, &session);
+    m_errorMonitor->VerifyFound();
+}
+
 TEST_F(NegativeVideo, BindVideoSessionMemory) {
     TEST_DESCRIPTION("vkBindVideoSessionMemoryKHR - memory binding related invalid usages");
 
@@ -690,7 +724,7 @@ TEST_F(NegativeVideo, BeginCodingUnsupportedCodecOp) {
 
     cb.Begin();
 
-    m_errorMonitor->SetDesiredError("VUID-vkCmdBeginVideoCodingKHR-commandBuffer-07231");
+    m_errorMonitor->SetDesiredError("VUID-vkCmdBeginVideoCodingKHR-commandBuffer-11760");
     cb.BeginVideoCoding(context.Begin());
     m_errorMonitor->VerifyFound();
 
@@ -1535,22 +1569,24 @@ TEST_F(NegativeVideo, CreateImageProfileIndependent) {
             vk::CreateImage(device(), &image_ci, nullptr, &image);
             m_errorMonitor->VerifyFound();
 
-            // Except when DECODE_DST usage is also requested
-            image_ci.usage |= VK_IMAGE_USAGE_VIDEO_DECODE_DST_BIT_KHR;
-            vk::CreateImage(device(), &image_ci, nullptr, &image);
-            vk::DestroyImage(device(), image, nullptr);
+            // Except when DECODE_DST usage is also requested, which is only legal when COINCIDE is supported.
+            if (config.SupportsDecodeOutputCoincide()) {
+                image_ci.usage |= VK_IMAGE_USAGE_VIDEO_DECODE_DST_BIT_KHR;
+                vk::CreateImage(device(), &image_ci, nullptr, &image);
+                vk::DestroyImage(device(), image, nullptr);
 
-            // Profile list is still validated though
-            VkVideoProfileListInfoKHR video_profiles = vku::InitStructHelper();
-            VkVideoProfileInfoKHR profiles[] = {*config.Profile(), *config.Profile()};
-            video_profiles.profileCount = 2;
-            video_profiles.pProfiles = profiles;
-            image_ci.pNext = &video_profiles;
+                // Profile list is still validated though
+                VkVideoProfileListInfoKHR video_profiles = vku::InitStructHelper();
+                VkVideoProfileInfoKHR profiles[] = {*config.Profile(), *config.Profile()};
+                video_profiles.profileCount = 2;
+                video_profiles.pProfiles = profiles;
+                image_ci.pNext = &video_profiles;
 
-            m_errorMonitor->SetAllowedFailureMsg("VUID-VkImageCreateInfo-pNext-06811");
-            m_errorMonitor->SetDesiredError("VUID-VkVideoProfileListInfoKHR-pProfiles-06813");
-            vk::CreateImage(device(), &image_ci, nullptr, &image);
-            m_errorMonitor->VerifyFound();
+                m_errorMonitor->SetAllowedFailureMsg("VUID-VkImageCreateInfo-pNext-06811");
+                m_errorMonitor->SetDesiredError("VUID-VkVideoProfileListInfoKHR-pProfiles-06813");
+                vk::CreateImage(device(), &image_ci, nullptr, &image);
+                m_errorMonitor->VerifyFound();
+            }
         }
 
         if (config.IsEncode()) {
@@ -1645,9 +1681,7 @@ TEST_F(NegativeVideo, CreateImageViewInvalidViewType) {
         image_view_ci.image = image;
         image_view_ci.viewType = VK_IMAGE_VIEW_TYPE_CUBE;
         image_view_ci.format = image_ci.format;
-        image_view_ci.subresourceRange.aspectMask = VK_IMAGE_ASPECT_COLOR_BIT;
-        image_view_ci.subresourceRange.levelCount = 1;
-        image_view_ci.subresourceRange.layerCount = 6;
+        image_view_ci.subresourceRange = {VK_IMAGE_ASPECT_COLOR_BIT, 0, 1, 0, 6};
 
         m_errorMonitor->SetAllowedFailureMsg("VUID-VkImageViewCreateInfo-image-01003");
         if (config.IsDecode()) {
@@ -1662,11 +1696,9 @@ TEST_F(NegativeVideo, CreateImageViewInvalidViewType) {
     }
 }
 
-TEST_F(NegativeVideo, CreateImageViewProfileIndependent) {
-    TEST_DESCRIPTION("vkCreateImageView - video usage not supported in view created from video profile independent image");
+TEST_F(NegativeVideo, CreateImageViewInvalidUsage) {
+    TEST_DESCRIPTION("vkCreateImageView - video usage not supported in view per the format feature requirements");
 
-    AddRequiredExtensions(VK_KHR_VIDEO_MAINTENANCE_1_EXTENSION_NAME);
-    AddRequiredFeature(vkt::Feature::videoMaintenance1);
     RETURN_IF_SKIP(Init());
 
     VideoConfig decode_config = GetConfig(GetConfigsWithReferences(GetConfigsDecode()));
@@ -1707,7 +1739,7 @@ TEST_F(NegativeVideo, CreateImageViewProfileIndependent) {
         }
 
         auto image_ci = vku::InitStruct<VkImageCreateInfo>();
-        image_ci.flags = VK_IMAGE_CREATE_VIDEO_PROFILE_INDEPENDENT_BIT_KHR | VK_IMAGE_CREATE_EXTENDED_USAGE_BIT;
+        image_ci.flags = VK_IMAGE_CREATE_EXTENDED_USAGE_BIT;
         image_ci.imageType = VK_IMAGE_TYPE_2D;
         image_ci.format = format;
         image_ci.extent = {1024, 1024, 1};
@@ -1727,9 +1759,7 @@ TEST_F(NegativeVideo, CreateImageViewProfileIndependent) {
         image_view_ci.image = image;
         image_view_ci.viewType = VK_IMAGE_VIEW_TYPE_2D;
         image_view_ci.format = format;
-        image_view_ci.subresourceRange.aspectMask = VK_IMAGE_ASPECT_COLOR_BIT;
-        image_view_ci.subresourceRange.levelCount = 1;
-        image_view_ci.subresourceRange.layerCount = 1;
+        image_view_ci.subresourceRange = {VK_IMAGE_ASPECT_COLOR_BIT, 0, 1, 0, 1};
 
         VkImageView image_view = VK_NULL_HANDLE;
 
@@ -1745,6 +1775,10 @@ TEST_F(NegativeVideo, BeginQueryIncompatibleQueueFamily) {
 
     RETURN_IF_SKIP(Init());
 
+    if (!HasQueueFamilySupportsResultStatusOnlyQueries()) {
+        GTEST_SKIP() << "Test requires at least one queue family to support result status queries";
+    }
+
     uint32_t queue_family_index = VK_QUEUE_FAMILY_IGNORED;
     for (uint32_t qfi = 0; qfi < QueueFamilyCount(); ++qfi) {
         if (!QueueFamilySupportsResultStatusOnlyQueries(qfi)) {
@@ -1977,6 +2011,10 @@ TEST_F(NegativeVideo, GetQueryPoolResultsStatusBit) {
 
     RETURN_IF_SKIP(Init());
 
+    if (!HasQueueFamilySupportsResultStatusOnlyQueries()) {
+        GTEST_SKIP() << "Test requires at least one queue family to support result status queries";
+    }
+
     if (!GetConfig()) {
         GTEST_SKIP() << "Test requires video support";
     }
@@ -1997,11 +2035,35 @@ TEST_F(NegativeVideo, GetQueryPoolResultsStatusBit) {
     m_errorMonitor->VerifyFound();
 }
 
+TEST_F(NegativeVideo, GetQueryPoolResultsStatusBitInvalidQueryType) {
+    TEST_DESCRIPTION("vkGetQueryPoolResults - test invalid combination of QueryResultFlags and QueryType");
+
+    RETURN_IF_SKIP(Init());
+
+    if (!GetConfig()) {
+        GTEST_SKIP() << "Test requires video support";
+    }
+
+    vkt::QueryPool query_pool(*m_device, VK_QUERY_TYPE_OCCLUSION, 1);
+
+    uint32_t status;
+    VkQueryResultFlags flags;
+
+    m_errorMonitor->SetDesiredError("VUID-vkGetQueryPoolResults-queryType-11874");
+    flags = VK_QUERY_RESULT_WITH_STATUS_BIT_KHR;
+    vk::GetQueryPoolResults(device(), query_pool, 0, 1, sizeof(status), &status, sizeof(status), flags);
+    m_errorMonitor->VerifyFound();
+}
+
 TEST_F(NegativeVideo, CopyQueryPoolResultsStatusBit) {
     TEST_DESCRIPTION("vkCmdCopyQueryPoolResults - test invalid use of VK_QUERY_RESULT_WITH_STATUS_BIT_KHR");
 
     RETURN_IF_SKIP(Init());
 
+    if (!HasQueueFamilySupportsResultStatusOnlyQueries()) {
+        GTEST_SKIP() << "Test requires at least one queue family to support result status queries";
+    }
+
     if (!GetConfig()) {
         GTEST_SKIP() << "Test requires video support";
     }
@@ -2027,6 +2089,31 @@ TEST_F(NegativeVideo, CopyQueryPoolResultsStatusBit) {
     m_command_buffer.End();
 }
 
+TEST_F(NegativeVideo, CopyQueryPoolResultsStatusBitInvalidQueryType) {
+    TEST_DESCRIPTION("vkCmdCopyQueryPoolResults - test invalid combination of QueryResultFlags and QueryType");
+
+    RETURN_IF_SKIP(Init());
+
+    if (!GetConfig()) {
+        GTEST_SKIP() << "Test requires video support";
+    }
+
+    vkt::QueryPool query_pool(*m_device, VK_QUERY_TYPE_OCCLUSION, 1);
+
+    VkQueryResultFlags flags;
+
+    vkt::Buffer buffer(*m_device, sizeof(uint32_t), VK_BUFFER_USAGE_TRANSFER_DST_BIT);
+
+    m_command_buffer.Begin();
+
+    m_errorMonitor->SetDesiredError("VUID-vkCmdCopyQueryPoolResults-queryType-11874");
+    flags = VK_QUERY_RESULT_WITH_STATUS_BIT_KHR;
+    vk::CmdCopyQueryPoolResults(m_command_buffer, query_pool, 0, 1, buffer, 0, sizeof(uint32_t), flags);
+    m_errorMonitor->VerifyFound();
+
+    m_command_buffer.End();
+}
+
 TEST_F(NegativeVideoBestPractices, GetVideoSessionMemoryRequirements) {
     TEST_DESCRIPTION("vkGetVideoSessionMemoryRequirementsKHR - best practices");
 
@@ -2103,3 +2190,35 @@ TEST_F(NegativeVideoBestPractices, BindVideoSessionMemory) {
         }
     }
 }
+
+TEST_F(NegativeVideo, NoQueueSupportForResultStatusOnly) {
+    TEST_DESCRIPTION("vkCreateQueryPool - test querypool creation when result status only query is not supported");
+
+    RETURN_IF_SKIP(Init());
+
+    VideoConfig config = GetConfig();
+    if (!config) {
+        GTEST_SKIP() << "Test requires video support";
+    }
+
+    bool has_queue_with_result_status_only_support = false;
+    for (uint32_t qfi = 0; qfi < QueueFamilyCount(); ++qfi) {
+        if (QueueFamilySupportsResultStatusOnlyQueries(qfi)) {
+            has_queue_with_result_status_only_support = true;
+            break;
+        }
+    }
+
+    if (has_queue_with_result_status_only_support) {
+        GTEST_SKIP() << "Test requires there is not queue family that supports result status only";
+    }
+
+    VkQueryPoolCreateInfo query_pool_create_info = vku::InitStructHelper();
+    query_pool_create_info.queryType = VK_QUERY_TYPE_RESULT_STATUS_ONLY_KHR;
+    query_pool_create_info.queryCount = 1;
+
+    m_errorMonitor->SetDesiredError("VUID-VkQueryPoolCreateInfo-queryType-11839");
+    VkQueryPool query_pool;
+    vk::CreateQueryPool(device(), &query_pool_create_info, nullptr, &query_pool);
+    m_errorMonitor->VerifyFound();
+}
